#include	"plot.h"
#include	<stdio.h>
#include	<math.h>

/*
Draw a mark. Mark numbers are
	1	open diamond
	2	closed diamond
	3	open square
	4	closed square
	5	open triangle
	6	closed triangle
	7	open flipped triangle
	8	closed flipped triangle
	9	open circle
	10	close circle
*/

pfsymble_( xc, yc, size, mark )
float	*xc;	/* center X coordinate */
float	*yc;	/* center Y coordinate */
float	*size;	/* Size of the mark */
int	*mark;	/* Mark type */
{
  int		device_x, backstore, device_ps;
  int		xp, yp;
  float		dwx, dwy;
  float		x0, y0;
  float		wx, wy;
  int		w, wh, wq;
  int		wp;
  int		x1, y1, x2, y2, x3, y3, x4, y4;
  XPoint	p[5];
  int		npoints;
  int		ndiv, idiv;
  double	dang, ang;
  int		dx, dy, x, y;
  FILE		*fp;
  FILE		*plsav_fp();

/* Check the device: Only X or PS are accepted. */

  device_x = FALSE;
  backstore = FALSE;
  device_ps = FALSE;

  if( DEVICE_N == X || DEVICE_N == XM 
		|| DEVICE_N == XN || DEVICE_N == XMN ) {
    device_x = TRUE;
    if( DEVICE_N == X || DEVICE_N == XM ) backstore = TRUE;
  } else if( DEVICE_N == POSTSCRIPT || DEVICE_N == POSTSCRIPTM ) {
    device_ps = TRUE;
  }

/* Convert the scale and check the window */

  if( plscal( 0, *xc, *yc, *xc, *yc, &xp, &yp, &xp, &yp ) ) {
    plcvdv( 0.0, 0.0, &x0, &y0 );
    plcvdv( *size, *size, &wx, &wy );
    dwx = ( wx > x0 ) ? wx - x0 : x0 - wx;
    dwy = ( wy > y0 ) ? wy - y0 : y0 - wy;
    w  = ( dwx < dwy ) ? dwx : dwy;
    wh = 0.5 * w;
    wq = 0.25 * w;

/* Calculate corner coordinates */

    if( *mark == 1 || *mark == 2 ) {			/* diamonds */
      wp = 0.707107 * w;
      x1 = xp; y1 = yp + wp; 
      x2 = xp - wp; y2 = yp;
      x3 = xp, y3 = yp - wp; 
      x4 = xp + wp; y4 = yp;
      if( device_x ) {
	p[0].x = x1;
	p[0].y = XWINDOW_YSIZE - y1 + 1;
	p[1].x = x2;
	p[1].y = XWINDOW_YSIZE - y2 + 1;
	p[2].x = x3;
	p[2].y = XWINDOW_YSIZE - y3 + 1;
	p[3].x = x4;
	p[3].y = XWINDOW_YSIZE - y4 + 1;
	p[4].x = x1;
	p[4].y = XWINDOW_YSIZE - y1 + 1;
	npoints = 5;
      }
    } else if( *mark == 5 || *mark == 6 ) {		/* triangles */
      x1 = xp - wh; y1 = yp + wq;
      x2 = xp; y2 = yp - 0.616026 * w;
      x3 = xp + wh; y3 = y1;
      if( device_x ) {
	p[0].x = x1;
	p[0].y = XWINDOW_YSIZE - y1 + 1;
	p[1].x = x2;
	p[1].y = XWINDOW_YSIZE - y2 + 1;
	p[2].x = x3;
	p[2].y = XWINDOW_YSIZE - y3 + 1;
	p[3].x = x1;
	p[3].y = XWINDOW_YSIZE - y1 + 1;
	npoints = 4;
      }
    } else if( *mark == 7 || *mark == 8 ) {		/* flipped triangles */
      x1 = xp; y1 = yp + 0.616026 * w;
      x2 = xp - wh; y2 = yp - wq;
      x3 = xp + wh; y3 = y2;
      if( device_x ) {
	p[0].x = x1;
	p[0].y = XWINDOW_YSIZE - y1 + 1;
	p[1].x = x2;
	p[1].y = XWINDOW_YSIZE - y2 + 1;
	p[2].x = x3;
	p[2].y = XWINDOW_YSIZE - y3 + 1;
	p[3].x = x1;
	p[3].y = XWINDOW_YSIZE - y1 + 1;
	npoints = 4;
      }
    }

/* Dispatch according to the device number. */

    if( device_x ) {
/* X windows */
      yp = XWINDOW_YSIZE - yp + 1;

      switch( *mark ) {

      case 1:	/* Open diamond */
	XDrawLines( XDISPLAY, XWINDOW, XGC, p, npoints, CoordModeOrigin );
	if( backstore ) {
	  XDrawLines( XDISPLAY, XPIXMAP, PGC, p, npoints, CoordModeOrigin );
	} break;

      case 2:	/* Closed diamond */
	npoints = npoints - 1;
	XFillPolygon( XDISPLAY, XWINDOW, XGC, p, npoints, 
						Convex, CoordModeOrigin );
	if( backstore ) {
	  XFillPolygon( XDISPLAY, XPIXMAP, PGC, p, npoints, 
						Convex, CoordModeOrigin );
	} break;

      case 3:	/* Open square */
	xp = xp - wh;
	yp = yp - wh;
	XDrawRectangle( XDISPLAY, XWINDOW, XGC, xp, yp, w, w );
	if( backstore ) {
	  XDrawRectangle( XDISPLAY, XPIXMAP, PGC, xp, yp, w, w );
	} break;

      case 4:	/* Closed square */
	xp = xp - wh;
	yp = yp - wh;
	XFillRectangle( XDISPLAY, XWINDOW, XGC, xp, yp, w, w );
	if( backstore ) {
	  XFillRectangle( XDISPLAY, XPIXMAP, PGC, xp, yp, w, w );
	} break;

      case 5:	/* Open triangle */
	XDrawLines( XDISPLAY, XWINDOW, XGC, p, npoints, CoordModeOrigin );
	if( backstore ) {
	  XDrawLines( XDISPLAY, XPIXMAP, PGC, p, npoints, CoordModeOrigin );
	} break;

      case 6:	/* Closed triangle */
	npoints = npoints - 1;
	XFillPolygon( XDISPLAY, XWINDOW, XGC, p, npoints, 
						Convex, CoordModeOrigin );
	if( backstore ) {
	  XFillPolygon( XDISPLAY, XPIXMAP, PGC, p, npoints, 
						Convex, CoordModeOrigin );
	} break;

      case 7:	/* Open flipped triangle */
	XDrawLines( XDISPLAY, XWINDOW, XGC, p, npoints, CoordModeOrigin );
	if( backstore ) {
	  XDrawLines( XDISPLAY, XPIXMAP, PGC, p, npoints, CoordModeOrigin );
	} break;

      case 8:	/* Closed flipped triangle */
	npoints = npoints - 1;
	XFillPolygon( XDISPLAY, XWINDOW, XGC, p, npoints, 
						Convex, CoordModeOrigin );
	if( backstore ) {
	  XFillPolygon( XDISPLAY, XPIXMAP, PGC, p, npoints, 
						Convex, CoordModeOrigin );
	} break;

      case 9:	/* Open circle */
	xp = xp - wh;
	yp = yp - wh;
	XDrawArc( XDISPLAY, XWINDOW, XGC, xp, yp, w, w, 0, 23040 );
	if( backstore ) {
	  XDrawArc( XDISPLAY, XPIXMAP, PGC, xp, yp, w, w, 0, 23040 );
	} break;

      case 10:	/* Closed circle */
	xp = xp - wh;
	yp = yp - wh;
	XFillArc( XDISPLAY, XWINDOW, XGC, xp, yp, w, w, 0, 23040 );
	if( backstore ) {
	  XFillArc( XDISPLAY, XPIXMAP, PGC, xp, yp, w, w, 0, 23040 );
	} break;
      }

    } else if( device_ps ) {
/* PostScript */

      fp = plsav_fp( RESTORE, 0 );

      switch( *mark ) {

      case 1:	/* Open diamond */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", x1, y1 );
	fprintf( fp, "%d %d l\n", x2, y2 );
	fprintf( fp, "%d %d l\n", x3, y3 );
	fprintf( fp, "%d %d l\n", x4, y4 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "stroke\n" );
	break;

      case 2:	/* Closed diamond */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", x1, y1 );
	fprintf( fp, "%d %d l\n", x2, y2 );
	fprintf( fp, "%d %d l\n", x3, y3 );
	fprintf( fp, "%d %d l\n", x4, y4 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "fill\n" );
	break;

      case 3:	/* Open square */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", xp - wh, yp + wh );
	fprintf( fp, "%d %d l\n", xp - wh, yp - wh );
	fprintf( fp, "%d %d l\n", xp + wh, yp - wh );
	fprintf( fp, "%d %d l\n", xp + wh, yp + wh );
	fprintf( fp, "%d %d l\n", xp - wh, yp + wh );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "stroke\n" );
	break;

      case 4:	/* Closed square */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", xp - wh, yp + wh );
	fprintf( fp, "%d %d l\n", xp - wh, yp - wh );
	fprintf( fp, "%d %d l\n", xp + wh, yp - wh );
	fprintf( fp, "%d %d l\n", xp + wh, yp + wh );
	fprintf( fp, "%d %d l\n", xp - wh, yp + wh );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "fill\n" );
	break;

      case 5:	/* Open triangle */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", x1, y1 );
	fprintf( fp, "%d %d l\n", x2, y2 );
	fprintf( fp, "%d %d l\n", x3, y3 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "stroke\n" );
	break;

      case 6:	/* Closed triangle */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", x1, y1 );
	fprintf( fp, "%d %d l\n", x2, y2 );
	fprintf( fp, "%d %d l\n", x3, y3 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "fill\n" );
	break;

      case 7:	/* Open flipped triangle */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", x1, y1 );
	fprintf( fp, "%d %d l\n", x2, y2 );
	fprintf( fp, "%d %d l\n", x3, y3 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "stroke\n" );
	break;

      case 8:	/* Closed flipped triangle */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d m\n", x1, y1 );
	fprintf( fp, "%d %d l\n", x2, y2 );
	fprintf( fp, "%d %d l\n", x3, y3 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "fill\n" );
	break;

      case 9:	/* Open circle */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d %d %d %d arc\n", xp, yp, wh, 0, 360 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "stroke\n" );
	break;

      case 10:	/* Closed circle */
	fprintf( fp, "stroke\n" );
	fprintf( fp, "newpath\n" );
	fprintf( fp, "%d %d %d %d %d arc\n", xp, yp, wh, 0, 360 );
	fprintf( fp, "closepath\n" );
	fprintf( fp, "fill\n" );
	break;

      }
    } else {	/* Non X, non PS device */

      switch( *mark ) {

      case 1:	/* Open diamond */
	plvect( 0, x1, y1, 0 );
	plvect( 0, x2, y2, 1 );
	plvect( 0, x2, y2, 0 );
	plvect( 0, x3, y3, 1 );
	plvect( 0, x3, y3, 0 );
	plvect( 0, x4, y4, 1 );
	plvect( 0, x4, y4, 0 );
	plvect( 0, x1, y1, 1 );
	break;

      case 2:	/* Closed diamond --> draw open diamond */
	plvect( 0, x1, y1, 0 );
	plvect( 0, x2, y2, 1 );
	plvect( 0, x2, y2, 0 );
	plvect( 0, x3, y3, 1 );
	plvect( 0, x3, y3, 0 );
	plvect( 0, x4, y4, 1 );
	plvect( 0, x4, y4, 0 );
	plvect( 0, x1, y1, 1 );
	break;

      case 3:	/* Open square */
	plvect( 0, xp - wh, yp + wh, 0 );
	plvect( 0, xp - wh, yp - wh, 1 );
	plvect( 0, xp - wh, yp - wh, 0 );
	plvect( 0, xp + wh, yp - wh, 1 );
	plvect( 0, xp + wh, yp - wh, 0 );
	plvect( 0, xp + wh, yp + wh, 1 );
	plvect( 0, xp + wh, yp + wh, 0 );
	plvect( 0, xp - wh, yp + wh, 1 );
	break;

      case 4:	/* Closed square --> draw open square */
	plvect( 0, xp - wh, yp + wh, 0 );
	plvect( 0, xp - wh, yp - wh, 1 );
	plvect( 0, xp - wh, yp - wh, 0 );
	plvect( 0, xp + wh, yp - wh, 1 );
	plvect( 0, xp + wh, yp - wh, 0 );
	plvect( 0, xp + wh, yp + wh, 1 );
	plvect( 0, xp + wh, yp + wh, 0 );
	plvect( 0, xp - wh, yp + wh, 1 );
	break;

      case 5:	/* Open triangle */
	plvect( 0, x1, y1, 0 );
	plvect( 0, x2, y2, 1 );
	plvect( 0, x2, y2, 0 );
	plvect( 0, x3, y3, 1 );
	plvect( 0, x3, y3, 0 );
	plvect( 0, x1, y1, 1 );
	break;

      case 6:	/* Closed triangle --> draw open triangle */
	plvect( 0, x1, y1, 0 );
	plvect( 0, x2, y2, 1 );
	plvect( 0, x2, y2, 0 );
	plvect( 0, x3, y3, 1 );
	plvect( 0, x3, y3, 0 );
	plvect( 0, x1, y1, 1 );
	break;

      case 7:	/* Open flipped triangle */
	plvect( 0, x1, y1, 0 );
	plvect( 0, x2, y2, 1 );
	plvect( 0, x2, y2, 0 );
	plvect( 0, x3, y3, 1 );
	plvect( 0, x3, y3, 0 );
	plvect( 0, x1, y1, 1 );
	break;

      case 8:	/* Closed flipped triangle --> draw open flipped triangle */
	plvect( 0, x1, y1, 0 );
	plvect( 0, x2, y2, 1 );
	plvect( 0, x2, y2, 0 );
	plvect( 0, x3, y3, 1 );
	plvect( 0, x3, y3, 0 );
	plvect( 0, x1, y1, 1 );
	break;

      case 9:	/* Open circle */
	plvect( 0, xp + wh, yp, 0 );
	ndiv = sqrt( (float)wh + 60.0 );
	dang = 6.28319 / ndiv;
	for( idiv = 1; idiv <= ndiv; idiv++ ) {
	  ang = dang * idiv;
	  dx = (float)wh * cos( ang );
	  dy = (float)wh * sin( ang );
	  x = xp + dx;
	  y = yp + dy;
	  plvect( 0, x, y, 1 );
	  plvect( 0, x, y, 0 );
	}
	break;

      case 10:	/* Closed circle --> draw open circle */
	plvect( 0, xp + wh, yp, 0 );
	ndiv = sqrt( (float)wh + 60.0 );
	dang = 6.28319 / ndiv;
	for( idiv = 1; idiv <= ndiv; idiv++ ) {
	  ang = dang * idiv;
	  dx = (float)wh * cos( ang );
	  dy = (float)wh * sin( ang );
	  x = xp + dx;
	  y = yp + dy;
	  plvect( 0, x, y, 1 );
	  plvect( 0, x, y, 0 );
	}
	break;
      }
    }
  }
}
