C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
C THIS IS A COLLECTION OF SUBROUTINES -WRITTEN IN FORTRAN-
C THAT WILL SMOOTH A GIVEN SEQUENCE OF DATA POINTS.
C THE ALGORITHM IS BY J W TUKEY AND ALBERTO TUBILLIO, AND THE
C CODING IS MOSTLY BY ROGER CHAFFEE.
C TO USE IT MAKE THE FOLLOWING CALL :
C
C             CALL SMCTRL (SMY,Y,N,LEVEL,F,E)
C WHERE
C SMY,Y,F,E ARE ONE-DIMENSIONAL ARRAYS OF BINS N AND
C LEVEL IS A SMOOTHING LEVEL, FROM 0 TO 5.
C Y SHOULD CONTAIN THE SEQUENCE TO BE SMOOTHED,
C AT THE COMPLETION OF THE CALL  Y IS LEFT UNMODIFIED AND
C SMY GIVES THE SMOOTHED SEQUENCE.
C F AND E ARE DUMMY ARRAYS NEEDED IN THE COMPUTATION.
C SMOOTH CALLS IN TURN THE SUBROUTINES:
C              SMRMED
C              SMRMEN
C              SMQHAN
C              SMSORT
C THESE FORM THE REST OF THE COLLECTION.
C        ***********NOTES***************
C Y IS ASSUMED TO BE THE SEQUENCE OF OBSERVED VALUES OF A
C FUNCTION  AT EQUALLY SPACED INTERVALS.
C THE SMOOTHER PROCEDURE IS A NON-LINEAR ONE WHICH SO FAR
C HAS PROVED TO BE THE BEST AMONG SEVERAL ONES CURRENTLY
C STUDIED  AT SLAC.
C AS OPPOSED TO LINEAR PROCEDURES ITS PERFORMANCE SHOULDN'T
C BE IMPAIRED IF THE DATA IS NOT WELL BEHAVED.
      SUBROUTINE SMCTRL (G,D,K,LEVEL,F,E)
      DIMENSION G(K),D(K),F(K),E(K)
      DIMENSION N1LEV(6),N2LEV(6)
      DATA N1LEV(1),N1LEV(2),N1LEV(3),N1LEV(4),
     X   N1LEV(5),N1LEV(6) /1,3,5,7,9,11/
      DATA N2LEV(1),N2LEV(2),N2LEV(3),N2LEV(4),
     X   N2LEV(5),N2LEV(6) /1,1,3,3,5,5/
C
      IF (K.LE.0) RETURN
      L = MAX0(0,MIN0(5,LEVEL)) + 2
    1 L = L-1
      N1 = N1LEV(L)
      N2 = N2LEV(L)
      IF (N1.GT.K) GO TO 1
      CALL SMRMED (G,D,K,N1)
         IF (K.LT.3) RETURN
C                 ENDPOINTS ARE SPECIAL
C                 USE G(1)=MEDIAN(D(1),G(2),3G(2)-2G(3))
         E(1)=D(1)
         E(2)=G(2)
         E(3)=3.*G(2)-2.*G(3)
         CALL SMSORT(E,3)
         G(1)=E(2)
         E(1)=D(K)
         E(2)=G(K-1)
         E(3)=3.*G(K-1)-2.*G(K-2)
         CALL SMSORT(E,3)
         G(K)=E(2)
      CALL SMRMED (F,G,K,N2)
      CALL SMQHAN(G,F,K)
      DO 2 KR=1,K
      F(KR)=D(KR)-G(KR)
    2 E(KR) = F(KR)
C                 G HAS SMOOTH. E AND F HAVE RESIDUALS
C                            GET 25-PERCENTILE POINTS
      CALL SMSORT(E,K)
      KM = MAX0(1,K/4)
      DQU = E(KM)
      I=K+1-KM
      UQU = E(I)
      STEP = AMAX1(0.2,1.5*(UQU-DQU))
      STEP2=STEP+STEP
C              CHANGE LARGE RESIDUALS
      DO 5 I=1,K
         IF (ABS(F(I)).GT.STEP) F(I)=SIGN(DIM(STEP2,ABS(F(I))),F(I))
    5 CONTINUE
C              SMOOTH THE CHANGED RESIDUALS
      CALL SMRMEN (E,F,K,N1)
      CALL SMRMED (F,E,K,N2)
      CALL SMQHAN  (E,F,K)
C              ADD THEM BACK TO THE FIRST SMOOTHED VALUES
      DO 6 KR=1,K
    6 G(KR)=G(KR)+E(KR)
C                 END OF SMOOTH.  DO ANOTHER, W/O CHANGING RESIDUALS
      CALL SMRMEN (E,G,K,N1)
      CALL SMRMED (F,E,K,N2)
      CALL SMQHAN (E,F,K)
      DO 7 KR=1,K
    7 F(KR)=G(KR)-E(KR)
      CALL SMRMEN (G,F,K,N1)
      CALL SMRMED (F,G,K,N2)
      CALL SMQHAN (G,F,K)
      DO 8 KR=1,K
    8 G(KR)=E(KR)+G(KR)
      RETURN
      END
cnext ****************************************
      SUBROUTINE SMQHAN (G,F,K)
C  QUADRATIC INTERPOLATION AND HANNING
      DIMENSION G(K),F(K)
      IF (K.LE.4) GO TO 10
      G(1)=F(1)
      G(2)=(F(1)+F(2)+F(2)+F(3))*0.25
      G(3)=(F(2)+F(3)+F(3)+F(4))*0.25
      K1=K-1
      DO 2 I=4,K1
      IF (F(I-2).NE.F(I-1)) GOTO 1
      IF (F(I-1).NE.F(I))   GOTO 1
      IF ((F(I-2).GE.F(I-3)).AND.(F(I+1).GE.F(I))) GOTO 1
      IF ((F(I-2).LE.F(I-3)).AND.(F(I+1).LE.F(I))) GOTO 1
C          SPECIAL CASE.  F(I-3)<=F(I-2)=F(I-1)=F(I)<=F(I+1)  (OR >=)
C          RECALCULATE SMOOTHED VALUES USING CUBIC FIT
      G(I-2)=F(I-2)
      G(I-1)=(4.0*(F(I-2)+F(I))-F(I-3)-F(I+1))/6.0
      G(I)=F(I)
      GOTO 2
C          NORMAL CASE (ALL OTHERS)  CALCULATE USING QUADRATIC FIT
    1 G(I)=(F(I-1)+F(I)+F(I)+F(I+1))*0.25
    2 CONTINUE
      G(K)=F(K)
      RETURN
   10 DO 11 I=1,K
   11 G(I) = F(I)
      RETURN
      END
cnext ****************************************
      SUBROUTINE SMSORT(A,N)
C        PUTS A IN ASCENDING ORDER
      INTEGER J,M,N,IDONE,IPASS
      REAL A(N),T
      IF (N.LE.1) GO TO 3
         M = N
         DO 2 IPASS=2,N
            M = M - 1
            IDONE = 1
            DO 1 J=1,M
               IF (A(J) .LE. A(J+1)) GO TO 1
                  IDONE = 0
                  T = A(J)
                  A(J) = A(J+1)
                  A(J+1) = T
 1          CONTINUE
            IF (IDONE.NE.0) GO TO 3
 2       CONTINUE
 3    CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE SMRMED (G,F,K,L)
C     THIS SUBROUTINE CALCULATES RUNNING MEDIANS OF LENGTH LEN (LE 11)
C     FROM THE ARRAY F AND PUTS THEM IN G.  F AND G MAY NOT
C     OCCUPY THE SAME CORE LOCATIONS.
C       A GENERAL ROUTINE WOULD REQUIRE TESTS THAT 0.LT.L.LE.K
C       AND L.LT.(DIMENSION OF B)  AND L=ODD.
C       THIS IS ASSURED IN SMCTRL, AND NOT TESTED HERE.
      DIMENSION   B(12),G(K),F(K)
      M=L/2
      MP1 = M+1
      MP2 = MP1+1
C                            FIRST M+1 VALUES
      DO 10 I=1,L
   10 B(I)=F(I)
      KM = -1
      DO 20 I=1,MP1
         KM=KM+2
         CALL SMSORT(B,KM)
         G(I) = B(I)
   20 CONTINUE
C                            VALUES M+2....K-M
      K1=K-L
      IF (K1.EQ.0) GO TO 65
      DO 60 KR=1,K1
         TEMP = F(KR)
         DO 40 J=1,L
            IF (B(J).NE.TEMP) GO TO 40
               I=J
               GO TO 50
   40    CONTINUE
C                            IMPOSSIBLE SITUATION........
         STOP 472
C              B(I) IS F(KR).  MOVE B(1) TO B(I-1) INTO B(2) TO B(I)
C              (AND LEAVE B(1) VACANT)
   50    IF (I.EQ.1) GO TO 54
   53       B(I)=B(I-1)
            I=I-1
            IF (I.NE.1) GO TO 53
C              NOW PUT F(KR+L) INTO THE LIST WHERE IT BELONGS
   54    J=0
C              PUT A VALUE AT THE END OF THE LIST, TO MAKE SURE
C              IT DOESNT RUN OVER.
         KRX=KR+L
         B(L+1) = F(KRX)
   55    J=J+1
         B(J) = B(J+1)
         IF (B(J).LT.B(L+1)) GO TO 55
         B(J)=B(L+1)
         KRX=KR+MP1
         G(KRX)=B(MP1)
   60 CONTINUE
C                            LAST M+1 VALUES
   65 DO 70 I=1,L
         KM=K+1-I
         B(I)=F(KM)
   70 CONTINUE
      I = 1
      J = K
      DO 80 KM = 1,L,2
         CALL SMSORT(B,KM)
         G(J) = B(I)
         J = J - 1
         I = I + 1
   80 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE SMRMEN (G,F,N,L)
C     THIS SUBROUTINE CALCULATES RUNNING MEANS OF LENGTH L FROM
C     THE ARRAY F AND PUTS THE RESULT IN G,NO CALLS ARE MADE
C       A GENERAL ROUTINE WOULD REQUIRE TESTS THAT 0.LT.L.LE.N
C       AND L=ODD.
C       THIS IS ASSURED IN SMCTRL, AND NOT TESTED HERE.
      DIMENSION F(N),G(N)
      M=L/2
C     THE FIRST AND LAST M POSITIONS OF G ARE FILLED WITH THE
C     MEANS OF THE FIRST (LAST) 1,3,...L-2 ELEMENTS OF Y
      G(1) = F(1)
      TEMP1 = F(1)
      G(N) = F(N)
      TEMP2 = F(N)
      K = 1
      IF (M.EQ.0) GO TO 5
      DO 2 I=1,M
         III = N-K
         TEMP2 = TEMP2 + F(III) + F(III-1)
         K = K + 2
         TEMP1 = TEMP1 + F(K-1) + F(K)
         RECIP = 1./FLOAT(K)
         G(I+1) = TEMP1*RECIP
         III = N-I
         G(III) = TEMP2*RECIP
    2 CONTINUE
C
    5 NMID = N-L-1
      IF (NMID.LE.0) GO TO 20
      DO 10 I=1,NMID
         III = I+K
         TEMP1 = TEMP1 + F(III) - F(I)
         III=I+M
         G(III+1) = TEMP1*RECIP
   10 CONTINUE
   20 RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
C                       N O T I C E
C \
C This  program and  associated  documentation,  and  the
C material and  data contained  therein,  were  developed
C under the sponsorship of the U. S. government.  Neither
C the U.S.  nor the U.S.D.O.E.,   nor the Leland Stanford
C Junior  University,  nor  their  employees,  nor  their
C respective  contractors,   subcontractors,    or  their
C employees, even unto the seventh generation,  makes any
C warranty, express or implied,  or assumes any liability
C or  responsibility   for  accuracy,    completeness  or
C usefulness of any information,   apparatus,  product or
C process disclosed,  or represents that its use will not
C infringe  privately-owned  rights.     Mention  of  any
C product, its manufacturer, or suppliers shall not,  nor
C is it  intended to,  imply approval,   disapproval,  or
C fitness for any particular use.   A royalty-free,  non-
C exclusive right  to use  and disseminate  same for  any
C purpose whatsoever  is expressly  reserved to  the U.S.
C and the University.
C
cnext ****************************************
      SUBROUTINE TDEND
      CALL TXEND
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDCASE(TEXT,CASE, X , Y )
      COMMON /TDFCOM/ITITLE,TSETSZ,TSETOR,TITSPC,IDATIT
cmt4       BYTE TEXT(256),CASE(256)
      character*1 TEXT(256),CASE(256)
      REAL X,Y
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      INTEGER T2INTR
      REAL T2REAL
      EXTERNAL T2INTR,T2REAL
cmt4       BYTE SEMIC,LBUFFR(400)
      character*1 SEMIC,LBUFFR(400)
      INTEGER INFOIN(9)
      REAL XY(2),XYZ(3)
      EQUIVALENCE (BUFFER(1),LBUFFR(1))
      DATA SEMIC /    1H;/
      DATA NERR/0/
cccc      NARG = T1NARG(4)
cccc      IF((NARG .GE. 2) .AND. (NARG .LE. 4))GOTO 10021
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 4		! Y.Teramoto
10021 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10041
      WRITE(DBGFIL,10050)
10050 FORMAT(30X,    6HTDCASE)
10041 CONTINUE
      IERFLG=0
      NCHARS=0
      I=0
      NTEXT = 0
10061 CONTINUE
      NTEXT=NTEXT+1
      NCHARS=NCHARS+1
      LBUFFR(NCHARS)=TEXT(NTEXT)
      IF(TEXT(NTEXT) .NE. SEMIC)GOTO 10081
      IF(I .NE. 0)GOTO 10101
      NCHARS=NCHARS-1
      IF(TEXT(NTEXT+1).NE.SEMIC)GOTO 10062
      I=1
      GOTO 10111
10101 CONTINUE
      I=0
10111 CONTINUE
10091 CONTINUE
10081 CONTINUE
      IF(NTEXT .LE. 256)GOTO 10131
      WRITE(6,10140)
10140 FORMAT(   49H *** ERROR *** TEXT HAS NO TERMINATING SEMICOLON.  ,
     *  14H CALL IGNORED.)
      CALL ERRTRA
      IERFLG = 1
      GOTO 10062
10131 CONTINUE
      GOTO 10061
10062 CONTINUE
      NCASE=257
      DO 10151 I=1,NCHARS
      LBUFFR(NCASE)=CASE(I)
      IF(LBUFFR(NCASE).EQ.SEMIC)GOTO 10152
      NCASE=NCASE+1
10151 CONTINUE
10152 CONTINUE
      NCASE = NCASE - 257
      IF (.NOT.FLAGS(73)) CALL TXDEF1
      IF (TSETSZ.EQ.0.) TSETSZ=TSIZDF
      IF (TITSPC.EQ.0.) TITSPC = FLOAT(NCHARS)
      IF(NARG .NE. 3)GOTO 10171
      INFOIN(1)=1
      DO 10181 I=2,9
      INFOIN(I)=0
10181 CONTINUE
10182 CONTINUE
      IF (.NOT.FLAGS(74)) CALL TXDEFW
      GO TO 10191
10220 CONTINUE
      IF (IABS(ITITLE)-3) 10230,10240,10250
10230 CONTINUE
      TSETSZ=TSIZDF
10240 CONTINUE
      TSETOR =0.
10250 CONTINUE
      SPACE=ABS(0.1*TSETSZ)
      YPOS = WINDOW(2) - SPACE - 0.4
      XPOS = 0.5*(WINDOW(1)+WINDOW(3)-SPACE*(TITSPC-1.))
      GO TO 10192
10260 CONTINUE
      IF (IABS(ITITLE)-3) 10270,10280,10290
10270 CONTINUE
      TSETSZ=TSIZDF
10280 CONTINUE
      TSETOR =90.
10290 CONTINUE
      SPACE=ABS(0.1*TSETSZ)
      IF(LBLCHR(2) .GT. 0)GOTO 10311
      I=5
      GOTO 10321
10311 CONTINUE
      I=LBLCHR(2)
10321 CONTINUE
10301 CONTINUE
      XPOS = WINDOW(1)-1.5*SPACE-ABS(LBLSIZ)*0.1*FLOAT(I)
      YPOS = 0.5*(WINDOW(2)+WINDOW(4)-SPACE*(TITSPC-1.))
      GO TO 10192
10330 CONTINUE
      IF (IABS(ITITLE)-3) 10340,10350,10360
10340 CONTINUE
      TSETSZ=1.5*TSIZDF
10350 CONTINUE
      TSETOR =0.
10360 CONTINUE
      SPACE=0.1*ABS(TSETSZ)
      XPOS=0.5*(WINDOW(3)+WINDOW(1)-SPACE*(TITSPC-1.))
      YPOS = WINDOW(4) + SPACE+SPACE
      GO TO 10192
10370 CONTINUE
      IF (IABS(ITITLE)-3) 10380,10390,10400
10380 CONTINUE
      TSETSZ=TSIZDF
10390 CONTINUE
      TSETOR =-90.
10400 CONTINUE
      SPACE=0.1*ABS(TSETSZ)
      XPOS = WINDOW(3)+1.5*SPACE
      YPOS = 0.5*(WINDOW(2)+WINDOW(4)+SPACE*(TITSPC-1.))
      GO TO 10192
10210 CALL T2ERR(INFOIN,INFOIN,X,ERRFIL,5)
10200 GO TO ( 10220,10220,10260,10260,10330,10330,10370,10195),KEYORD
10195 CONTINUE
      IERFLG = 1
      NERR = NERR+1
      IF(NERR .GE. 5)GOTO 10421
      WRITE(6,10430)X,X
10430 FORMAT (   41H *** ERROR *** INVALID POSITION KEYWORD ",A1,      3
     *H" (,Z8,   15H). CALL IGNORED)
      CALL ERRTRA
10421 CONTINUE
      GO TO 10192
10191 CALL TOKEN(INFOIN,X,   15HX,B,Y,L,G,T,R,;)
      GOTO(10195,10200,10195,10195,10195,10195,10195,10210,10195),INTERP
10192 CONTINUE
      GOTO10161
10171 IF(NARG .NE. 4)GOTO 10441
      XPOS=T2REAL(X)
      YPOS=T2REAL(Y)
      IF (IABS(ITITLE)-3) 10450,10460,10470
10450 CONTINUE
      TSETSZ=TSIZDF
10460 CONTINUE
      TSETOR =0.
10470 CONTINUE
10441 CONTINUE
10161 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10491
      WRITE(DBGFIL,10500)TITX,TITY,TSETSZ,TSETOR,NCHARS,(TEXT(I),I=1,NCH
     *ARS)
10500 FORMAT (60X,    5H(X,Y),2G10.2,5X,    3HSIZ,F5.1,    6H   ANG,F6.1
     *,I5,    3H CH,  /,(60X,17(Z2,1X)))
10491 CONTINUE
      IF(IERFLG .NE. 0)GOTO 10521
      IF(IDATIT .EQ. 0 .OR. NARG .NE. 4)GOTO 10541
      IF (.NOT.FLAGS(75)) CALL TXDEF2
      IF(.NOT.(FLAGS(75)))GOTO 10561
      XYZ(1)=XPOS
      XYZ(2)=YPOS
      XYZ(3)=XYZLIM(3,1)
      CALL T2XFRM(3,1,XYZ,NONLIN,XFRM13,2,XY)
      TITX=XY(1)
      TITY=XY(2)
      GOTO 10571
10561 CONTINUE
      IERFLG = 1
      NERR = NERR+1
      IF(NERR .GE. 5)GOTO 10591
      WRITE(ERRFIL,10600)
10600 FORMAT (   45H *** ERROR *** DATA COORDINATE SYSTEM NOT SET,     1
     *7H. TITLE NOT MADE.)
      CALL ERRTRA
10591 CONTINUE
10571 CONTINUE
10551 CONTINUE
      GOTO10531
10541 IF(NARG .EQ. 2)GOTO 10611
      TITX = XPOS
      TITY = YPOS
      GOTO 10621
10611 CONTINUE
      ANGLE=TSETOR*.017453
      SPACE=ABS(0.2*TSETSZ)
      TITX = TITX + SPACE*SIN(ANGLE)
      TITY = TITY - SPACE*COS(ANGLE)
10621 CONTINUE
10531 CONTINUE
      IF (NCASE.NE.0) TSETSZ=ABS(TSETSZ)
      FLAGS(30)=.TRUE.
      XYZ(1)=TITX
      XYZ(2)=TITY
      CALL T2XFRM(2,1,XYZ,LINEAR,XFRM34,2,XY)
      IF (IERFLG.EQ.0) CALL TXTEXT(XY(1),XY(2),TSETSZ,TSETOR, LBUFFR,NCH
     *ARS,LBUFFR(257),NCASE)
      FLAGS(30)=.FALSE.
10521 CONTINUE
      ITITLE=1
      TITSPC=0.
      RETURN
      END
cnext ********************************************************
      SUBROUTINE TDHIST(NP,X,Y,DX,DY,JTYPE)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1  STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      DIMENSION X(NP),Y(NP),DX(NP),DY(NP)
      DIMENSION JTYPE(1)
      INTEGER T2INTR
      REAL T2REAL
      EXTERNAL T2INTR,T2REAL
      DATA NERR/0/
cccc      NARG = T1NARG(6)
cccc      IF((NARG .GE. 3) .AND. (NARG .LE. 6))GOTO 10641
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 6		! Y.Teramoto
10641 CONTINUE
      INTNP=T2INTR(NP)
      FLAGS(30)=.TRUE.
      IF(.NOT.(FLAGS(29)))GOTO 10661
      WRITE(DBGFIL,10670)INTNP
10670 FORMAT(30X,   18HTDHIST CALLED WITH,I4,    7H POINTS)
10661 CONTINUE
      CALL T2TYPE(    4HX   ,    6HTDHIST,INTNP,X)
      CALL T2TYPE(    4HY   ,    6HTDHIST,INTNP,Y)
      IXXXXX=NARG-2
10680 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 4))GOTO 10741
      WRITE(6,10750)IXXXXX
10750 FORMAT(    9H NARG-2 (,I5,   31H) OUT OF RANGE FOR CASE (1 TO4))
      CALL ERRTRA
      STOP 7777
10741 CONTINUE
      GO TO (10690,10700,10710,10720),IXXXXX
10690 CONTINUE
      CALL T2STLX(X,Y,    4HNONE,    4HNONE,INTNP)
      CALL T2HIST(X,Y,    4HNONE,    4HNONE,    4HNONE,    4HNONE,0,0,IN
     *TNP,1)
      GOTO 10682
10700 CONTINUE
      CALL T2TYPE(    4HDX  ,    6HTDHIST,INTNP,DX)
      CALL T2STLX(X,Y,DX,    4HNONE,INTNP)
      CALL T2HIST(X,Y,    4HNONE,DX,    4HNONE,    4HNONE,0,0,INTNP,1)
      GOTO 10682
10710 CONTINUE
      CALL T2TYPE(    4HDX  ,    6HTDHIST,INTNP,DX)
      CALL T2TYPE(    4HDY  ,    6HTDHIST,INTNP,DY)
      CALL T2STLX(X,Y,DX,DY,INTNP)
      CALL T2HIST(X,Y,    4HNONE,DX,DY,    4HNONE,0,0,INTNP,1)
      GOTO 10682
10720 CONTINUE
      CALL T2TYPE(    4HDX  ,    6HTDHIST,INTNP,DX)
      CALL T2TYPE(    4HDY  ,    6HTDHIST,INTNP,DY)
      CALL T2STLX(X,Y,DX,DY,INTNP)
      CALL T2HIST(X,Y,    4HNONE,DX,DY,    4HNONE,JTYPE(1),0,INTNP,1)
      GOTO 10682
10682 CONTINUE
      FLAGS(30)=.FALSE.
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDJOIN(NP,X,Y,DX,DY, LEVEL , MODE )
      DIMENSION X(NP),Y(NP),DX(NP),DY(NP)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      INTEGER T2INTR
      REAL T2REAL
      EXTERNAL T2INTR,T2REAL
      DATA NERR/0/
cccc      NARG = T1NARG(7)
cccc      IF((NARG .GE. 3) .AND. (NARG .LE. 7))GOTO 10771
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 7		! Y.Teramoto
10771 CONTINUE
      FLAGS(30)=.TRUE.
      INTNP=T2INTR(NP)
      IF(.NOT.(FLAGS(29)))GOTO 10791
      WRITE(DBGFIL,10800)INTNP
10800 FORMAT(30X,   18HTDJOIN CALLED WITH,I4,    7H POINTS)
10791 CONTINUE
      CALL T2TYPE(    4HX   ,    6HTDJOIN,INTNP,X)
      CALL T2TYPE(    4HY   ,    6HTDJOIN,INTNP,Y)
10811 CONTINUE
      I=NARG-2
      LLL=0
      MMM=0
      GO TO (10820,10830,10840,10850,10860),I
10860 CONTINUE
      MMM=T2INTR(MODE)
10850 CONTINUE
      LLL=T2INTR(LEVEL)
10840 CONTINUE
      CALL T2TYPE(    4HDX  ,    6HTDJOIN,INTNP,DX)
      CALL T2TYPE(    4HDY  ,    6HTDJOIN,INTNP,DY)
      CALL T2STLX(X,Y,DX,DY,INTNP)
      GOTO 10812
10830 CONTINUE
      CALL T2TYPE(    4HDX  ,    6HTDJOIN,INTNP,DX)
      CALL T2STLX(X,Y,DX,    4HNONE,INTNP)
      GOTO 10812
10820 CONTINUE
      CALL T2STLX(X,Y,    4HNONE,    4HNONE,INTNP)
10812 CONTINUE
      CALL T2JOIN(X,Y,    4HNONE,MMM,LLL,INTNP,1)
      FLAGS(30)=.FALSE.
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDLIMS(WORD,NP,VALS,ERRS)
      INTEGER NP
      REAL VALS(NP),ERRS(NP)
cmt4       BYTE WORD(4)
      character*1  WORD(4)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      INTEGER T2INTR
      REAL T2REAL
      EXTERNAL T2INTR,T2REAL
      INTEGER INFOIN(9)
      INTEGER IFLAG(2)
      REAL EXTREM(2)
      DATA NERR/0/
cccc      NARG = T1NARG(4)
cccc      IF((NARG .GE. 3) .AND. (NARG .LE. 4))GOTO 10881
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 4		! Y.Teramoto
10881 CONTINUE
      INTNP=T2INTR(NP)
      IF(.NOT.(FLAGS(29)))GOTO 10901
      WRITE(DBGFIL,10910)WORD,INTNP
10910 FORMAT (30X,    7HTDLIMS(,4A1,   13H) CALLED WITH,I4,    7H POINTS
     *)
10901 CONTINUE
      CALL T2TYPE(    4HVALS,    6HTDLIMS,INTNP,VALS)
      IF (NARG.EQ.4) CALL T2TYPE(    4HERRS,    6HTDLIMS,INTNP,ERRS)
      DO 10921 I=2,9
      INFOIN(I)=0
10921 CONTINUE
10922 CONTINUE
      INFOIN(1)=1
      GO TO 10931
10960 CONTINUE
      IXYZ=INTEG
      GO TO 10932
10950 CALL T2ERR(INFOIN,INFOIN,WORD,ERRFIL,5)
10940 GO TO ( 10960,10960,10960,10935),KEYORD
10935 CONTINUE
      IXYZ=0
      GO TO 10932
10931 CALL TOKEN(INFOIN,WORD,   13HX:1,Y:2,Z:3,;)
      GOTO(10935,10940,10935,10935,10935,10935,10935,10950,10935),INTERP
10932 CONTINUE
      INFOIN(1)=4
      GO TO 10971
11000 CONTINUE
      IFLAG(1)=-1
      IFLAG(2)=1
      GO TO 10972
11010 CONTINUE
      IFLAG(1)=1
      IFLAG(2)=-1
      GO TO 10972
10990 CALL T2ERR(INFOIN,INFOIN,WORD,ERRFIL,5)
10980 GO TO ( 11000,11010,10975),KEYORD
10975 CONTINUE
      IFLAG(1)=-1
      IFLAG(2)=-1
      GO TO 10972
10971 CALL TOKEN(INFOIN,WORD,    9HMIN,MAX,;)
      GOTO(10975,10980,10975,10975,10975,10975,10975,10990,10975),INTERP
10972 CONTINUE
      IF(IXYZ .EQ. 0)GOTO 11031
      EXTREM(1)=XYZLIM(IXYZ,1)
      EXTREM(2)=XYZLIM(IXYZ,2)
      IXYZ2 = 2*IXYZ
      IF(IFLAG(1) .NE. -1)GOTO 11051
      IF (FLAGS(IXYZ2+3)) IFLAG(1) = 0
      FLAGS(IXYZ2+3) = .TRUE.
      FLAGS(IXYZ2+63) = .TRUE.
11051 CONTINUE
      IF(IFLAG(2) .NE. -1)GOTO 11071
      IF (FLAGS(IXYZ2+4)) IFLAG(2) = 0
      FLAGS(IXYZ2+4) = .TRUE.
      FLAGS(IXYZ2+64) = .TRUE.
11071 CONTINUE
      IF(NARG .NE. 4)GOTO 11091
      CALL T2STL1(VALS,ERRS,INTNP,NONLIN(IXYZ),IFLAG,EXTREM,1,1,1)
      GOTO 11101
11091 CONTINUE
      CALL T2STL1(VALS,    4HNONE,INTNP,NONLIN(IXYZ),IFLAG,EXTREM,1,1,1)
11101 CONTINUE
11081 CONTINUE
      XYZLIM(IXYZ,1)=EXTREM(1)
      XYZLIM(IXYZ,2)=EXTREM(2)
      GOTO 11111
11031 CONTINUE
      IF(NERR .GT. 5)GOTO 11131
      WRITE(6,11140)
11140 FORMAT(   28H *** ERROR *** CALL IGNORED.)
      CALL ERRTRA
11131 CONTINUE
11111 CONTINUE
11021 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11161
      WRITE(DBGFIL,11170)(FLAGS(I),I=5,10),XYZLIM
11170 FORMAT (60X,    7HTDLIMS ,3(2L1,1X)/63X,    4HLIMS,6G10.3)
11161 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDMAIN( I )
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      DATA NERR/0/
cccc      NARG = T1NARG(1)
cccc      IF((NARG .GE. 0) .AND. (NARG .LE. 1))GOTO 11191
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 1		! Y.Teramoto
11191 CONTINUE
      CALL T2MAIN(J)
      IF (NARG.EQ.1) I=J
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDNEWP( ALIAS )
cmt4       BYTE ALIAS(8)
      character*1 ALIAS(8)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
cmt4       BYTE BLANK,ZERO,MINUS
      character*1 BLANK,ZERO,MINUS
c Y.Teramoto modified DATA BLANK/1H / -> DATA BLANK/z40/
      DATA BLANK/x'40'/,ZERO/0/,MINUS/x'FF'/
      DATA NERR/0/
cccc      NARG = T1NARG(1)
cccc      IF((NARG .GE. 0) .AND. (NARG .LE. 1))GOTO 11211
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 1		! Y.Teramoto
11211 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11231
      WRITE(DBGFIL,11240)
11240 FORMAT(30X,    8HNEW PLOT)
11231 CONTINUE
      NC = 0
      IF(NARG .EQ. 0)GOTO 11261
11270 CONTINUE
11271 CONTINUE
      DO 11281 I=1,4
      IF(ALIAS(I).EQ.ZERO)GOTO 11272
      IF(ALIAS(I).EQ.MINUS)GOTO 11272
11281 CONTINUE
11282 CONTINUE
      NC = 4
      DO 11291 I=5,8
      IF(ALIAS(I).EQ.ZERO)GOTO 11272
      IF(ALIAS(I).EQ.MINUS)GOTO 11272
11291 CONTINUE
11292 CONTINUE
      NC = 8
11272 CONTINUE
11261 CONTINUE
      IF(NC .LE. 0)GOTO 11311
      DO 11321 I=1,NC
      PXNAME(I)=ALIAS(I)
11321 CONTINUE
11322 CONTINUE
11311 CONTINUE
      IF(NC .GE. 8)GOTO 11341
      NC=NC+1
      DO 11351 I=NC,8
      PXNAME(I)=BLANK
11351 CONTINUE
11352 CONTINUE
11341 CONTINUE
      FLAGS(30)=.TRUE.
      CALL T2REST
      CALL TXNEXT
      FLAGS(30)=.FALSE.
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDPLOT( NP ,X,Y,DX,DY, ISYM )
      DIMENSION X(NP),Y(NP),DX(NP),DY(NP)
cmt4       BYTE ISYM(2)
      character*1 ISYM(2)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      INTEGER T2INTR
      REAL T2REAL
      EXTERNAL T2INTR,T2REAL
      REAL TSYM(2)
      EXTERNAL TXSYM
      REAL TXSYM
cmt4       BYTE CSYM(3),BLANKS(3)
      character*1 CSYM(3),BLANKS(3)
      DATA TSYM/0.,0./
      DATA CSYM(3)/    1H /
      DATA BLANKS/1H ,1H ,1H /
      DATA NERR/0/
cccc      NARG = T1NARG(6)
cccc      IF((NARG .GE. 0) .AND. (NARG .LE. 6))GOTO 11371
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 6		! Y.Teramoto
11371 CONTINUE
      FLAGS(30)=.TRUE.
      IF(NARG .LT. 3)GOTO 11391
      INTNP=T2INTR(NP)
      IF(.NOT.(FLAGS(29)))GOTO 11411
      WRITE(DBGFIL,11420)INTNP
11420 FORMAT(30X,   18HTDPLOT CALLED WITH,I4,    7H POINTS)
11411 CONTINUE
      IF(NARG .NE. 6)GOTO 11441
      CSYM(1)=ISYM(1)
      CSYM(2)=ISYM(2)
      TSYM(2) = TXSYM(CSYM)
      GOTO 11451
11441 CONTINUE
      IF (SYMBOL.EQ.0.) SYMBOL=TXSYM(BLANKS)
      TSYM(2)=SYMBOL
11451 CONTINUE
11431 CONTINUE
      CALL T2TYPE(    4HX   ,    6HTDPLOT,INTNP,X)
      CALL T2TYPE(    4HY   ,    6HTDPLOT,INTNP,Y)
      IF(NARG .LT. 5)GOTO 11471
      CALL T2TYPE(    4HDX  ,    6HTDPLOT,INTNP,DX)
      CALL T2TYPE(    4HDY  ,    6HTDPLOT,INTNP,DY)
      CALL T2STLX(X,Y,DX,DY,INTNP)
      CALL T2PLOT(X,Y,    4HNONE,DX,DY,    4HNONE,TSYM,INTNP,1)
      GOTO11461
11471 IF(NARG .NE. 4)GOTO 11481
      CALL T2TYPE(    4HDX  ,    6HTDPLOT,INTNP,DX)
      CALL T2STLX(X,Y,DX,    4HNONE,INTNP)
      CALL T2PLOT(X,Y,    4HNONE,DX,    4HNONE,    4HNONE,TSYM,INTNP,1)
      GOTO11461
11481 IF(NARG .NE. 3)GOTO 11491
      CALL T2STLX(X,Y,    4HNONE,    4HNONE,INTNP)
      CALL T2PLOT(X,Y,    4HNONE,    4HNONE,    4HNONE,    4HNONE,TSYM,I
     *NTNP,1)
11491 CONTINUE
11461 CONTINUE
      GOTO 11501
11391 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11521
      WRITE(DBGFIL,11530)NARG
11530 FORMAT(60X,   18HTDPLOT CALLED WITH,I2,    5H ARGS)
11521 CONTINUE
      CALL T2TCKS
11501 CONTINUE
11381 CONTINUE
      FLAGS(30)=.FALSE.
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSET(TEXT)
cmt4       BYTE TEXT(257)
      character*1 TEXT(257)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      INTEGER INFOIN(9),LENGTH
cmt4       BYTE SEMIC,LBUFFR(400)
      character*1 SEMIC,LBUFFR(400)
      EQUIVALENCE(INFOIN(1),LENGTH)
      EQUIVALENCE (BUFFER(1),LBUFFR(1))
      DATA SEMIC/    1H;/
      LENGTH = 0
      IERFLG = 0
11601 CONTINUE
      LENGTH=LENGTH+1
      LBUFFR(LENGTH)=TEXT(LENGTH)
      IF(LBUFFR(LENGTH).EQ.SEMIC)GOTO 11602
      IF(LENGTH .LT. 256)GOTO 11621
      WRITE(ERRFIL,11630)
11630 FORMAT (   41H *** WARNING *** NO TERMINATING SEMICOLON,     14H C
     *ALL IGNORED.)
      IERFLG = 1
      CALL ERRTRA
      GOTO 11602
11621 CONTINUE
      GOTO 11601
11602 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11651
      WRITE(DBGFIL,11660)(LBUFFR(I),I=1,LENGTH)
11660 FORMAT (30X,    7HTDSET(',95A1)
11651 CONTINUE
      IF(IERFLG .NE. 0)GOTO 11681
      FLAGS(30) = .TRUE.
      DO 11691 I=2,9
      INFOIN(I)=0
11691 CONTINUE
11692 CONTINUE
      CALL T2SETC(INFOIN,LBUFFR)
      FLAGS(30)=.FALSE.
11681 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDTITL(TEXT, X , Y )
      COMMON /TDFCOM/ITITLE,TSETSZ,TSETOR,TITSPC,IDATIT
cmt4       BYTE TEXT(256)
      character*1 TEXT(256)
      REAL X,Y
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      DATA NERR/0/
cccc      NARG = T1NARG(3)
cccc      IF((NARG .GE. 1) .AND. (NARG .LE. 3))GOTO 11711
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 3			! Teramoto
11711 CONTINUE
cccc      IF (NARG.EQ.1) CALL TDCASE(TEXT,    2H; )
cccc      IF (NARG.EQ.2) CALL TDCASE(TEXT,    2H; ,X)
      IF (NARG.EQ.3) CALL TDCASE(TEXT,    2H; ,X,Y)
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDTITN( VAL , W , D , X , Y )
      COMMON /TDFCOM/ITITLE,TSETSZ,TSETOR,TITSPC,IDATIT
      INTEGER W,D
      REAL VAL,X,Y
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
cmt4       BYTE TEXT(20),SEMIC
      character*1 TEXT(20),SEMIC
      INTEGER T2INTR
      REAL T2REAL
      DATA SEMIC/    1H;/
      DATA NERR/0/
cccc      NARG = T1NARG(5)
cccc      IF((NARG .GE. 3) .AND. (NARG .LE. 5))GOTO 11731
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 5		! Y.Teramoto
11731 CONTINUE
      X=T2REAL(VAL)
      LW=T2INTR(W)
      IF (IABS(LW).GT.19) LW=ISIGN(19,LW)
      LD=T2INTR(D)
      CALL T2CRMF(X,LW,LD,TEXT,IF,IL)
      LW=IABS(LW)
      TEXT(LW+1)=SEMIC
cccc      IF (NARG.EQ.3) CALL TDCASE(TEXT,    2H; )
cccc      IF (NARG.EQ.4) CALL TDCASE(TEXT,    2H; ,X)
      IF (NARG.EQ.5) CALL TDCASE(TEXT,    2H; ,X,Y)
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDTSET( SIZE , ANGLE , SPACES , IDATA )
      COMMON /TDFCOM/ITITLE,TSETSZ,TSETOR,TITSPC,IDATIT
      INTEGER T2INTR
      REAL T2REAL
      EXTERNAL T2INTR,T2REAL
cccc      EXTERNAL T1NARG
cccc      INTEGER T1NARG
      DATA NERR/0/
cccc      NARG = T1NARG(4)
cccc      IF((NARG .GE. 1) .AND. (NARG .LE. 4))GOTO 11751
cccc      NERR=NERR+1
cccc      CALL T2ERRA(NERR,NARG)
cccc      RETURN
	NARG = 4		! Y.Teramoto
11751 CONTINUE
      TITSPC=0.
      TSETOR=0.
      IDATIT=0
      GO TO (11760,11770,10820,10830),NARG
10830 CONTINUE
      IDATIT=IDATA
10820 CONTINUE
      TITSPC=T2REAL(SPACES)
11770 CONTINUE
      TSETOR=T2REAL(ANGLE)
11760 CONTINUE
      TSETSZ=T2REAL(SIZE)
      ITITLE=NARG+2
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2ERRA(NERR,NARG)
      INTEGER NERR,NARG
      IF(NERR .GT. 5)GOTO 11791
      WRITE(6,11800)NARG
11800 FORMAT (   14H *** ERROR ***,I3,   26H ARGUEMENTS. CALL IGNORED.)
      CALL ERRTRA
11791 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2TYPE(NAME,ROUTIN,N,ARRAY)
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2STLX(XVALS,YVALS,XERRS,YERRS,NP)
      INTEGER NP
      REAL XVALS(NP),YVALS(NP),XERRS(NP),YERRS(NP)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      INTEGER IFLAG(2)
      REAL EXTREM(2)
      IF(NP .LE. 0)GOTO 11961
      IF(.NOT.(FLAGS(5)))GOTO 11981
      IFLAG(1)=1
      EXTREM(1)=XYZLIM(1,1)
      GOTO 11991
11981 CONTINUE
      IFLAG(1)=-1
11991 CONTINUE
11971 CONTINUE
      IF(.NOT.(FLAGS(6)))GOTO 12011
      IFLAG(2)=1
      EXTREM(2)=XYZLIM(1,2)
      GOTO 12021
12011 CONTINUE
      IFLAG(2)=-1
12021 CONTINUE
12001 CONTINUE
      CALL T2STL1(XVALS,XERRS,NP,NONLIN(1),IFLAG,EXTREM,1,1,1)
      XYZLIM(1,1)=EXTREM(1)
      XYZLIM(1,2)=EXTREM(2)
      IF(.NOT.(FLAGS(7)))GOTO 12041
      IFLAG(1)=1
      EXTREM(1)=XYZLIM(2,1)
      GOTO 12051
12041 CONTINUE
      IFLAG(1)=-1
12051 CONTINUE
12031 CONTINUE
      IF(.NOT.(FLAGS(8)))GOTO 12071
      IFLAG(2)=1
      EXTREM(2)=XYZLIM(2,2)
      GOTO 12081
12071 CONTINUE
      IFLAG(2)=-1
12081 CONTINUE
12061 CONTINUE
      CALL T2STL1(YVALS,YERRS,NP,NONLIN(2),IFLAG,EXTREM,1,1,1)
      XYZLIM(2,1)=EXTREM(1)
      XYZLIM(2,2)=EXTREM(2)
      DO 12091 I=5,8
      FLAGS(I)=.TRUE.
12091 CONTINUE
12092 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12111
      WRITE(DBGFIL,12120)XYZLIM
12120 FORMAT (60X,   13HT2STLX.XYZLIM,3G10.3/73X,3G10.3)
12111 CONTINUE
11961 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2RDMS
      CALL T2MERR
      RETURN
      END
cnext ****************************************
      SUBROUTINE T23HST(ARRAY,NDIM1,NDIM2,IXYZ)
      CALL T2MERR
      RETURN
      END
cnext ****************************************
      SUBROUTINE T23JIN(ARRAY,NDIM1,NDIM2,IXYZ)
      CALL T2MERR
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2MERR
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      WRITE(ERRFIL,12130)
12130 FORMAT(' *** ERROR ***  YOU HAVE INVOKED A SUBROUTINE,',
     *' WHICH IS PART OF THE 3-D PACKAGE.',/,16X,'THE 3-D PACKAGE HAS',
     *' NOT BEEN LOADED IN THIS JOB,',' AND THE JOB CANNOT BE RUN',
     *'SUCCESSFULLY.')
      CALL ERRTRA
      CALL TXEND
      STOP
      END
cnext ****************************************
ccccccccccc 21-Aug-91 Modified by Y.Teramoto
	subroutine	topmain
ccccccccccccccccccccccc
c
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
      REAL FLOTNG
cmt4       BYTE STRING
      character*1 STRING
      INTEGER INFO(10)
cmt4       BYTE LINE(80),KEYSTR(9)
      character*1 LINE(80),KEYSTR(9)
      DATA KEYSTR/    1HK,    1HE,    1HY,    1H1,    1HK,    1HE,    1H
     *Y,    1H2,    1H;/
      MAXSTR=20
10011 CONTINUE
      WRITE(6,10020)
10020 FORMAT(   12H TYPE A CARD)
      READ (5,10) LINE
10    FORMAT (80A1)
      WRITE(6,10030)LINE
10030 FORMAT(    9H CARD IS ,80A1)
      INFO(1)=80
      DO 10041 I=2,9
      INFO(I)=0
10041 CONTINUE
10042 CONTINUE
10051 CONTINUE
      CALL TOKEN(INFO,LINE   ,   41HKEY1,KEY2,KEYS3:3,KEYS4:4,RIGHT:8,RI
     *TE:8;)
      GO TO (10060,10070,10080,10090,10100,10110,10120,  10130,10140),IN
     *TERP
10060 CONTINUE
      GOTO 10052
10070 CONTINUE
      WRITE(6,10150)
10150 FORMAT(    4H KEY)
      GO TO 10160
10080 CONTINUE
      WRITE(6,10170)
10170 FORMAT(    8H INTEGER)
      GO TO 10160
10090 CONTINUE
      WRITE(6,10180)
10180 FORMAT(   15H FLOATING POINT)
      GO TO 10160
10100 CONTINUE
      WRITE(6,10190)
10190 FORMAT(   14H QUOTED STRING)
      GO TO 10160
10110 CONTINUE
      WRITE(6,10200)
10200 FORMAT(   11H IDENTIFIER)
      GO TO 10160
10120 CONTINUE
      WRITE(6,10210)
10210 FORMAT(   14H ILLEGAL FIELD)
      GO TO 10160
10130 CONTINUE
      WRITE(6,10220)
10220 FORMAT(   18H AMBIGUOUS KEYWORD)
      GO TO 10160
10140 CONTINUE
      WRITE(6,10230)
10230 FORMAT(   18H SPECIAL CHARACTER)
      GO TO 10160
10160 CONTINUE
      WRITE(6,10240)INTERP,INTEG,FLOTNG,KEYORD,NSTRNG
10240 FORMAT (I4,I15,G15.5,2I3)
      IF (NSTRNG.GT.0) WRITE(6,10250)(STRING(I),I=1,NSTRNG)
10250 FORMAT(1X,80A1)
      GOTO 10051
10052 CONTINUE
      GOTO 10011
10012 CONTINUE
      STOP
      END
cnext ****************************************
c      SUBROUTINE TOKEN(INFO,CHARS,KEYS)
cnext ****************************************
      SUBROUTINE TOKRED(INFO,CHARS,LENGTH,INFILE,FORMT)
      INTEGER INFO(10),LENGTH,INFILE,FORMT(4)
cmt4       BYTE CHARS(LENGTH)
      character*1 CHARS(LENGTH)
      COMMON /TOKENT/ TYPTAB
cmt4       BYTE TYPTAB(128)
      character*1 TYPTAB(128)
      INTEGER ISTAT,CHARAC,CHTYPE,MAXCOL,ICOL
cmt4       BYTE SLASH
      character*1 SLASH
      EQUIVALENCE (IXXXXX,XXXXXX)
      DATA SLASH/    1H//
      MAXCOL=MAX0(0,INFO(1))
      ICOL=MAX0(0,INFO(3))
      IF (INFO(1).LT.0) GO TO 11160
      IF(INFO(9) .EQ. 0)GOTO 11181
      ICOL=INFO(8)
      INFO(9)=0
      GOTO 11191
11181 CONTINUE
11201 CONTINUE
      IF(ICOL .LT. MAXCOL)GOTO 11221
11231 CONTINUE
      READ (INFILE,FORMT,END=11160) CHARS
      IF(CHARS(1).NE.SLASH)GOTO 11232
      IF(CHARS(2) .EQ. SLASH)GOTO 11231
11232 CONTINUE
      INFO(1)=LENGTH
      IF (INFO(2).LE.0) INFO(2)=LENGTH
      INFO(7)=INFO(7)+1
      INFO(9)=0
      INFO(10)=0
      ICOL=0
      CHTYPE=8
      GOTO 11241
11221 CONTINUE
      ICOL=ICOL+1
      IF(ICOL .GT. MAXCOL)GOTO 11261
cmt4       CHARAC=CHARS(ICOL)
      CHARAC=ichar(CHARS(ICOL))
      GOTO 11271
11261 CONTINUE
      CHARAC=0
11271 CONTINUE
11251 CONTINUE
cmt4       CHTYPE=TYPTAB(CHARAC+1)
      CHTYPE=ichar(TYPTAB(CHARAC+1))
11241 CONTINUE
11211 CONTINUE
      IF(CHTYPE .NE. 8)GOTO 11201
11202 CONTINUE
11191 CONTINUE
11171 CONTINUE
      INFO(3)=ICOL
      INFO(8)=ICOL
      INFO(4)=0
      INFO(5)=0
      INFO(6)=0
      RETURN
11160 CONTINUE
      IF(INFO(1) .NE. 0)GOTO 11291
      WRITE(6,11300)
11300 FORMAT(   30H *** WARNING *** NO INPUT FILE)
11291 CONTINUE
      INFO(1)=-1
      INFO(3)=0
      INFO(4)=0
      INFO(5)=0
      INFO(9)=0
      INFO(10)=0
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
c      SUBROUTINE TXCTOL(CHAR,LEN,XARRAY,YARRAY,BARRAY,NPTS)
cnext ****************************************
      SUBROUTINE TXATTN
      RETURN
      END
cnext ****************************************
      SUBROUTINE TXDEFW
      COMMON/TXCOM/PLIM(4,3),DLIM(4,3), XLINE,YLINE,XFRMFC,TXTSIZ,TXTANG
     *,TXTAN0,TXTWID,TXTTXT,TXTCOL, IPAT,NPAT, PAT(20),PATCUM(20),PATTOT
     *,XPAT,YPAT, XARRAY(60),YARRAY(60), NPTS,IDDEVC,IDDFLG,SIDEWZ,MOVDR
     *W(2),NPXEL,PXEL(500)
      INTEGER*4 MOVDRW,NPXEL,PXEL
      COMMON /TXARGS/ OARG,QTXT,QLIN
      CHARACTER*8 OARG(20),QLIN(3),QTXT(7)
      REAL XLINE,YLINE,TXTSIZ,TXTANG,TXTAN0,XFRMFC,PLIM ,PAT,PATCUM,PATT
     *OT,XPAT,YPAT
      INTEGER NPTS,TXTWID,TXTTXT,TXTCOL,IDDEVC,IDDFLG,SIDEWZ ,IPAT,NPAT
      INTEGER*4 DLIM
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      IF(.NOT.(FLAGS(29)))GOTO 10831
      WRITE(DBGFIL,10840)XYPART,SCREEN
10840 FORMAT(60X,   14HTXDEFW XYPART ,4F5.1,    7H SCREEN,2F5.1)
10831 CONTINUE
      IF (FLAGS(74)) RETURN
      IF (.NOT. FLAGS(73)) CALL TXDEF1
cccccccccccccccccccccc 11-Sep-91 Modified by Y.Teramoto
      SIZ=ABS(XYPART(1,2))*13.*REDUCE(2)/SCREEN(1)
cccc      SIZ=ABS(XYPART(1,2))*10.*REDUCE(2)/SCREEN(1)
cccccccccccccccccccccccc
      T=ABS(XYPART(2,2))*10.*REDUCE(2)/SCREEN(2)
      IF (SIZ.LT.T) SIZ=T
      IF (SIZ.LT.1.) SIZ=1.
      IF(SIZ .GT. 3.)GOTO 10861
      SIZ=0.4*SIZ-2.4
      GOTO 10871
10861 CONTINUE
      SIZ=-3.6/SIZ
10871 CONTINUE
10851 CONTINUE
      IF (.NOT.FLAGS(76)) TSIZDF=SIZ
      IF (.NOT.FLAGS(77)) LBLSIZ=SIZ
      IF (.NOT.FLAGS(78)) TIKSIZ(1)=0.05*SIZ
      T=ABS(TSIZDF)
      EL=ABS(LBLSIZ)
      IF((FLAGS(11)))GOTO 10891
      IF(XYPART(1,2) .NE. 1.)GOTO 10911
      WINDOW(1)=SCREEN(1)*0.2
      WINDOW(3)=SCREEN(1)*0.9
      GOTO 10921
10911 CONTINUE
      SIZ=SCREEN(1)/XYPART(1,2)
      VMAX=SIZ*XYPART(1,1)
      VMIN=VMAX-SIZ
      WINDOW(3) = VMAX-REDUCE(2)*0.6*T
      WINDOW(1) = VMIN+REDUCE(2)*(0.65*T+0.5*EL)
10921 CONTINUE
10901 CONTINUE
10891 CONTINUE
      IF((FLAGS(12)))GOTO 10941
      IF(XYPART(2,2) .NE. 1.)GOTO 10961
      WINDOW(2)=SCREEN(2)*0.2
      WINDOW(4)=SCREEN(2)*0.9
      GOTO 10971
10961 CONTINUE
      SIZ=SCREEN(2)/XYPART(2,2)
      VMAX=SIZ*XYPART(2,1)
      VMIN=VMAX-SIZ
      WINDOW(4) = VMAX-REDUCE(2)*0.7*T
      WINDOW(2) = VMIN+REDUCE(2)*(0.5*T+0.2*EL)
10971 CONTINUE
10951 CONTINUE
10941 CONTINUE
      FLAGS(74) = .TRUE.
      IF(.NOT.(FLAGS(29)))GOTO 10991
      WRITE(DBGFIL,11000)WINDOW
11000 FORMAT(60X,    6HWINDOW,4F6.2)
10991 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TXDEF2
      COMMON/TXCOM/PLIM(4,3),DLIM(4,3), XLINE,YLINE,XFRMFC,TXTSIZ,TXTANG
     *,TXTAN0,TXTWID,TXTTXT,TXTCOL, IPAT,NPAT, PAT(20),PATCUM(20),PATTOT
     *,XPAT,YPAT, XARRAY(60),YARRAY(60), NPTS,IDDEVC,IDDFLG,SIDEWZ,MOVDR
     *W(2),NPXEL,PXEL(500)
      INTEGER*4 MOVDRW,NPXEL,PXEL
      COMMON /TXARGS/ OARG,QTXT,QLIN
      CHARACTER*8 OARG(20),QLIN(3),QTXT(7)
      REAL XLINE,YLINE,TXTSIZ,TXTANG,TXTAN0,XFRMFC,PLIM ,PAT,PATCUM,PATT
     *OT,XPAT,YPAT
      INTEGER NPTS,TXTWID,TXTTXT,TXTCOL,IDDEVC,IDDFLG,SIDEWZ ,IPAT,NPAT
      INTEGER*4 DLIM
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1  STRING
cccccccccccccccc 6-Sep-91 Modified by Y.Teramoto
	real*4	scrnsize( 4 ), scrnsizel( 4 )
cccccccccccccccccccccccccccccc
      REAL EQUAL(12),XYZLMI(3,2)
      DATA EQUAL/0.,1.,0.,0.,0.,0.,1.,0.,0.,0.,0.,1./
      IF(.NOT.(FLAGS(29)))GOTO 11021
      WRITE(DBGFIL,11030)
11030 FORMAT(60X,    6HTXDEF2)
11021 CONTINUE
      IF(.NOT.(FLAGS(5)) .OR. .NOT.(FLAGS(6)) .OR. .NOT.(FLAGS(7)) .OR.
     *.NOT.(FLAGS(8)))GOTO 11051
      FLAGS(56)=IDIMNS.EQ.3
      IF (NDIMNS(1).NE.0) FLAGS(56)=NDIMNS(1).EQ.3
      CALL TXDEFW
      IF(.NOT.(FLAGS(29)))GOTO 11071
      WRITE(DBGFIL,11080)WINDOW,NONLIN,XYZLIM
11080 FORMAT (63X,    6HWINDOW,4G10.3/63X,    6HNONLIN,3I10/  63X,    6H
     *XYZLIM,3G10.3/69X,3G10.3)
11071 CONTINUE
      IX=SIDEWZ+1
      IY=3-IX
      FACX=FLOAT(DLIM(3,1)-DLIM(1,1))/SCREEN(IX)
      FACY=FLOAT(DLIM(4,1)-DLIM(2,1))/SCREEN(IY)
ccccccccccccccccccc 6-Sep-91 Modified by Y.Teramoto
	call pxscrn( 0, 1, scrnsize )
	facx_p = ( scrnsize( 3 ) - scrnsize( 1 ) ) / screen( ix )
	facy_p = ( scrnsize( 4 ) - scrnsize( 2 ) ) / screen( iy )
cccccccccccccccccccccccccccccccccccc
      IF(SIDEWZ .NE. 0)GOTO 11101
      DLIM(3,2)=DLIM(1,1) + IFIX(WINDOW(3)*FACX)+1
      DLIM(1,2)=DLIM(1,1) + IFIX(WINDOW(1)*FACX)
      DLIM(4,2)=DLIM(2,1) + IFIX(WINDOW(4)*FACY)+1
      DLIM(2,2)=DLIM(2,1) + IFIX(WINDOW(2)*FACY)
cccccccccccccccccccc 6-Sep-91 Modified by Y.Teramoto
	scrnsizel( 3 ) = scrnsize( 1 ) + WINDOW( 3 ) * facx_p
	scrnsizel( 1 ) = scrnsize( 1 ) + WINDOW( 1 ) * facx_p
	scrnsizel( 4 ) = scrnsize( 2 ) + WINDOW( 4 ) * facy_p
	scrnsizel( 2 ) = scrnsize( 2 ) + WINDOW( 2 ) * facy_p
	call pxscrn( 1, 2, scrnsizel )
ccccccccccccccccccccccccccccccc
      GOTO 11111
11101 CONTINUE
      DLIM(3,2)=DLIM(1,1) + IFIX(WINDOW(4)*FACX)+1
      DLIM(1,2)=DLIM(1,1) + IFIX(WINDOW(2)*FACX)
      DLIM(2,2)=DLIM(4,1) - IFIX(WINDOW(3)*FACY)+1
      DLIM(4,2)=DLIM(4,1) - IFIX(WINDOW(1)*FACY)
cccccccccccccccccccc 6-Sep-91 Modified by Y.Teramoto
	scrnsizel( 3 ) = scrnsize( 1 ) + WINDOW( 4 ) * facx_p
	scrnsizel( 1 ) = scrnsize( 1 ) + WINDOW( 2 ) * facx_p
	scrnsizel( 2 ) = scrnsize( 4 ) + WINDOW( 3 ) * facy_p
	scrnsizel( 4 ) = scrnsize( 4 ) + WINDOW( 1 ) * facy_p
	call pxscrn( 1, 2, scrnsizel )
ccccccccccccccccccccccccccccccc
11111 CONTINUE
11091 CONTINUE
      CALL T2XFRM(2,2,WINDOW,LINEAR,XFRM34,2,PLIM(1,2))
      IF(PLIM(4,2) .GE. PLIM(2,2))GOTO 11131
      T=PLIM(4,2)
      PLIM(4,2)=PLIM(2,2)
      PLIM(2,2)=T
11131 CONTINUE
      NONLIN(4)=0
      CALL T2XFRM(3,2,XYZLIM,NONLIN,EQUAL,3,XYZLMI)
      NONLIN(4)=1
      DO 11141 I=1,12
      XFRM12(I)=0.
11141 CONTINUE
11142 CONTINUE
      DO 11151 I=1,3
      D=XYZLMI(I,2)-XYZLMI(I,1)
      IF(D .EQ. 0.)GOTO 11171
      XFRM12(5*I-3)=WORLD(I)/D
      XFRM12(4*I-3)=-XYZLMI(I,1)*XFRM12(5*I-3)
11171 CONTINUE
11151 CONTINUE
11152 CONTINUE
      CALL T2MPRJ(WINDOW,XFRM23)
      DO 11181 I=1,11
      XFRM14(I)=0.
11181 CONTINUE
11182 CONTINUE
      XFRM14(12)=1.
      XFRM14(2)=XFRM34(2)
      XFRM14(6)=XFRM34(5)
      XFRM14(3)=XFRM34(3)
      XFRM14(7)=XFRM34(6)
      XFRM14(4)=XFRM34(1)
      XFRM14(8)=XFRM34(4)
      CALL T2XMUL(XFRM14,XFRM23,3,XFRM24)
      CALL T2XMUL(XFRM23,XFRM12,3,XFRM13)
      CALL T2XMUL(XFRM24,XFRM12,3,XFRM14)
      IF(.NOT.(FLAGS(29)))GOTO 11201
      WRITE(DBGFIL,11210)XFRM12,XFRM13,XFRM14,XFRM23,XFRM24
11210 FORMAT (60X,    6HXFRM12,4G10.3,2(/66X,4G10.3)/  60X,    6HXFRM13,
     *4G10.3,2(/66X,4G10.3)/  60X,    6HXFRM14,4G10.3,2(/66X,4G10.3)/  6
     *0X,    6HXFRM23,4G10.3,2(/66X,4G10.3)/  60X,    6HXFRM24,4G10.3,2(
     */66X,4G10.3))
11201 CONTINUE
      FLAGS(75) = .TRUE.
11051 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TXQUIK
      COMMON/TXCOM/PLIM(4,3),DLIM(4,3), XLINE,YLINE,XFRMFC,TXTSIZ,TXTANG
     *,TXTAN0,TXTWID,TXTTXT,TXTCOL, IPAT,NPAT, PAT(20),PATCUM(20),PATTOT
     *,XPAT,YPAT, XARRAY(60),YARRAY(60), NPTS,IDDEVC,IDDFLG,SIDEWZ,MOVDR
     *W(2),NPXEL,PXEL(500)
      INTEGER*4 MOVDRW,NPXEL,PXEL
      COMMON /TXARGS/ OARG,QTXT,QLIN
      CHARACTER*8 OARG(20),QLIN(3),QTXT(7)
      REAL XLINE,YLINE,TXTSIZ,TXTANG,TXTAN0,XFRMFC,PLIM ,PAT,PATCUM,PATT
     *OT,XPAT,YPAT
      INTEGER NPTS,TXTWID,TXTTXT,TXTCOL,IDDEVC,IDDFLG,SIDEWZ ,IPAT,NPAT
      INTEGER*4 DLIM
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
      SUBROUTINE T2AXES
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL XYTXT(2,5),XYDEV(2,5)
      EQUIVALENCE (BUFFER(1),XYDEV(1,1)),  (BUFFER(11),XYTXT(1,1))
      IF(.NOT.(FLAGS(29)))GOTO 10021
      WRITE(DBGFIL,10030)FLAGS
10030 FORMAT(60X,   16HT2AXES FLAG DUMP,10(/62X,10L1,1X,10L1))
10021 CONTINUE
      IF(NPOINT .LE. 0)GOTO 10051
      DO 10061 I=5,10
      FLAGS(I)=FLAGS(I).OR.FLAGS(I+60)
10061 CONTINUE
10062 CONTINUE
      CALL T2STLM
10051 CONTINUE
      IF (.NOT.FLAGS(75)) CALL TXDEF2
      IF (.NOT.FLAGS(74)) CALL TXDEFW
      IF((FLAGS(2)))GOTO 10081
      DO 10091 I=1,5
      XYTXT(1,I)=WINDOW(1)
      XYTXT(2,I)=WINDOW(4)
10091 CONTINUE
10092 CONTINUE
      DO 10101 I=2,3
      XYTXT(1,I+1)=WINDOW(3)
      XYTXT(2,I)=WINDOW(2)
10101 CONTINUE
10102 CONTINUE
      CALL T2XFRM(2,5,XYTXT,LINEAR,XFRM34,2,XYDEV)
      DO 10111 I=1,4
      IF (FLAGS(I+18).AND.FLAGS(I+39)) CALL TXLINX(XYDEV(1,I),2,LINTEX)
10111 CONTINUE
10112 CONTINUE
      FLAGS(2) = .TRUE.
10081 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2AXIS (XYZ,IXYZ,VALUES,SCRTCH,NB,NUM,ITICKS,IAXIS,ITIC
     *K,ILABEL)
      INTEGER IXYZ,NB,ITICKS(NUM),ITICK,IAXIS,ILABEL
      REAL XYZ(3,2),VALUES(NUM),SCRTCH(NB)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
ccccccccccccccccccc 29-Jan-93 Modified by A.Miyamoto/Y.Teramoto
cmt4       BYTE VTEXT(20),VCASE(12),BLANK
      character*1 VTEXT(20),VCASE(12),BLANK
cccc      BYTE VTEXT(12),VCASE(12),BLANK
ccccccccccccccccccc end of 29-Jan-93 Modification
      REAL XY(2,4),TVAL(4),PNT(3,4),POSLBL(3),ALONG(3),UPDIR(3),  XYZWRL
     *(3,2),XYTXT(2,2),XYPNT(2,2)
      INTEGER IAX1,IAX2,IAX3
      DATA BLANK/    1H /
      IF(.NOT.(FLAGS(29)))GOTO 10131
      WRITE(DBGFIL,10140)IXYZ,NUM,IAXIS,ITICK,ILABEL,XYZ
10140 FORMAT (60X,   13HT2AXIS.  AXIS,I2,I4,   16H TICKS. IA,IT,IL,3I2/
     * 55X,2(5X,3G10.3))
10131 CONTINUE
      TICK=TIKSIZ(1)
      DO 10151 I=1,3
      POSLBL(I)=XYZ(I,1)
10151 CONTINUE
10152 CONTINUE
      VM = AMAX1(ABS(XYZ(IXYZ,1)),ABS(XYZ(IXYZ,2)))
      IPLACE=ALOG10(VM)
      IWID=5-IPLACE
      IF (IWID.LT.0) IWID=0
      IF (IWID.GT.9) IWID=9
      BASE=SCLPRM(1,IXYZ)
      IF(.NOT.(FLAGS(56)))GOTO 10171
      CALL T2XFRM(3,2,XYZ,NONLIN,XFRM12,3,XYZWRL)
      IF(.NOT.(FLAGS(29)))GOTO 10191
      WRITE(DBGFIL,10200)XYZWRL
10200 FORMAT(60X,    6HLMW   ,3G11.4/66X,3G11.4)
10191 CONTINUE
      IAX1=IXYZ
      CALL T2AXST(XYZWRL,IAX1,IAX2,IAX3)
      IF(IAX2 .LE. 0)GOTO 10221
      IF(NUM .LE. 0)GOTO 10241
      DO 10251 I=1,NUM
      POSLBL(IAX1)=VALUES(I)
      CALL T2XFRM(3,1,POSLBL,NONLIN,XFRM12,3,ALONG)
      SCRTCH(I)=ALONG(IAX1)
10251 CONTINUE
10252 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10271
      WRITE(DBGFIL,10280)(SCRTCH(I),I=1,NUM)
10280 FORMAT (60X,    8HTICKS AT,99(/65X,6G10.3))
10271 CONTINUE
10241 CONTINUE
      CALL TXSCIS(2)
      IF(IAXIS .EQ. 0)GOTO 10301
      CALL T2XFRM(3,2,XYZWRL,LINEAR,XFRM24,2,XY)
      CALL TXLINX(XY,2,LINTEX)
10301 CONTINUE
      IF(ITICK .EQ. 0)GOTO 10321
      PNT(IAX3,1)=XYZWRL(IAX3,1)
      PNT(IAX3,2)=XYZWRL(IAX3,1)
      TVAL(1)=XYZWRL(IAX2,1)+TICK
      TVAL(2)=XYZWRL(IAX2,1)-TICK
      TVAL(3)=XYZWRL(IAX2,1)+3.*TICK
      TVAL(4)=XYZWRL(IAX2,1)-3.*TICK
      IF(NUM .LE. 0)GOTO 10341
      DO 10351 I=1,NUM
      J=2+ISIGN(1,ITICKS(I))
      PNT(IAX2,1)=TVAL(J)
      PNT(IAX2,2)=TVAL(J+1)
      PNT(IAX1,1)=SCRTCH(I)
      PNT(IAX1,2)=PNT(IAX1,1)
      CALL T2XFRM(3,2,PNT,LINEAR,XFRM24,2,XY)
      CALL TXLINX(XY,2,LINTEX)
10351 CONTINUE
10352 CONTINUE
10341 CONTINUE
10321 CONTINUE
      IF(ILABEL .EQ. 0)GOTO 10371
      POSLBL(IAX2)=XYZWRL(IAX2,1) -(0.1*ABS(LBLSIZ)+3.*ABS(TICK))
      POSLBL(IAX3)=XYZWRL(IAX3,1)
      IF(.NOT.(FLAGS(29)))GOTO 10391
      WRITE(DBGFIL,10400)POSLBL
10400 FORMAT (60X,    6HPOSLBL,3G12.4)
10391 CONTINUE
      DO 10411 I=1,3
      ALONG(I)=0.
      UPDIR(I)=0.
10411 CONTINUE
10412 CONTINUE
      ALONG(IAX1)=ABS(LBLSIZ)
      UPDIR(IAX2)=1.
      IF(NUM .LE. 0)GOTO 10431
      DO 10441 ITIK=1,NUM
      II=IABS(ITICKS(ITIK))
      IF(II .EQ. 1)GOTO 10461
      CALL T2TLAB(II,VALUES(ITIK),BASE,IWID,1, VTEXT,NTEXT,VCASE,NCASE)
      POSLBL(IAX1) = SCRTCH(ITIK)
      CALL TX3TXT(POSLBL,ALONG,UPDIR,  VTEXT,NTEXT,VCASE,NCASE,1)
10461 CONTINUE
10441 CONTINUE
10442 CONTINUE
10431 CONTINUE
10371 CONTINUE
      CALL TXSCIS(1)
10221 CONTINUE
      GOTO10161
10171 IF(IXYZ .EQ. 3)GOTO 10471
      IAX1=IXYZ
      IAX2=3-IAX1
      CALL T2XFRM(3,2,XYZ,NONLIN,XFRM13,2,XYTXT)
      IF(.NOT.(FLAGS(29)))GOTO 10491
      WRITE(DBGFIL,10500)XYTXT
10500 FORMAT(60X,    6HLMTXT ,2G11.4/66X,2G11.4)
10491 CONTINUE
      IF(NUM .LE. 0)GOTO 10521
      DO 10531 I=1,NUM
      POSLBL(IXYZ)=VALUES(I)
      CALL T2XFRM(3,1,POSLBL,NONLIN,XFRM13,2,ALONG)
      SCRTCH(I)=ALONG(IXYZ)
10531 CONTINUE
10532 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10551
      WRITE(DBGFIL,10560)(SCRTCH(I),I=1,NUM)
10560 FORMAT (60X,    8HTICKS AT,99(/65X,6G10.3))
10551 CONTINUE
10521 CONTINUE
      CALL TXSCIS(2)
      IF(IAXIS .EQ. 0)GOTO 10581
      CALL T2XFRM(2,2,XYTXT,LINEAR,XFRM34,2,XY)
      CALL TXLINX(XY,2,LINTEX)
10581 CONTINUE
      IF(ITICK .EQ. 0 .OR. NUM .LE. 0)GOTO 10601
      TVAL(1)=XYTXT(IAX2,1)+TICK
      TVAL(2)=XYTXT(IAX2,1)-TICK
      TVAL(3)=XYTXT(IAX2,1)+3.*TICK
      TVAL(4)=XYTXT(IAX2,1)-3.*TICK
      DO 10611 I=1,NUM
      IF(IABS(ITICKS(I)) .GE. 5)GOTO 10631
      J=2+ISIGN(1,ITICKS(I))
      XYPNT(IAX2,1)=TVAL(J)
      XYPNT(IAX2,2)=TVAL(J+1)
      XYPNT(IAX1,1)=SCRTCH(I)
      XYPNT(IAX1,2)=XYPNT(IAX1,1)
      CALL T2XFRM(2,2,XYPNT,LINEAR,XFRM34,2,XY)
      CALL TXLINX(XY,2,LINTEX)
10631 CONTINUE
10611 CONTINUE
10612 CONTINUE
10601 CONTINUE
      CALL TXSCIS(1)
      IF(ILABEL .EQ. 0 .OR. NUM .LE. 0)GOTO 10651
      CHSIZ=0.1*REDUCE(2)*ABS(LBLSIZ)
      IFIRST=12
      LAST=1
      LFTADJ=0
      DO 10661 ITIK=1,NUM
      II=IABS(ITICKS(ITIK))
      IF(II .NE. 2)GOTO 10681
      CALL T2TLAB(II,VALUES(ITIK),BASE,IWID,LFTADJ, VTEXT,NTEXT,VCASE,NC
     *ASE)
      J=IFIRST
      DO 10691 I=1,J
      IF(VTEXT(I) .EQ. BLANK)GOTO 10711
      IFIRST=I
      GOTO 10692
10711 CONTINUE
10691 CONTINUE
10692 CONTINUE
      IF (NTEXT.GT.LAST) LAST=NTEXT
10681 CONTINUE
10661 CONTINUE
10662 CONTINUE
      NCHAR=LAST-IFIRST+1
      IF(ILABEL .NE. 1)GOTO 10731
      IF (NCHAR.GT.0) LBLCHR(2)=NCHAR
      POSX2=XYTXT(1,1)  -0.9*(FRELBL(1)+FLOAT(NCHAR-1))*CHSIZ
      LFTADJ=0
      GOTO10721
10731 IF(ILABEL .NE. 2)GOTO 10741
      POSLBL(2)=XYTXT(2,1)-(FRELBL(2)+0.2)*CHSIZ
      LFTADJ=1
      GOTO10721
10741 IF(ILABEL .NE. 3)GOTO 10751
      POSLBL(1)=XYTXT(1,2)+FRELBL(3)*CHSIZ
      LFTADJ=0
      GOTO 10761
10751 CONTINUE
      POSLBL(2)=XYTXT(2,2)+FRELBL(4)*CHSIZ
      LFTADJ=1
10761 CONTINUE
10721 CONTINUE
      DO 10771 ITIK=1,NUM
      II=IABS(ITICKS(ITIK))
      IF(II .EQ. 1)GOTO 10791
      IF(II .NE. 2)GOTO 10811
      J=LFTADJ
      GOTO 10821
10811 CONTINUE
      J=1
10821 CONTINUE
10801 CONTINUE
      CALL T2TLAB(II,VALUES(ITIK),BASE,IWID,J,  VTEXT,NTEXT,VCASE,NCASE)
      POSLBL(IXYZ)=SCRTCH(ITIK)
      IF(J .NE. 0)GOTO 10841
      IC1=IFIRST
      NTEXT=NCHAR
      GOTO 10851
10841 CONTINUE
      IC1=1
10851 CONTINUE
10831 CONTINUE
      IF(ILABEL .NE. 1)GOTO 10871
      IF(II .EQ. 2)GOTO 10891
      POSLBL(1)=XYTXT(1,1) -0.9*(FRELBL(1)+FLOAT(NTEXT-1))*CHSIZ
      GOTO 10901
10891 CONTINUE
      POSLBL(1)=POSX2
10901 CONTINUE
10881 CONTINUE
10871 CONTINUE
      IF(IXYZ .NE. 1)GOTO 10921
      POSLBL(1)=POSLBL(1)-0.45*CHSIZ*FLOAT(NTEXT-1)
10921 CONTINUE
      CALL T2XFRM(2,1,POSLBL,LINEAR,XFRM34,2,XY)
      CALL TXTEXT(XY(1,1),XY(2,1),LBLSIZ,0.,  VTEXT(IC1),NTEXT,VCASE(IC1
     *),NCASE)
10791 CONTINUE
10771 CONTINUE
10772 CONTINUE
10651 CONTINUE
10471 CONTINUE
10161 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2GRID(IGRIDS)
      INTEGER IGRIDS(4)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL XYZ(3,2),XY(2,2),ALONG(3),UP(3)
      IF(.NOT.(FLAGS(29)))GOTO 10941
      WRITE(DBGFIL,10950)IGRIDS
10950 FORMAT(60X,    6HT2GRID,6I4)
10941 CONTINUE
      MINX=IGRIDS(1)
      MAXX=IGRIDS(2)-1
      MINY=IGRIDS(2)
      MAXY=IGRIDS(3)-1
      MINZ=IGRIDS(3)
      MAXZ=IGRIDS(4)-1
      IF (.NOT.FLAGS(56)) MAXZ=MINZ
      CALL TXSCIS(2)
      IF((GRDTYP .NE. 2) .AND. (GRDTYP .NE. 4))GOTO 10971
      IF(.NOT.(FLAGS(82)))GOTO 10991
      XYZ(3,1)=ORAXES(3)
      GOTO 11001
10991 CONTINUE
      XYZ(3,1)=XYZLIM(3,1)
11001 CONTINUE
10981 CONTINUE
      XYZ(3,2)=XYZ(3,1)
      XYZ(1,1)=XYZLIM(1,1)
      XYZ(1,2)=XYZLIM(1,2)
      DO 11011 IY=MINY,MAXY
      XYZ(2,1)=BUFFER(IY)
      XYZ(2,2)=XYZ(2,1)
      CALL T2XFRM(3,2,XYZ,NONLIN,XFRM14,2,XY)
      CALL TXLINX(XY,2,LINTEX)
11011 CONTINUE
11012 CONTINUE
10971 CONTINUE
      IF((GRDTYP .NE. 3) .AND. (GRDTYP .NE. 4))GOTO 11031
      IF(.NOT.(FLAGS(82)))GOTO 11051
      XYZ(3,1)=ORAXES(3)
      GOTO 11061
11051 CONTINUE
      XYZ(3,1)=XYZLIM(3,1)
11061 CONTINUE
11041 CONTINUE
      XYZ(3,2)=XYZ(3,1)
      XYZ(2,1)=XYZLIM(2,1)
      XYZ(2,2)=XYZLIM(2,2)
      DO 11071 IX=MINX,MAXX
      XYZ(1,1)=BUFFER(IX)
      XYZ(1,2)=XYZ(1,1)
      CALL T2XFRM(3,2,XYZ,NONLIN,XFRM14,2,XY)
      CALL TXLINX(XY,2,LINTEX)
11071 CONTINUE
11072 CONTINUE
11031 CONTINUE
      IF(GRDTYP .NE. 5)GOTO 11091
      IF(.NOT.(FLAGS(56)))GOTO 11111
      DO 11121 I=1,3
      ALONG(I)=0.
      UP(I)=0.
11121 CONTINUE
11122 CONTINUE
      ALONG(1)=ABS(GRDSIZ)
      UP(2)=1.
      DO 11131 IZ=MINZ,MAXZ
      XYZ(3,1)=BUFFER(IZ)
      DO 11141 IY=MINY,MAXY
      XYZ(2,1)=BUFFER(IY)
      DO 11151 IX=MINX,MAXX
      XYZ(1,1)=BUFFER(IX)
      CALL T2XFRM(3,1,XYZ,NONLIN,XFRM12,3,XY)
      CALL TX3TXT(XY,ALONG,UP,GRDSYM,1,DUMMY,-1,0)
11151 CONTINUE
11152 CONTINUE
11141 CONTINUE
11142 CONTINUE
11131 CONTINUE
11132 CONTINUE
      GOTO 11161
11111 CONTINUE
      XYZ(3,1)=XYZLIM(3,1)
      CALL TXTEXT(0.,0.,GRDSIZ,0.,DUM,0,DUM,0)
      DO 11171 IY=MINY,MAXY
      XYZ(2,1)=BUFFER(IY)
      DO 11181 IX=MINX,MAXX
      XYZ(1,1)=BUFFER(IX)
      CALL T2XFRM(3,1,XYZ,NONLIN,XFRM14,2,XY)
      CALL TXTSYM(XY(1,1),XY(2,1),GRDSYM)
11181 CONTINUE
11182 CONTINUE
11171 CONTINUE
11172 CONTINUE
11161 CONTINUE
11101 CONTINUE
11091 CONTINUE
      CALL TXSCIS(1)
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2LADJ(IN,NIN,OUT,NOUT)
      INTEGER NIN,NOUT
cmt4       BYTE IN(NIN),OUT(NIN)
cmt4       BYTE CHAR,BLNK,ZERO,POINT
      character*1 IN(NIN),OUT(NIN)
      character*1 CHAR,BLNK,ZERO,POINT
      DATA BLNK/    1H /,ZERO/    1H0/,POINT/    1H./
      IOUT=0
      LAST=0
      DO 11191 I=1,NIN
      CHAR = IN(I)
      IF(CHAR .EQ. BLNK)GOTO 11211
      IOUT=IOUT+1
      OUT(IOUT)=CHAR
      IF(CHAR.NE.ZERO) LAST=IOUT
11211 CONTINUE
11191 CONTINUE
11192 CONTINUE
      IF (OUT(LAST).EQ.POINT) LAST=LAST-1
      NOUT=LAST
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2STCK(MSCAL,VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,T
     *YPE)
      INTEGER MSCAL,IN1,IN2,NUMBER,TYPE(MAXN)
      REAL VMIN,VMAX,PARMS(10),VALS(MAXN)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      IF(MSCAL .GE. 0)GOTO 11231
      I = -MSCAL
      IXXXXX=I
11240 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 6))GOTO 11321
      WRITE(6,11330)IXXXXX
11330 FORMAT(    4H I (,I5,   31H) OUT OF RANGE FOR CASE (1 TO6))
      CALL ERRTRA
      STOP 7777
11321 CONTINUE
      GO TO (11250,11260,11270,11280,11290,11300),IXXXXX
11250 CONTINUE
      CALL TDSLIN(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      GOTO 11242
11260 CONTINUE
      CALL TDSLOG(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      GOTO 11242
11270 CONTINUE
      CALL TDSLOG(-VMAX,-VMIN,IN1,IN2,PARMS,MAXN,  NUMBER,VALS,TYPE)
      DO 11341 I=1,NUMBER
      VALS(I)=-VALS(I)
11341 CONTINUE
11342 CONTINUE
      GOTO 11242
11290 CONTINUE
      CALL TDSYR(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      GOTO 11242
11280 CONTINUE
      CALL TDSMON(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      GOTO 11242
11300 CONTINUE
      CALL TDSNRM(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      GOTO 11242
11242 CONTINUE
      GOTO 11351
11231 CONTINUE
      CALL TDSLIN(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      IA1=MSCAL
      A2=VMIN
      A3=VMAX
      IA4=IN1
      IA5=IN2
      DO 11361 I=1,10
      BUFFER(I)=PARMS(I)
11361 CONTINUE
11362 CONTINUE
      CALL TDTICK(IA1,A2,A3,IA4,IA5,BUFFER,NUMBER,VALS,TYPE)
11351 CONTINUE
11221 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11381
      WRITE(DBGFIL,11390)(VALS(I),I=1,NUMBER)
11390 FORMAT(    7H T2STCK,9G13.6)
      WRITE(DBGFIL,11400)(TYPE(I),I=1,NUMBER)
11400 FORMAT(5X,9I13)
11381 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2TCKS
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL XYZ(3,2)
      INTEGER IBUFFR(400),IGRIDS(4)
      EQUIVALENCE (IBUFFR(1),BUFFER(1))
      IF (FLAGS(3)) RETURN
      FLAGS(3)=.TRUE.
      IF((FLAGS(75)))GOTO 12481
      CALL T2STLM
      CALL TXDEF2
12481 CONTINUE
      IF((FLAGS(75)))GOTO 12501
      WRITE(ERRFIL,12510)
12510 FORMAT (   49H *** ERROR *** DATA COORDINATE SYSTEM NOT DEFINED/
     *15X,   31HAXES CANNOT BE DRAWN   (T2TCKS))
      FLAGS(23)=.TRUE.
      GOTO 12521
12501 CONTINUE
      NB=BUFSIZ/5
      IB1=BUFSIZ-NB
      IB2=IB1-NB
      IB3=IB2-2*NB
      IT1=1
      IF (.NOT.FLAGS(2)) CALL T2AXES
      DO 12531 IXYZ=1,3
      IGRIDS(IXYZ)=IB3
      IF(XYZLIM(IXYZ,1) .EQ. XYZLIM(IXYZ,2))GOTO 12551
      CALL T2STCK(NONLIN(IXYZ), XYZLIM(IXYZ,1),XYZLIM(IXYZ,2), NXYZ1(IXY
     *Z),NXYZ2(IXYZ), SCLPRM(1,IXYZ), NB, NUM,BUFFER(IB1),IBUFFR(IT1))
      IF(GRDTYP .LE. 1)GOTO 12571
      IB=IB1
      IT=IT1
      DO 12581 I=1,NUM
      IF(IBUFFR(IT) .LE. 0)GOTO 12601
      IB3=IB3+1
      IF(IB3.GE.IB2)GOTO 12582
      BUFFER(IB3-1)=BUFFER(IB)
12601 CONTINUE
      IB=IB+1
      IT=IT+1
12581 CONTINUE
12582 CONTINUE
12571 CONTINUE
      IF((.NOT.(FLAGS(56))) .AND. (.NOT.(FLAGS(82))))GOTO 12621
      MAX=1
      GOTO 12631
12621 CONTINUE
      MAX=2
12631 CONTINUE
12611 CONTINUE
      DO 12641 I=1,MAX
      DO 12651 J=1,3
      IF((FLAGS(82)))GOTO 12671
      XYZ(J,1)=XYZLIM(J,I)
      GOTO 12681
12671 CONTINUE
      XYZ(J,1)=ORAXES(J)
12681 CONTINUE
12661 CONTINUE
      XYZ(J,2)=XYZ(J,1)
12651 CONTINUE
12652 CONTINUE
      XYZ(IXYZ,1)=XYZLIM(IXYZ,1)
      XYZ(IXYZ,2)=XYZLIM(IXYZ,2)
      IAXIS=0
      ITICK=0
      ILABEL=0
      IF(.NOT.(FLAGS(56)))GOTO 12701
      IF(.NOT.(FLAGS(IXYZ+49)))GOTO 12721
      IAXIS=1
      IF (FLAGS(IXYZ+43)) ITICK=1
      IF (FLAGS(IXYZ+46)) ILABEL=1
12721 CONTINUE
      GOTO 12731
12701 CONTINUE
      IPNT=2*I-IXYZ+1
      IF(.NOT.(FLAGS(29)))GOTO 12751
      WRITE(DBGFIL,12760)IXYZ,I,IPNT
12760 FORMAT(60X,   11HIXYZ,I,IPNT,3I3)
12751 CONTINUE
      IF(.NOT.(FLAGS(IPNT+18)) .OR. .NOT.(FLAGS(IXYZ+49)))GOTO 12781
      IF (FLAGS(82)) IAXIS=1
      IF (FLAGS(IXYZ+43).AND. FLAGS(IPNT+31)) ITICK=1
      IF (FLAGS(IXYZ+46) .AND. FLAGS(IPNT+35)) ILABEL=IPNT
12781 CONTINUE
12731 CONTINUE
12691 CONTINUE
      IF (IAXIS+ITICK+ILABEL.NE.0) CALL T2AXIS(XYZ,IXYZ,BUFFER(IB1), BUF
     *FER(IB2), NB,NUM, IBUFFR(IT1), IAXIS,ITICK,ILABEL)
12641 CONTINUE
12642 CONTINUE
12551 CONTINUE
      IF(IB3 .NE. IGRIDS(IXYZ))GOTO 12801
      IF(IB3 .GE. IB2)GOTO 12821
      IF(.NOT.(FLAGS(82)))GOTO 12841
      BUFFER(IB3)=ORAXES(IXYZ)
      GOTO 12851
12841 CONTINUE
      BUFFER(IB3)=XYZLIM(IXYZ,1)
12851 CONTINUE
12831 CONTINUE
      IB3=IB3+1
12821 CONTINUE
12801 CONTINUE
12531 CONTINUE
12532 CONTINUE
      IGRIDS(4)=IB3
      IF (GRDTYP.GT.1) CALL T2GRID(IGRIDS)
12521 CONTINUE
12491 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2TLAB(ITICK,VAL,BASE,IWID,LFTADJ,TEXT,NTEXT,CASE,NCASE
     *)
      INTEGER ITICK,IWID,LFTADJ,NTEXT,NCASE
      REAL VAL
cmt4       BYTE TEXT(12),CASE(12)
      character*1 TEXT(12),CASE(12)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
cmt4       BYTE ZERO,BLANK,LETL,LETV,PERIOD,  MONTHS(3,12),CONE,BASCHR(3),DIG
cmt4      *ITS(10)
      character*1 STRING
      character*1 ZERO,BLANK,LETL,LETV,PERIOD,  MONTHS(3,12),CONE,
     *BASCHR(3),DIGITS(10)
      DATA ZERO/    1H0/,BLANK/    1H /,LETL/    1HL/,LETV/    1HV/,  PE
     *RIOD/    1H./
      DATA BASEXX/10./,BASFAC/1./
      DATA MONTHS/1HD,1HE,1HC,1HJ,1HA,1HN,1HF,1HE,1HB,1HM,1HA,1HR,1HA,1H
     *P,1HR,1HM,1HA,1HY,1HJ,1HU,1HN,1HJ,1HU,1HL,1HA,1HU,1HG,1HS,1HE,1HP,
     *1HO,1HC,1HT,1HN,1HO,1HV/
      DATA BASCHR/1H1,1H0,1H1/
      DATA CONE/    1H1/
      DATA DIGITS/1H0,1H1,1H2,1H3,1H4,1H5,1H6,1H7,1H8,1H9/
      IF(ITICK .EQ. 1)GOTO 12871
      LEFT=LFTADJ
      DO 12881 J=1,12
      TEXT(J)=BLANK
      CASE(J)=BLANK
12881 CONTINUE
12882 CONTINUE
      NTEXT=12
      NCASE=0
      IXXXXX=ITICK-1
12890 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 6))GOTO 12971
      WRITE(6,12980)IXXXXX
12980 FORMAT(   10H ITICK-1 (,I5,   31H) OUT OF RANGE FOR CASE (1 TO6))
      CALL ERRTRA
      STOP 7777
12971 CONTINUE
      GO TO (12900,12910,12920,12930,12940,12950),IXXXXX
12900 CONTINUE
      CALL T2CRMF(VAL,12,IWID,TEXT,IFIRST,ILAST)
      GOTO 12892
12910 CONTINUE
      IF(VAL .NE. 0.)GOTO 13001
      ITICK=ISIGN(2,ITICK)
      TEXT(1)=ZERO
      NTEXT=1
      GOTO 13011
13001 CONTINUE
      IF(BASEXX .EQ. BASE)GOTO 13031
      BASEXX=BASE
      BASFAC=1./ALOG10(BASE)
      CALL T2CRMF(BASEXX,3,0,BASCHR,IFIRST,ILAST)
      BASCHR(3)=CONE
13031 CONTINUE
      DO 13041 I=1,3
      TEXT(I)=BASCHR(I)
13041 CONTINUE
13042 CONTINUE
      CASE(3)=LETV
      LEFT=4
      NCASE=3
      X=BASFAC*ALOG10(ABS(VAL))
      CALL T2CRMF(X,9,2,TEXT(4),IFIRST,ILAST)
13011 CONTINUE
12991 CONTINUE
      GOTO 12892
12920 CONTINUE
      LEFT=0
      V=ABS(VAL)
      POWER=ALOG10(V)
      IPOWER=INT(300.+POWER)-300
      I=V*10.**(-IPOWER)+0.5
      IF (I.LE.0 .OR. I.GT.9) STOP 775
      TEXT(1)=DIGITS(I+1)
      NTEXT=1
      GOTO 12892
12930 CONTINUE
      LEFT=0
      J = MOD(INT(VAL),12) + 1
      DO 13051 K=1,3
      TEXT(K) = MONTHS(K,J)
13051 CONTINUE
13052 CONTINUE
      CASE(2)=LETL
      CASE(3)=LETL
      NTEXT=3
      NCASE=3
      GOTO 12892
12950 CONTINUE
      LEFT=0
      J = MOD(INT(VAL),12) + 1
      TEXT(1) = MONTHS(1,J)
      NTEXT=1
      GOTO 12892
12940 CONTINUE
      LEFT=0
      X = VAL
      IF (X.LT.100.) X = X+1900.
      CALL T2CRMF(X,5,0,TEXT,IFIRST,ILAST)
      NTEXT=5
      GOTO 12892
12892 CONTINUE
13061 CONTINUE
      IF(TEXT(NTEXT).NE.ZERO)GOTO 13062
      NTEXT=NTEXT-1
      GOTO 13061
13062 CONTINUE
      IF(TEXT(NTEXT).EQ.PERIOD) NTEXT=NTEXT-1
      IF(LEFT .LE. 0)GOTO 13081
      DO 13091 MIN=LEFT,NTEXT
      IF(TEXT(MIN).NE.BLANK)GOTO 13092
      IF(CASE(MIN).NE.BLANK)GOTO 13092
13091 CONTINUE
      MIN=NTEXT
13092 CONTINUE
      DO 13101 I=MIN,NTEXT
      TEXT(LEFT)=TEXT(I)
      CASE(LEFT)=CASE(I)
      LEFT=LEFT+1
13101 CONTINUE
13102 CONTINUE
      NTEXT=LEFT-1
      IF (NCASE.NE.0) NCASE=NTEXT
13081 CONTINUE
12871 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSLIN(VMIN,VMAX,IN1,IN2,BASEIN,MAXN,NUMBER,VALS,TYPE)
      INTEGER NUMBER,N1,N2,TYPE(MAXN)
      REAL VMIN,VMAX,BASEIN,VALS(MAXN)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      DOUBLE PRECISION DONE,DTEN,DBASE,DROUND,DX2,DSTEP2
      REAL ROUND(7)
      INTEGER*4 I2
      DATA NROUND/7/
      DATA ROUND/1.,2.,2.5,5.,10.,20.,1000000./
      DONE = 1.
      DTEN = 10.
      BASE = BASEIN
      IF (BASE.LE.0.) BASE=10.
      VDIF = VMAX-VMIN
      SMALL = 5.0E-4 * VDIF
      XVMIN = VMIN-SMALL
      XVMAX = VMAX+SMALL
      S = SIGN(1.,VDIF)
      N1 = IN1
      N2 = IN2
13110 CONTINUE
      IF (N1.EQ.0) N1= 6
      ITYPE1=ISIGN(2,N1)
      IF (N2.EQ.0) N2=-5
      ITYPE2=ISIGN(1,N2)
      N1=IABS(N1)
      N2=IABS(N2)
      PSTEP = ABS(VDIF)/(FLOAT(N1)*BASE)
      JORDER = ALOG10(PSTEP)
      IF (PSTEP.LT.1.0) JORDER=JORDER-1
      PSTEP = PSTEP/10.**JORDER
      I=0
13121 CONTINUE
      I=I+1
      IF(ROUND(I) .LT. PSTEP)GOTO 13121
13122 CONTINUE
      IF (I.EQ.NROUND) STOP 774
      DBASE = BASE
      DROUND = ROUND(I)
      DX2 = N2
      DSTEP2 = DBASE*DROUND*(DTEN**JORDER)/DX2
      IF (VDIF.LT.0.) DSTEP2 = -DSTEP2
      I2 = XVMIN/DSTEP2
13131 CONTINUE
      DX2=I2
      VAL=DX2*DSTEP2
      IF(SIGN(1.,VAL-XVMIN).EQ.S)GOTO 13132
      I2=I2+1
      GOTO 13131
13132 CONTINUE
      IF((IABS(I2) .LE. 999 999) .AND. (ABS(VAL) .LE. 1.E10))GOTO 13151
      WRITE(ERRFIL,13160)
13160 FORMAT (   39H *** WARNING *** DATA RANGE MAY EXCEED ,     38HAVAI
     *LABLE PRECISION. CHECK YOUR INPUT.)
      FLAGS(23)=.TRUE.
13151 CONTINUE
      I2 = N2-MOD(I2,N2)
      IF (I2.GE.N2) I2 = I2-N2
      NUMBER = 0
13171 CONTINUE
      VAL = DX2*DSTEP2
      IF(SIGN(1.,VAL-XVMAX).EQ.S)GOTO 13172
      IF(NUMBER .LT. MAXN)GOTO 13191
      WRITE(ERRFIL,13200)
13200 FORMAT (   50H *** ERROR *** TOO MANY TICKS. DEFAULT N1,N2 USED.)
      FLAGS(23)=.TRUE.
      N1=0
      N2=0
      GO TO 13110
13191 CONTINUE
      NUMBER = NUMBER+1
      VALS(NUMBER) = VAL
      IF(I2 .NE. 0)GOTO 13221
      I2 = N2
      TYPE(NUMBER) = ITYPE1
      GOTO 13231
13221 CONTINUE
      TYPE(NUMBER) = ITYPE2
13231 CONTINUE
13211 CONTINUE
      DX2 = DX2+DONE
      I2 = I2-1
      GOTO 13171
13172 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSLOG(VMIN,VMAX,IN1,IN2,BASE,MAXN,NUMBER,VALS,TYPE)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL VMIN,VMAX,VALS(MAXN)
      INTEGER IN1,IN2,TYPE(MAXN)
      INTEGER N1LIST(8),N2LIST(8),LIST1(9),LIST2(9)
      DATA N1LIST/2,32,-1,-1,-1,-1,-1,-1/
      DATA N2LIST/-2,-2,-2,-2,-2,-2,-2,-1/
      BFAC=1./ALOG10(BASE)
      N1 = IN1
      N2 = IN2
      IF (SIGN(1.,VMIN).NE.SIGN(1.,VMAX)) GO TO 13240
      WMAX=ABS(VMAX)
      WMIN=ABS(VMIN)
      IF(WMIN .LE. WMAX)GOTO 13261
      WMIN=WMAX
      WMAX=ABS(VMIN)
13261 CONTINUE
      IF (WMIN.EQ.0.) GO TO 13240
      FAC = (WMAX/WMIN)**0.001
      WMAX=WMAX*FAC
      WMIN=WMIN/FAC
      IF((N1 .NE. 0) .AND. (N2 .NE. 0))GOTO 13281
      IDEC = INT(0.5 + ALOG10(WMAX/WMIN)*SQRT(BFAC))
      IF (IDEC.LE.0) GO TO 13240
      IF(IDEC .LE. 8)GOTO 13301
      IF(N1 .NE. 0)GOTO 13321
      N1 = -3
      N2 = IDEC/5
13321 CONTINUE
      IF (N2.EQ.0) N2=-1
      GOTO 13331
13301 CONTINUE
      IF (N1.EQ.0) N1=N1LIST(IDEC)
      IF (N2.EQ.0) N2=N2LIST(IDEC)
      IF (WMAX.GE.1.E6 .OR. WMIN.LT.1.E-5) N1=-1
13331 CONTINUE
13291 CONTINUE
13281 CONTINUE
      ORDER=ALOG10(WMIN)*BFAC
      MINORD=INT(ORDER)
      IF (FLOAT(MINORD).GT.ORDER) MINORD=MINORD-1
      ORDER=ALOG10(WMAX)*BFAC
      MAXORD=INT(ORDER)
      IF (FLOAT(MAXORD).LT.ORDER) MAXORD=MAXORD+1
      CALL TDSLG1(N1,LIST1)
      CALL TDSLG1(N2,LIST2)
      IT1=2
      IT2=-1
      IF(N1 .GE. 0)GOTO 13351
      IT1=4
      IT2=-1
13351 CONTINUE
      IT1 = ISIGN(IT1,IT2)
      IMAX=INT(BASE)-1
      IF(N1 .LE. 0)GOTO 13371
      ITDEC=2
      GOTO 13381
13371 CONTINUE
      ITDEC=3
13381 CONTINUE
13361 CONTINUE
      IF(N1 .NE. (-3))GOTO 13401
      MOVORD=IABS(N2)+1
      IF (MINORD.LT.0) MINORD = MINORD - (MOVORD-1)
      MINORD = MOVORD*(MINORD/MOVORD)
      IF (MAXORD.GT.0) MAXORD = MAXORD + (MOVORD-1)
      MAXORD = MOVORD*(MAXORD/MOVORD)
      IMAX=1
      GOTO 13411
13401 CONTINUE
      MOVORD=1
13411 CONTINUE
13391 CONTINUE
      NUMBER=0
      DO 13421 JORDER=MINORD,MAXORD,MOVORD
      POWER=BASE**JORDER
      IF(.NOT.((SIGN(1.,WMIN-POWER).EQ.SIGN(1.,POWER-WMAX))))GOTO 13441
      IF (NUMBER.GE.MAXN) GO TO 13450
      NUMBER=NUMBER+1
      VALS(NUMBER)=POWER
      TYPE(NUMBER)=ITDEC
13441 CONTINUE
      IF(MOVORD .LE. 1)GOTO 13471
      DO 13481 I=2,MOVORD
      POWER = POWER*BASE
      IF(.NOT.((SIGN(1.,WMIN-POWER).EQ.SIGN(1.,POWER-WMAX))))GOTO 13501
      IF (NUMBER.GE.MAXN) GO TO 13450
      NUMBER=NUMBER+1
      VALS(NUMBER)=POWER
      TYPE(NUMBER)=IT2
13501 CONTINUE
13481 CONTINUE
13482 CONTINUE
13471 CONTINUE
      IF(IMAX .LT. 2)GOTO 13521
      DO 13531 I=2,IMAX
      VALUE = POWER*FLOAT(I)
      IF(.NOT.((SIGN(1.,WMIN-VALUE).EQ.SIGN(1.,VALUE-WMAX))))GOTO 13551
      IT = 0
      IF (LIST2(I).NE.0) IT = IT2
      IF (LIST1(I).NE.0) IT = IT1
      IF(IT .EQ. 0)GOTO 13571
      IF (NUMBER.GE.MAXN) GO TO 13450
      NUMBER=NUMBER+1
      VALS(NUMBER)=VALUE
      TYPE(NUMBER)=IT
13571 CONTINUE
13551 CONTINUE
13531 CONTINUE
13532 CONTINUE
13521 CONTINUE
13421 CONTINUE
13422 CONTINUE
      GO TO 13580
13450 CONTINUE
      WRITE(ERRFIL,13590)
13590 FORMAT(   29H *** ERROR *** TOO MANY TICKS)
      FLAGS(23)=.TRUE.
13580 CONTINUE
      IF(WMAX .GE. 0.)GOTO 13611
      DO 13621 I=1,NUMBER
      VALS(I)=-VALS(I)
13621 CONTINUE
13622 CONTINUE
13611 CONTINUE
      RETURN
13240 CONTINUE
      CALL TDSLIN(VMIN,VMAX,0,0,BASE,MAXN,NUMBER,VALS,TYPE)
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSLG1(N,LIST)
      DIMENSION LIST(9)
      NA=IABS(N)
      IF (NA.EQ.2) NA=1023
      J=NA/4
      DO 13631 I=2,9
      JJ=J/2
      LIST(I)=J-JJ-JJ
      J=JJ
13631 CONTINUE
13632 CONTINUE
      IF(NA .NE. 3)GOTO 13651
      LIST(1)=0
      GOTO 13661
13651 CONTINUE
      LIST(1)=1
13661 CONTINUE
13641 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSMON(VMIN,VMAX,IN1,IN2,BASE,MAXN,NUMBER,VALS,TYPE)
ccccccccccccccccc 26-dec-1995 Modified by Y.Teramoto
      REAL VMIN,VMAX,VALS(*)
      INTEGER IN1,IN2,TYPE(*)
ccc      REAL VMIN,VMAX,VALS(NUMBER)
ccc      INTEGER IN1,IN2,TYPE(NUMBER)
ccccccccccccccccc end of 26-dec-1995 Modification.
      NUMBER = 0
      IVMX = AMAX1(VMAX,VMIN) - 0.145
      IVMN = MIN1(VMAX,VMIN)
      IVST = (2*(IVMX-IVMN))/MAXN + 1
      IF(IVST .LE. 1 .OR. IVST .GT. 12)GOTO 13681
13691 CONTINUE
      IF(MOD(12,IVST).EQ.0)GOTO 13692
      IVST=IVST+1
      GOTO 13691
13692 CONTINUE
      IVMN=IVST*((IVMN-1)/IVST)+1
13681 CONTINUE
      IF(IN1 .NE. 0)GOTO 13711
      N1=1
      IF(IVMX-IVMN .GE. 13)GOTO 13731
      LTYPE=5
      GOTO 13741
13731 CONTINUE
      LTYPE=7
13741 CONTINUE
13721 CONTINUE
      GOTO 13751
13711 CONTINUE
      LTYPE=6-ISIGN(1,IN1)
      N1=IABS(IN1)
13751 CONTINUE
13701 CONTINUE
      IF(IN2 .NE. 0)GOTO 13771
      N2=6
      GOTO 13781
13771 CONTINUE
      N2=IABS(IN2)
13781 CONTINUE
13761 CONTINUE
      DO 13791 IV=IVMN,IVMX,IVST
      IF(NUMBER.GE.MAXN)GOTO 13792
      NUMBER = NUMBER + 1
      V = FLOAT(IV)+0.005
      VALS(NUMBER) = V
      IF(MOD(IV-1,N2) .NE. 0)GOTO 13811
      TYPE(NUMBER) = 1
      GOTO 13821
13811 CONTINUE
      TYPE(NUMBER) = -1
13821 CONTINUE
13801 CONTINUE
      IF(MOD(IV-1,N1) .NE. 0)GOTO 13841
      NUMBER = NUMBER + 1
      VALS(NUMBER) = V+0.155
      TYPE(NUMBER) = LTYPE
13841 CONTINUE
13791 CONTINUE
13792 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSNRM(VMIN,VMAX,IN1,IN2,PARMS,MAXN,NUMBER,VALS,TYPE)
      REAL VMIN,VMAX,PARMS(10),VALS(NUMBER)
      INTEGER IN1,IN2,TYPE(NUMBER)
      REAL TABLE(10),MEAN,SUGMA
      DOUBLE PRECISION DTEMP,DPOWER,DTEN,DTENTH,DVAL
      DATA DTEN/10.D0/,DTENTH/0.1D0/
      DATA TABLE /0.,.125,.25,.38,.52,.67,.84,1.04,1.28,1.64/
      N1=IABS(IN1)
      IF (N1.EQ.0) N1=6
      MEAN=PARMS(2)
      SIGMA=ABS(PARMS(3))
      IF (SIGMA.EQ.0.) SIGMA=1.
      VMN=(VMIN-MEAN)/SIGMA
      VMX=(VMAX-MEAN)/SIGMA
      IF(VMX .GE. VMN)GOTO 13861
      T=VMX
      VMX=VMN
      VMN=T
13861 CONTINUE
      DO 13871 I=1,10
      IF (ABS(VMN).GE.TABLE(I)) NINTMN=I-1
      IF (ABS(VMX).GE.TABLE(I)) NINTMX=I-1
13871 CONTINUE
13872 CONTINUE
      IF (VMN.LT.0.0) NINTMN=-NINTMN
      IF (VMX.LT.0.0) NINTMX=-NINTMX
      NSTEP = (NINTMX-NINTMN)/N1 + 1
      NINT=NINTMN
      IF (NINT*NINTMX.LT.0) NINT=NSTEP*(NINT/NSTEP)-NSTEP/2
      IF (NINT.LE.-10) NINT=NINT+NSTEP
      NUM=MAXN
      VALS(NUM)=VMN
      VX=VMN
13881 CONTINUE
      VN=VX
      IF(NINT .GT. NINTMX)GOTO 13901
      I=IABS(NINT)
      VX=TABLE(I+1)
      IF (NINT.LT.0) VX=-VX
      VX=VX*SIGMA+MEAN
      GOTO 13911
13901 CONTINUE
      IF(NINT.GE.9999)GOTO 13882
      NINT=9999
      VX=VMX
13911 CONTINUE
13891 CONTINUE
      VAL=0.
      DVAL=0.
      HALF=0.5*(VN+VX)
      IF(VN .EQ. 0.0)GOTO 13931
      L=ALOG10(ABS(VN))
      DPOWER=DTEN**L
      DO 13941 I=1,6
      VAL=DVAL
      POWER=DPOWER
      J=(HALF-VAL)/POWER
      DTEMP=J
      DVAL=DVAL+DPOWER*DTEMP
      VAL=DVAL
      IF (VAL.LT.VN) VAL=VAL+POWER
      IF(VAL.LT.VX)GOTO 13942
      DPOWER=DPOWER*DTENTH
13941 CONTINUE
13942 CONTINUE
13931 CONTINUE
      NINT=NINT+NSTEP
      NUM=NUM-1
      VALS(NUM)=VAL
      TYPE(NUM)=2
      GOTO 13881
13882 CONTINUE
      NUMBER=0
13950 CONTINUE
      DO 13951 NNN=NUM,MAXN
      DIFF=VMX-VALS(NNN)
      VMX=VALS(NNN)
      IF(DIFF .GT. 0.0)GOTO 13971
      XN=1.
      N=1
      GOTO 13981
13971 CONTINUE
      L=ALOG10(DIFF)
13991 CONTINUE
      STEP=10.**L
      N=DIFF/STEP
      IF(N.GE.2)GOTO 13992
      L=L-1
      GOTO 13991
13992 CONTINUE
      XN=N
      IF(STEP*XN .LT. DIFF)GOTO 14011
      XN=XN-1.
      N=N-1
14011 CONTINUE
13981 CONTINUE
13961 CONTINUE
      DO 14021 I=1,N
      IF(NUMBER.GE.NNN)GOTO 13952
      NUMBER=NUMBER+1
      XN=XN-1.
      VALS(NUMBER)=VMX+XN*STEP
      TYPE(NUMBER)=-1
14021 CONTINUE
14022 CONTINUE
      TYPE(NUMBER)=2
13951 CONTINUE
13952 CONTINUE
      TYPE(NUMBER)=-1
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDSYR(VMIN,VMAX,IN1,IN2,BASE,MAXN,NUMBER,VALS,TYPE)
cccccccccccccccc 26-dec-1995 Modified by Y.Teramoto
      REAL VMIN,VMAX,VALS(*)
      INTEGER IN1,IN2,TYPE(*)
ccc      REAL VMIN,VMAX,VALS(NUMBER)
ccc      INTEGER IN1,IN2,TYPE(NUMBER)
cccccccccccccccc end of 26-dec-1995 modification.
      REAL HATH(12)
      DATA HATH/0.,.031,.059,.090,.120,.151,.181,.212,.242,.273,.303,.33
     *4/
      NUMBER = 0
      VMX = AMAX1(VMAX,VMIN)
      VMN = AMIN1(VMAX,VMIN)
      J=IABS(IN1)
      IF (J.EQ.0) J=6
      VM = AMAX1(1.0, AINT((VMX-VMN)/FLOAT(J)) )
      V = AINT(VMN) + 0.183 - VM
      IF (V.LT.VMN) V=V+VM
14031 IF(V.GT.VMX) GOTO 14032
      NUMBER = NUMBER + 1
      VALS(NUMBER) = V
      TYPE(NUMBER) = 6
      V = V + VM
      GOTO 14031
14032 CONTINUE
      INC=IABS(IN2)
      IF(INC .NE. 0)GOTO 14051
      INC=12
      V=VMX-VMN
      IF (V.LT.15.) INC=6
      IF (V.LT.4.) INC=1
14051 CONTINUE
      VM=AINT(VMN)
14060 CONTINUE
14061 CONTINUE
      J=1
      DO 14071 I=1,12,INC
      V=VM+HATH(I)
      IF(V.GT.VMX)GOTO 14062
      IF(V .LT. VMN)GOTO 14091
      NUMBER=NUMBER+1
      TYPE(NUMBER)=J
      VALS(NUMBER)=V
14091 CONTINUE
      J=-1
14071 CONTINUE
14072 CONTINUE
      VM=VM+1.
      GOTO 14061
14062 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE TDTICK(ISCAL,VMIN,VMAX,N1,N2,BASE,NUMBER,VALS,ITYPE)
      INTEGER ISCAL,I1,I2,NUMBER,ITYPE(NUMBER)
      REAL VMIN,VMAX,BASE,VALS(NUMBER)
      RETURN
      END
cnext ****************************************
      FUNCTION TDFNCT(VALUE,NUMBER)
      TDFNCT=VALUE
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
C                       N O T I C E
C \
C This  program and  associated  documentation,  and  the
C material and  data contained  therein,  were  developed
C under the sponsorship of the U. S. government.  Neither
C the U.S.  nor the U.S.D.O.E.,   nor the Leland Stanford
C Junior  University,  nor  their  employees,  nor  their
C respective  contractors,   subcontractors,    or  their
C employees, even unto the seventh generation,  makes any
C warranty, express or implied,  or assumes any liability
C or  responsibility   for  accuracy,    completeness  or
C usefulness of any information,   apparatus,  product or
C process disclosed,  or represents that its use will not
C infringe  privately-owned  rights.     Mention  of  any
C product, its manufacturer, or suppliers shall not,  nor
C is it  intended to,  imply approval,   disapproval,  or
C fitness for any particular use.   A royalty-free,  non-
C exclusive right  to use  and disseminate  same for  any
C purpose whatsoever  is expressly  reserved to  the U.S.
C and the University.
cnext ****************************************
      SUBROUTINE T2MAIN(IARGUE)
      COMMON /T2NPMX/ NPTMAX
      COMMON /T2XPNT/ DATBUF(5125)
      INTEGER NPTMAX
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL AROXYZ(3,2),AROD(4),LNAMES(7)
      INTEGER AROMOD(2)
      LOGICAL FSAVE
cmt4       BYTE BLANK
      character*1 BLANK
      DATA HNONE/    4HNONE/,BLANK /    1H /
      DATA LNAMES/    2HSM,    2HX ,    2HDX,    2HY ,    2HDY,    2HZ ,
     *    2HDZ/
cccccccccccccccccc 25-Dec-1992 Modified by Y.Teramoto
	info( 1 ) = 0
cccccccccccccccccc end of modification.
10010 CONTINUE
10011 CONTINUE
      CALL TOKRED(INFO,CARD,LENCRD,INPFIL,INPFMT)
      IF(.NOT.((INFO(1).LE.0)))GOTO 10031
      ISTOP=1
      GOTO 10012
10031 CONTINUE
      IF(.NOT.(FLAGS(16)))GOTO 10051
      GO TO 10061
10090 CONTINUE
      ISTOP=INTEG
      GO TO 10062
10080 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10070 GO TO ( 10090,10090,10090,10090,10090,10090,10065),KEYORD
10065 CONTINUE
      IF(.TRUE.) GOTO 10011
      GO TO 10062
10061 CALL TOKEN(INFO,CARD,   48HNEWPLOT:0,NEWFRAME:0,NEW:0,RETURN,STOP:
     *1,END:1,;)
      GOTO(10065,10070,10065,10065,10065,10065,10065,10080,10065),INTERP
10062 CONTINUE
      GOTO 10012
10051 CONTINUE
      CALL T2ERR(INFO,INFO,CARD,OUTFIL,6)
      IF(.NOT.(FLAGS(29)))GOTO 10111
      WRITE(DBGFIL,10120)INFO
10120 FORMAT(60X,4HINFO,10I5)
10111 CONTINUE
      GO TO 10131
10140 CONTINUE
      CALL T2RDPT
      GO TO 10132
10150 CONTINUE
      GO TO 10132
10160 CONTINUE
      INFO(3)=INFO(4)
      CALL T2SETC(INFO,CARD)
      GO TO 10132
10190 CONTINUE
      AROD(3) = ASIZE
      AROD(4) = AFLARE
      GO TO 10201
10230 CONTINUE
      I = INTEG
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10251
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10261
10251 CONTINUE
      AROD(I)=FLOTNG
10261 CONTINUE
10241 CONTINUE
      GO TO 10201
10270 CONTINUE
      IFT = INTEG
      AROD(IFT) = 0.
      AROMOD(IFT) = 0
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10291
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10301
10291 CONTINUE
      AROXYZ(1,IFT)=FLOTNG
10301 CONTINUE
10281 CONTINUE
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10321
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10331
10321 CONTINUE
      AROXYZ(2,IFT)=FLOTNG
10331 CONTINUE
10311 CONTINUE
      GO TO 10341
10350 CONTINUE
      AROXYZ(3,IFT)=FLOTNG
      AROMOD(IFT) = 1
      GO TO 10342
10345 CONTINUE
      INFO(3)=INFO(4)
      AROXYZ(3,IFT) = 0.
      GO TO 10342
10341 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(10345,10345,10350,10350,10345,10345,10345,10345,10345),INTERP
10342 CONTINUE
      GO TO 10361
10390 CONTINUE
      AROMOD(IFT) = 1
      GO TO 10361
10400 CONTINUE
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10421
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10431
10421 CONTINUE
      AROD(IFT) = FLOTNG
10431 CONTINUE
10411 CONTINUE
      GO TO 10361
10380 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10370 GO TO ( 10390,10400,10365),KEYORD
10365 CONTINUE
      INFO(3)=INFO(4)
      IF(.TRUE.)GOTO 10362
      GO TO 10361
10361 CALL TOKEN(INFO,CARD,   11HDATA,LESS,;)
      GOTO(10362,10370,10365,10365,10365,10365,10365,10380,10365),INTERP
10362 CONTINUE
      GO TO 10201
10201 CALL TOKEN(INFO,CARD,   36HSIZE:3,FLAIR:4,FLARE:4,FROM:1,TO:2,;)
      GO TO (10202,10207,10205,10205,10205,10205,10205,10206, 10205),INT
     *ERP
10206 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10207 GO TO ( 10230,10230,10230,10270,10270,10205),KEYORD
10205 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 10201
10202 CONTINUE
      CALL T2ARRO(AROXYZ,AROMOD,AROD)
      GO TO 10132
10440 CONTINUE
      AROMOD(2) = INTEG
      AROMOD(1) = 0
      AROD(1) = CIRSIZ(1,INTEG)
      AROD(2) = CIRSIZ(2,INTEG)
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10461
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10471
10461 CONTINUE
      AROXYZ(1,1)=FLOTNG
10471 CONTINUE
10451 CONTINUE
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10491
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10501
10491 CONTINUE
      AROXYZ(2,1)=FLOTNG
10501 CONTINUE
10481 CONTINUE
      GO TO 10511
10520 CONTINUE
      AROXYZ(3,1)=FLOTNG
      AROMOD(1) = 1
      GO TO 10512
10515 CONTINUE
      INFO(3)=INFO(4)
      AROXYZ(3,1) = 0.
      GO TO 10512
10511 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(10515,10515,10520,10520,10515,10515,10515,10515,10515),INTERP
10512 CONTINUE
      GO TO 10531
10560 CONTINUE
      AROMOD(1) = 1
      GO TO 10531
10570 CONTINUE
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10591
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 10601
10591 CONTINUE
      AROD(1)=FLOTNG
      AROD(2)=FLOTNG
10601 CONTINUE
10581 CONTINUE
      GO TO 10611
10620 CONTINUE
      AROD(2)=FLOTNG
      GO TO 10612
10615 CONTINUE
      INFO(3)=INFO(4)
      GO TO 10612
10611 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(10615,10615,10620,10620,10615,10615,10615,10615,10615),INTERP
10612 CONTINUE
      GO TO 10531
10531 CALL TOKEN(INFO,CARD,   11HDATA,SIZE,;)
      GO TO (10532,10537,10535,10535,10535,10535,10535,10536, 10535),INT
     *ERP
10536 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10537 GO TO ( 10560,10570,10535),KEYORD
10535 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 10531
10532 CONTINUE
      CALL T2CRCL(AROXYZ,AROMOD,AROD)
      GO TO 10132
10630 CONTINUE
      GO TO 10641
10670 CONTINUE
      WRITE(OUTFIL,10680)FLAGS
10680 FORMAT(4(1X,5L1))
      GO TO 10641
10690 CONTINUE
      CALL PDUMP(DEVICE,BUFFER,0)
      GO TO 10641
10641 CALL TOKEN(INFO,CARD,   14HFLAGS,COMMON,;)
      GO TO (10642,10647,10645,10645,10645,10645,10645,10646, 10645),INT
     *ERP
10646 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10647 GO TO ( 10670,10690,10645),KEYORD
10645 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 10641
10642 CONTINUE
      GO TO 10132
10700 CONTINUE
      CALL T2FREQ
      GO TO 10132
10710 CONTINUE
      ITXTUR = 0
      JTYPE = 0
      LEVEL = 0
      IXYZ = 0
      N1 = 1
      N2 = NPOINT
      GO TO 10721
10750 CONTINUE
      IXYZ=INTEG
      GO TO 10721
10760 CONTINUE
      FLAGS(31) = .TRUE.
      GO TO 10721
10770 CONTINUE
      JTYPE = INTEG
      GO TO 10721
10780 CONTINUE
      ITXTUR=256
      GO TO 10721
10790 CONTINUE
      ITXTUR=IOR(ITXTUR,INTEG)
      GO TO 10721
10800 CONTINUE
      ITXTUR=192
      GO TO 10721
10810 CONTINUE
      ITXTUR=320
      GO TO 10721
10820 CONTINUE
      ITXTUR=384
      GO TO 10721
10830 CONTINUE
      ITXTUR=448
      GO TO 10721
10840 CONTINUE
      CALL T2PNTS(N1,N2)
      GO TO 10721
10740 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10730 GO TO ( 10750,10750,10750,10760,10770,10770,10780,10790,10790,1079
     *0,10790,10800,10800,10810,10820,10830,10840,10725),KEYORD
10850 CONTINUE
      LEVEL=INTEG
      GO TO 10721
10721 CALL TOKEN(INFO,CARD,  124HX:1,Y:2,Z:3,TEXT,SPLINE:1,GENERAL:2,SOL
     *ID,DASHES:128,DA:128,DOT:64,DOTS:64,DOT-DASH,DOTDASH,FUNNY,PATTERN
     *ED,DAASHES,POINTS,;)
      GOTO(10722,10730,10850,10850,10725,10725,10725,10740,10725),INTERP
10725 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 10721
10722 CONTINUE
      IF(IBLKTP .NE. 1)GOTO 10871
      I1=IVRPTR(2)
      I2=IVRPTR(4)
      I3=IVRPTR(6)
      IF(N1 .EQ. 1)GOTO 10891
      I=NINCR*(N1-1)
      IF (DATBUF(I1).NE.HNONE) I1=I1+I
      IF (DATBUF(I2).NE.HNONE) I2=I2+I
      IF (DATBUF(I3).NE.HNONE) I3=I3+I
10891 CONTINUE
      CALL T2JOIN(DATBUF(I1),DATBUF(I2),DATBUF(I3),  ITXTUR+JTYPE,LEVEL,
     *N2-N1+1,NINCR)
      GOTO 10901
10871 CONTINUE
      I=IVRPTR(8)
      CALL T23JIN(DATBUF(I),NMESH1,NMESH2,IXYZ)
10901 CONTINUE
10861 CONTINUE
      FLAGS(31) = .FALSE.
      GO TO 10132
10910 CONTINUE
      N1=1
      N2=NPOINT
      GO TO 10921
10950 CONTINUE
      CALL T2PNTS(N1,N2)
      GO TO 10921
10921 CALL TOKEN(INFO,CARD,    8HPOINTS,;)
      GO TO (10922,10927,10925,10925,10925,10925,10925,10926, 10925),INT
     *ERP
10926 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10927 GO TO ( 10950,10925),KEYORD
10925 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 10921
10922 CONTINUE
      MIN=1
      IF (IVARBL(1).EQ.1) MIN=2
      DO 10961 I=MIN,NINCR
      K=IVARBL(I)
      BUFFER(I)=LNAMES(K)
10961 CONTINUE
10962 CONTINUE
      WRITE(OUTFIL,10970)(BUFFER(I),I=MIN,NINCR)
10970 FORMAT(12(8X,A2))
      MAX=LISTPT+NHEADR+(N1-1)*NINCR-1
      DO 10981 I=N1,N2
      M=MAX+MIN
      MAX=MAX+NINCR
      WRITE(OUTFIL,10990)(DATBUF(J),J=M,MAX)
10990 FORMAT(6X,12G10.4)
10981 CONTINUE
10982 CONTINUE
      GO TO 10132
11000 CONTINUE
      N1 = 1
      N2 = NPOINT
      GO TO 11011
11040 CONTINUE
      FSAVE=FLAGS(82)
      GO TO 11051
11080 CONTINUE
      FLAGS(82)=.TRUE.
      DO 11091 I=1,3
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11111
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 11121
11111 CONTINUE
      ORAXES(I)=FLOTNG
11121 CONTINUE
11101 CONTINUE
11091 CONTINUE
11092 CONTINUE
      GO TO 11052
11070 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11060 GO TO ( 11080,11055),KEYORD
11055 CONTINUE
      INFO(3)=INFO(4)
      GO TO 11052
11051 CALL TOKEN(INFO,CARD,    4HAT,;)
      GOTO(11055,11060,11055,11055,11055,11055,11055,11070,11055),INTERP
11052 CONTINUE
      FLAGS(3) = .FALSE.
      CALL T2TCKS
      FLAGS(82)=FSAVE
      N2 = N1-1
      GO TO 11012
11130 CONTINUE
      CALL T2PNTS(N1,N2)
      GO TO 11012
11030 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11020 GO TO ( 11040,11040,11130,11015),KEYORD
11015 CONTINUE
      INFO(3)=INFO(4)
      GO TO 11012
11011 CALL TOKEN(INFO,CARD,   22HAXIS:1,AXES:1,POINTS,;)
      GOTO(11015,11020,11015,11015,11015,11015,11015,11030,11015),INTERP
11012 CONTINUE
      I1=IVRPTR(2)
      I2=IVRPTR(4)
      I3=IVRPTR(6)
      I4=IVRPTR(3)
      I5=IVRPTR(5)
      I6=IVRPTR(7)
      I7=IVRPTR(1)
      IF(N1 .EQ. 1)GOTO 11151
      I=NINCR*(N1-1)
      IF (DATBUF(I1).NE.HNONE) I1=I1+I
      IF (DATBUF(I2).NE.HNONE) I2=I2+I
      IF (DATBUF(I3).NE.HNONE) I3=I3+I
      IF (DATBUF(I4).NE.HNONE) I4=I4+I
      IF (DATBUF(I5).NE.HNONE) I5=I5+I
      IF (DATBUF(I6).NE.HNONE) I6=I6+I
      IF (DATBUF(I7).NE.HNONE) I7=I7+I
11151 CONTINUE
      IF (N2.GE.N1) CALL T2PLOT(DATBUF(I1),DATBUF(I2),DATBUF(I3), DATBUF
     *(I4),DATBUF(I5),DATBUF(I6), DATBUF(I7),N2-N1+1,NINCR)
      GO TO 10132
11160 CONTINUE
      LEVEL = INTEG
      IXYZ = 0
      ITXTUR = 0
      N1 = 1
      N2 = NPOINT
      GO TO 11171
11200 CONTINUE
      IXYZ=INTEG
      GO TO 11171
11210 CONTINUE
      ITXTUR=256
      GO TO 11171
11220 CONTINUE
      ITXTUR=IOR(ITXTUR,INTEG)
      GO TO 11171
11230 CONTINUE
      ITXTUR=192
      GO TO 11171
11240 CONTINUE
      ITXTUR=320
      GO TO 11171
11250 CONTINUE
      ITXTUR=384
      GO TO 11171
11260 CONTINUE
      ITXTUR=448
      GO TO 11171
11270 CONTINUE
      CALL T2PNTS(N1,N2)
      GO TO 11171
11171 CALL TOKEN(INFO,CARD,  100HX:1,Y:2,Z:3,SOLID,DASHES:128,DA:128,DOT
     *:64,DOTS:64,DOT-DASH,DOTDASH,FUNNY,PATTERNED,DAASHES,POINTS,;)
      GO TO (11172,11177,11175,11175,11175,11175,11175,11176, 11175),INT
     *ERP
11176 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11177 GO TO ( 11200,11200,11200,11210,11220,11220,11220,11220,11230,1123
     *0,11240,11250,11260,11270,11175),KEYORD
11175 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 11171
11172 CONTINUE
      IF(IBLKTP .NE. 1)GOTO 11291
      I1=IVRPTR(2)
      I2=IVRPTR(4)
      I3=IVRPTR(6)
      I4=IVRPTR(3)
      I5=IVRPTR(5)
      I6=IVRPTR(7)
      IF(N1 .EQ. 1)GOTO 11311
      I=NINCR*(N1-1)
      IF (DATBUF(I1).NE.HNONE) I1=I1+I
      IF (DATBUF(I2).NE.HNONE) I2=I2+I
      IF (DATBUF(I3).NE.HNONE) I3=I3+I
      IF (DATBUF(I4).NE.HNONE) I4=I4+I
      IF (DATBUF(I5).NE.HNONE) I5=I5+I
      IF (DATBUF(I6).NE.HNONE) I6=I6+I
11311 CONTINUE
      CALL T2HIST(DATBUF(I1),DATBUF(I2),DATBUF(I3),  DATBUF(I4),DATBUF(I
     *5),DATBUF(I6),  ITXTUR,LEVEL,N2-N1+1,NINCR)
      GOTO 11321
11291 CONTINUE
      I=IVRPTR(8)
      CALL T23HST(DATBUF(I),NMESH1,NMESH2,IXYZ)
11321 CONTINUE
11281 CONTINUE
      GO TO 10132
11330 CONTINUE
      IF(.NOT.((.NOT.FLAGS(28).AND.NPOINT.NE.0)))GOTO 11351
      I1=IVRPTR(2)
      I2=IVRPTR(4)
      I3=IVRPTR(6)
      I4=IVRPTR(3)
      I5=IVRPTR(5)
      I6=IVRPTR(7)
      I7=IVRPTR(1)
      CALL T2PLOT(DATBUF(I1),DATBUF(I2), DATBUF(I3),DATBUF(I4),DATBUF(I5
     *),DATBUF(I6), DATBUF(I7),NPOINT,NINCR)
11351 CONTINUE
      CALL TOKEN(INFO,CARD,   11HPLOT,FRAME;)
      IF(INTERP .EQ. 2)GOTO 11371
      INFO(3)=INFO(4)
11371 CONTINUE
      DO 11381 I=1,8
      PXNAME(I)=BLANK
11381 CONTINUE
11382 CONTINUE
      GO TO 11391
11400 CONTINUE
      MAX=MIN0(8,NSTRNG)
      DO 11411 I=1,MAX
      PXNAME(I)=STRING(I)
11411 CONTINUE
11412 CONTINUE
      GO TO 11392
11395 CONTINUE
      GO TO 11392
11391 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(11395,11395,11395,11395,11400,11395,11395,11395,11395),INTERP
11392 CONTINUE
      IF(FLAGS(1))GOTO 10011
      ISTOP=0
      IF(FLAGS(59))GOTO 10012
      CALL T2REST
      CALL TXNEXT
      GO TO 10132
11420 CONTINUE
      PAUSE
      GO TO 10132
11430 CONTINUE
      GO TO 11441
11470 CONTINUE
      CALL T2RDMS
      GO TO 11442
11480 CONTINUE
      CALL T2RDPT
      GO TO 11442
11441 CALL TOKEN(INFO,CARD,   13HMESH,POINTS,;)
      GO TO (11445,11447,11445,11445,11445,11445,11445,11446, 11445),INT
     *ERP
11446 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11447 GO TO ( 11470,11480,11445),KEYORD
11445 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11442 CONTINUE
      GO TO 10132
11490 CONTINUE
      ISTOP=INTEG
      IF(.TRUE.) GOTO 10012
      GO TO 10132
11500 CONTINUE
      CALL T2SETC(INFO,CARD)
      GO TO 10132
11510 CONTINUE
      N1=1
      N2=NPOINT
      LEVEL=3
      LIST=0
      IARRAY=0
      GO TO 11521
11550 CONTINUE
      IARRAY=INTEG
      GO TO 11521
11560 CONTINUE
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11581
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 11591
11581 CONTINUE
      LEVEL=INTEG
11591 CONTINUE
11571 CONTINUE
      GO TO 11521
11600 CONTINUE
      CALL T2PNTS(N1,N2)
      GO TO 11521
11610 CONTINUE
      GO TO 11621
11630 CONTINUE
      LIST=INTEG
      GO TO 11622
11625 CONTINUE
      LIST=10000
      INFO(3)=INFO(4)
      GO TO 11622
11621 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(11625,11625,11630,11630,11625,11625,11625,11625,11625),INTERP
11622 CONTINUE
      GO TO 11521
11521 CALL TOKEN(INFO,CARD,   25HX:1,Y,LEVEL,POINTS,LIST,;)
      GO TO (11522,11527,11525,11525,11525,11525,11525,11526, 11525),INT
     *ERP
11526 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11527 GO TO ( 11550,11550,11560,11600,11610,11525),KEYORD
11525 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 11521
11522 CONTINUE
      I = NINCR*(N1-1)
      N = N2-N1+1
      IF(IARRAY .NE. 0)GOTO 11651
      I=I+IVRPTR(4)
      GOTO 11661
11651 CONTINUE
      I=I+IVRPTR(2)
11661 CONTINUE
11641 CONTINUE
      CALL T2SMOO(DATBUF(I),N,LEVEL,NINCR)
      GO TO 10132
11670 CONTINUE
      CALL T2TTXT
      GO TO 10132
11680 CONTINUE
      CALL T2VALU
      GO TO 10132
10180 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
10170 GO TO ( 10190,10440,10440,10440,10440,10630,10700,10700,10710,1091
     *0,11000,11160,11160,11330,11330,11330,11420,11430,11490,11490,1149
     *0,11500,11510,11670,11670,11680,11680,11680,11680,10135),KEYORD
10131 CALL TOKEN(INFO,CARD,  193HARROW,BOX:1,CIRCLE:3,ELLIPSE:3,DIAMOND:
     *2,DUMP,FREQUENCY,BIN,JOIN,LIST,PLOT,HISTOGRAM,BARGRAPH:1,NEW,NEWFR
     *AME,NEWPLOT,PAUSE,READ,RETURN,STOP:1,END:1,SET,SMOOTH,TITLE:1,TEXT
     *:2,X:1,DX:2,Y:3,DY:4,;)
      GOTO(10150,10170,10140,10140,10135,10160,10135,10180,10135),INTERP
10135 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 10132
10132 CONTINUE
      GOTO 10011
10012 CONTINUE
      IF(.NOT.((.NOT.FLAGS(28).AND.NPOINT.NE.0)))GOTO 11701
      I1=IVRPTR(2)
      I2=IVRPTR(4)
      I3=IVRPTR(6)
      I4=IVRPTR(3)
      I5=IVRPTR(5)
      I6=IVRPTR(7)
      I7=IVRPTR(1)
      CALL T2PLOT(DATBUF(I1),DATBUF(I2), DATBUF(I3),DATBUF(I4),DATBUF(I5
     *),DATBUF(I6), DATBUF(I7),NPOINT,NINCR)
11701 CONTINUE
      GO TO 11711
11720 CONTINUE
      IARGUE=INTEG
      GO TO 11712
11715 CONTINUE
      IARGUE=0
      GO TO 11712
11711 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(11715,11715,11720,11720,11715,11715,11715,11715,11715),INTERP
11712 CONTINUE
      CALL TXVOID
      IF (ISTOP.NE.0) INFO(1)=-1
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2DOIT
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      CALL T2PNTR
      WRITE(OUTFIL,11730)
11730 FORMAT(   36H TOP DRAWER (10/10/80) NOW EXECUTING/)
      CALL T2MAIN(I)
      CALL TXEND
cccccccccccccccccc 25-Dec-1992 Modified by Y.Teramoto
cccc      STOP
	return
cccccccccccccccccc
      END
cnext ****************************************
      SUBROUTINE T2FREQ
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      INTEGER ISET(4),IBUFFR(200)
      REAL XPARM(4),PARM(4)
      COMMON /T2NPMX/ NPTMAX
      COMMON /T2XPNT/ DATBUF(5125)
      INTEGER NPTMAX
      EQUIVALENCE (BUFFER(1),IBUFFR(1))
      DATA HNONE/    4HNONE/
      DO 11741 I=1,4
      ISET(I)=0
      PARM(I)=0.
11741 CONTINUE
11742 CONTINUE
      GO TO 11751
11780 CONTINUE
      I=INTEG
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11801
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 11811
11801 CONTINUE
      PARM(I)=FLOTNG
      ISET(I)=1
11811 CONTINUE
11791 CONTINUE
      GO TO 11751
11751 CALL TOKEN(INFO,CARD,   67HMIN:1,FROM:1,MAX:2,TO:2,BINWIDTH:3,STEP
     *:3,BY:3,WIDTH:3,N:4,BINS:4,;)
      GO TO (11752,11757,11755,11755,11755,11755,11755,11756, 11755),INT
     *ERP
11756 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
11757 GO TO ( 11780,11780,11780,11780,11780,11780,11780,11780,11780,1178
     *0,11755),KEYORD
11755 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 11751
11752 CONTINUE
      NBINS = PARM(4)
      IF (NBINS.LE.0 .OR. NBINS.GT.200) ISET(4)=0
      IF (PARM(1).EQ.PARM(2)) ISET(2)=0
      IF (PARM(3).EQ.0.) ISET(3)=0
      DO 11821 III=1,2
      IF(.NOT.(FLAGS(29)))GOTO 11841
      WRITE(DBGFIL,11850)PARM,ISET
11850 FORMAT(60X,   11HPARM,ISET  ,4G10.3,4I2)
11841 CONTINUE
      NSET = ISET(1)+ISET(2)+ISET(3)+ISET(4)
      IF(NSET .GE. 3)GOTO 11871
      IF((.NOT.(FLAGS(5))) .AND. (.NOT.(FLAGS(65))))GOTO 11891
      IBUFFR(1)=1
      GOTO 11901
11891 CONTINUE
      IBUFFR(1)=0
11901 CONTINUE
11881 CONTINUE
      IF((.NOT.(FLAGS(6))) .AND. (.NOT.(FLAGS(66))))GOTO 11921
      IBUFFR(2)=1
      GOTO 11931
11921 CONTINUE
      IBUFFR(2)=0
11931 CONTINUE
11911 CONTINUE
      XPARM(1)=XYZLIM(1,1)
      XPARM(2)=XYZLIM(1,2)
      I1=IVRPTR(2)
      I2=IVRPTR(3)
      CALL T2STL1(DATBUF(I1),DATBUF(I2),  NPOINT,NONLIN(1),IBUFFR,XPARM,
     *NINCR,1,1)
      CALL TDSLIN(XPARM(1),XPARM(2),  NXYZ1(1),NXYZ2(1),SCLPRM(1,1),200,
     *  NBINS,BUFFER(201),IBUFFR)
      XPARM(3) = BUFFER(202)-BUFFER(201)
      XPARM(4)=NBINS
      DO 11941 I=1,4
      IF(ISET(I) .NE. 0)GOTO 11961
      PARM(I)=XPARM(I)
      ISET(I)=1
      NSET=NSET+1
      IF(NSET.EQ.3)GOTO 11942
11961 CONTINUE
11941 CONTINUE
11942 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11981
      WRITE(DBGFIL,11990)XPARM
11990 FORMAT(60X,   11HMY PARMS   ,4G10.3)
11981 CONTINUE
11871 CONTINUE
      IF (ISET(1).EQ.0) PARM(1)=PARM(2)-PARM(3)*PARM(4)
      IF (ISET(2).EQ.0) PARM(2)=PARM(1)+PARM(3)*PARM(4)
      IF (ISET(3).EQ.0) PARM(3)=(PARM(2)-PARM(1))/PARM(4)
      IF (ISET(4).EQ.0) PARM(4)=(PARM(2)-PARM(1))/PARM(3)
      IF(.NOT.(FLAGS(29)))GOTO 12011
      WRITE(DBGFIL,12020)PARM
12020 FORMAT(60X,   11HFINAL PARMS,4G10.3)
12011 CONTINUE
      NBINS = PARM(4)
      IF(NBINS.LE.200)GOTO 11822
      ISET(3)=0
      ISET(4)=0
      WRITE(ERRFIL,12030)NBINS
12030 FORMAT(   16H *** WARNING ***,I6,   17H IS TOO MANY BINS)
      CALL T2ERR(INFO,INFO,CARD,ERRFIL, 2)
11821 CONTINUE
11822 CONTINUE
      RBINW=1./PARM(3)
12040 CONTINUE
12041 CONTINUE
      IYV=0
      YVAL=1.
      MIN=IVRPTR(4)
      IF(DATBUF(MIN).EQ.HNONE)GOTO 12042
      MAX=IVRPTR(4)+NINCR*(NPOINT-1)
      DO 12051 I=MIN,MAX,NINCR
      IF(DATBUF(I) .EQ. 0.)GOTO 12071
      IYV=1
      GOTO 12042
12071 CONTINUE
12051 CONTINUE
12052 CONTINUE
12042 CONTINUE
      I=IVRPTR(3)
      IF(DATBUF(I) .NE. HNONE)GOTO 12091
      DX=0.
      IDXV=0
      GOTO 12101
12091 CONTINUE
      IDXV=1
12101 CONTINUE
12081 CONTINUE
      DO 12111 I=1,NBINS
      BUFFER(I+200)=0.
      BUFFER(I)=0.
12111 CONTINUE
12112 CONTINUE
      DO 12121 I=1,NPOINT
      ID=(I-1)*NINCR
      IXX=IVRPTR(2)+ID
      IDX=IVRPTR(3)+ID
      IYY=IVRPTR(4)+ID
      IF (IDXV.NE.0) DX=DATBUF(IDX)
      ALO = DATBUF(IXX)-4.*DX
      AHI = DATBUF(IXX)+4.*DX
      IBLO = INT((ALO-XYZLIM(1,1))*RBINW + 1.0)
      IF(IBLO.GT.NBINS)GOTO 12121
      IBHI = INT((AHI-XYZLIM(1,1))*RBINW + 1.0)
      IF(IBHI.LE.0)GOTO 12121
      IF (IYV.NE.0) YVAL=DATBUF(IYY)
      IF(IBLO .NE. IBHI)GOTO 12141
      BUFFER(IBLO) = BUFFER(IBLO)+YVAL
      BUFFER(IBLO+200) =  BUFFER(IBLO+200)+YVAL**2
      GOTO 12151
12141 CONTINUE
      WT=1.00006*YVAL
      IF (IBLO.LE.0) IBLO=1
      IF (IBHI.GT.NBINS) IBHI=NBINS
      Z = (XYZLIM(1,1)+PARM(3)*(FLOAT(IBLO)-1.5)-DATBUF(IXX))/DX
      DZ = PARM(3)/DX
      A = T2FRQF(Z)
      DO 12161 J=IBLO,IBHI
      Z = Z+DZ
      B = A
      A = T2FRQF(Z)
      BUFFER(J)=BUFFER(J)+WT*(A-B)
      BUFFER(J+200)=BUFFER(J+200)+(WT*(A-B))**2
12161 CONTINUE
12162 CONTINUE
12151 CONTINUE
12131 CONTINUE
12121 CONTINUE
12122 CONTINUE
      NPOINT=NBINS
      ZZ = XYZLIM(1,1)
      I1=IVRPTR(4)
      I2=IVRPTR(5)
      I3=IVRPTR(2)
      I4=IVRPTR(3)
      DO 12171 I=1,NPOINT
      DATBUF(I1)=BUFFER(I)
      DATBUF(I2)=SQRT(BUFFER(I+200))
      DATBUF(I3) = ZZ
      DATBUF(I4) = 0.
      I1=I1+NINCR
      I2=I2+NINCR
      I3=I3+NINCR
      I4=I4+NINCR
      ZZ = ZZ+PARM(3)
12171 CONTINUE
12172 CONTINUE
      DATBUF(LISTPT+2)=7+NNAME+NINCR*(NPOINT+1)
      DATBUF(LISTPT+4)=NPOINT
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2PNTR
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      COMMON /T2NPMX/ NPTMAX
      COMMON /T2XPNT/ DATBUF(5125)
      INTEGER NPTMAX
      DATA HNONE/    4HNONE/,BLANK/    1H /
      DATBUF(1)=HNONE
      LISTPT=2
      NINCR=5
      NNAME=1
      IBLKTP=1
      IDIMNS=2
      IVARBL(1)=2
      IVARBL(2)=3
      IVARBL(3)=4
      IVARBL(4)=5
      IVARBL(5)=1
      DATBUF(LISTPT)=0.
      DATBUF(LISTPT+1)=0.
      DATBUF(LISTPT+9)=BLANK
      CALL T2SETS
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2PNTS(N1,N2)
      INTEGER N1,N2
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      NN1=N1
      NN2=N2
      CALL TOKEN(INFO,CARD,    5HFROM;)
      IF(INTERP .EQ. 2)GOTO 12191
      INFO(3)=INFO(4)
12191 CONTINUE
      GO TO 12201
12210 CONTINUE
      NN1=INTEG
      CALL TOKEN(INFO,CARD,    3HTO;)
      IF(INTERP .EQ. 2)GOTO 12231
      INFO(3)=INFO(4)
12231 CONTINUE
      GO TO 12241
12250 CONTINUE
      NN2=INTEG
      GO TO 12242
12245 CONTINUE
      INFO(3)=INFO(4)
      GO TO 12242
12241 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(12245,12245,12250,12250,12245,12245,12245,12245,12245),INTERP
12242 CONTINUE
      GO TO 12202
12205 CONTINUE
      INFO(3)=INFO(4)
      GO TO 12202
12201 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(12205,12205,12210,12210,12205,12205,12205,12205,12205),INTERP
12202 CONTINUE
      N1 = MAX0(MIN0(NN1,NN2),1)
      N2 = MIN0(MAX0(NN1,NN2),NPOINT)
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2RDPT
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL TXSYM
      EXTERNAL TXSYM
cmt4       BYTE BLANKS(3)
      character*1 BLANKS(3)
      COMMON /T2NPMX/ NPTMAX
      COMMON /T2XPNT/ DATBUF(5125)
      INTEGER NPTMAX
      INTEGER IBUFF(100),IBUFV(100)
      EQUIVALENCE (BUFFER(201),IBUFF(1)),(BUFFER(301),IBUFV(1))
      DATA BLANKS/1H ,1H ,1H /
      IF(IBLKTP .NE. 2)GOTO 12271
      IF((FLAGS(28)))GOTO 12291
      WRITE(ERRFIL,12300)
12300 FORMAT (   36H *** ERROR *** MESH DATA OVERWRITTEN)
      FLAGS(23)=.TRUE.
12291 CONTINUE
      CALL T2PNTR
12271 CONTINUE
      MX=NFIELD+2
      DO 12311 IFLD=1,MX
      BUFFER(IFLD)=0.
      IBUFF(IFLD)=0
12311 CONTINUE
12312 CONTINUE
      IF (SYMBOL.EQ.0.) SYMBOL=TXSYM(BLANKS)
      DO 12321 IPOS=1,NINCR
      IVAL=IVARBL(IPOS)
      IBUFV(IPOS)=IFIELD(IVAL)
12321 CONTINUE
12322 CONTINUE
      IVAL=IFIELD(1)
cccccccccccccccc 26-dec-1995 Modified by Y.Teramoto
	if( IVAL .GE. -1 ) then
	  IBUFF(IVAL)=-1
	  BUFFER(IVAL+2)=SYMBOL
	endif
ccc      IBUFF(IVAL)=-1
ccc      BUFFER(IVAL+2)=SYMBOL
cccccccccccccccc end of 26-dec-1995 modification
cccccccccccccccc 6-Sep-91 Modified by Y.Teramoto
ccccccccccc 29-Jan-93 Modified back by A.Miyamoto/Y.Teramoto
      DO 12331 I=2,15
cccc	DO 12331 I=2,NFIELD
cccccccccccccccccccccccc
cccccccccccccc end of the modification
      IV=IFIELD(I)
cccccccccccccc 26-dec-1995 Modified by Y.Teramoto
	if( IV .ge. -1 ) then
	  IBUFF(IV)=1
	endif
cccc      IBUFF(IV)=1
cccccccccccccc end of 26-dec-1995 modification
12331 CONTINUE
12332 CONTINUE
      IF (NPOINT.LT.MXECHO) IPRNT=1
      IF (FLAGS(59)) IPRNT=0
      IF (NPOINT.LE.5125) MAXP = 0
      IF(.NOT.(FLAGS(29)))GOTO 12351
      WRITE(DBGFIL,12360)LISTPT,NHEADR,NINCR,NPOINT, IFIELD,IVARBL,(IBUF
     *F(I),I=1,15), (IBUFV(I),I=1,15)
12360 FORMAT (60X,   26HLISTPT,NHEADR,NINCR,NPOINT,4I6/  60X,    6HIFIEL
     *D,15I4/60X,    6HIVARBL,15I4/  60X,    5HIBUFF,15I4/60X,    5HIBUF
     *V,15I4)
12351 CONTINUE
      INFO(9)=1
12370 CONTINUE
12371 CONTINUE
      CALL TOKRED(INFO,CARD,LENCRD,INPFIL,INPFMT)
      IF(.NOT.((INFO(1).LE.0)))GOTO 12391
      IF(.NOT.(FLAGS(29)))GOTO 12411
      WRITE(DBGFIL,12420)
12420 FORMAT(60X,   17HEND OF INPUT FILE)
12411 CONTINUE
      INFO(9)=1
      GOTO 12372
12391 CONTINUE
      IF(IPRNT .EQ. 0)GOTO 12441
      CALL T2ERR(INFO,INFO,CARD,OUTFIL,6)
      IF(.NOT.(FLAGS(29)))GOTO 12461
      WRITE(DBGFIL,12470)INFO
12470 FORMAT(60X,4HINFO,10I5)
12461 CONTINUE
12441 CONTINUE
12480 CONTINUE
      DO 12481 IFLD=1,NFIELD
      IF (IBUFF(IFLD)) 12490,12500,12510
12510 CONTINUE
      GO TO 12521
12530 CONTINUE
      IF(IFLD.EQ.1)GOTO 12371
      IF(.TRUE.) GOTO 12482
      GO TO 12522
12540 CONTINUE
      BUFFER(IFLD+2)=FLOTNG
      GO TO 12522
12525 CONTINUE
      IF(IFLD .NE. 1)GOTO 12561
      INFO(9)=1
      GOTO 12372
12561 CONTINUE
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 12522
12521 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(12530,12525,12540,12540,12525,12525,12525,12525,12525),INTERP
12522 CONTINUE
      GOTO 12481
12490 CONTINUE
      GO TO 12571
12580 CONTINUE
      IF(.TRUE.) GOTO 12482
      GO TO 12572
12575 CONTINUE
      BUFFER(IFLD+2)=TXSYM(STRING)
      GO TO 12572
12571 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(12580,12575,12575,12575,12575,12575,12575,12575,12575),INTERP
12572 CONTINUE
12500 CONTINUE
      GO TO 12591
12600 CONTINUE
      IF(IFLD.EQ.1)GOTO 12371
      IF(.TRUE.) GOTO 12482
      GO TO 12592
12610 CONTINUE
      GO TO 12592
12595 CONTINUE
      IF(IFLD .NE. 1)GOTO 12631
      INFO(9)=1
      IF(.TRUE.) GOTO 12372
12631 CONTINUE
      GO TO 12592
12591 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(12600,12595,12610,12610,12595,12595,12595,12595,12595),INTERP
12592 CONTINUE
12481 CONTINUE
12482 CONTINUE
      IF(.NOT.(FLAGS(28)))GOTO 12651
      NPOINT=0
      FLAGS(28)=.FALSE.
      FLAGS(4)=.FALSE.
      IF (.NOT. FLAGS(59))IPRNT=1
12651 CONTINUE
      IF(NPOINT .LT. MXECHO)GOTO 12671
      IF(IPRNT .NE. 1)GOTO 12691
      WRITE(OUTFIL,12700)MXECHO
12700 FORMAT (   27H  PRINTING SUPPRESSED AFTER,I6,    8H POINTS.,     1
     *9H  (PLOT CONTINUES.))
      IPRNT=0
12691 CONTINUE
12671 CONTINUE
      IFIRST=LISTPT+NHEADR+NINCR*NPOINT-1
      IF(IFIRST+NINCR .LE. 5125)GOTO 12721
      IF(MAXP .NE. 0)GOTO 12741
      MAXP=1
      WRITE(ERRFIL,12750)NPOINT
12750 FORMAT (   36H *** WARNING *** POINT BUFFER SIZE (,  I5,   37H) RE
     *ACHED. SUBSEQUENT POINTS IGNORED.)
      CALL T2ERR(INFO,INFO,CARD,ERRFIL, 2)
12741 CONTINUE
      GOTO 12761
12721 CONTINUE
      DO 12771 IPOS=1,NINCR
      IFLD=IBUFV(IPOS)
      K=IPOS+IFIRST
      IVAL = IVARBL(IPOS)
      DATBUF(K)=BUFFER(IFLD+2)*FACTXY(IVAL)
12771 CONTINUE
12772 CONTINUE
      I5=0
      DO 12781 I=79,81
      I5=I5+2
      IF(.NOT.(FLAGS(I)))GOTO 12801
      I1=IVRPTR(I5)+NPOINT*NINCR
      I2=IVRPTR(I5+1)+NPOINT*NINCR
      DATBUF(I2)=DATBUF(I2)*ABS(DATBUF(I1))
12801 CONTINUE
12781 CONTINUE
12782 CONTINUE
      NPOINT=NPOINT+1
12761 CONTINUE
12711 CONTINUE
      GOTO 12371
12372 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12821
      WRITE(DBGFIL,12830)NPOINT
12830 FORMAT(60X,I5,   21H POINTS NOW IN BUFFER)
12821 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2SMOO(ARRAY,N,INLEVL,INCRMT)
      INTEGER N,INLEVL
      REAL ARRAY(INCRMT,N)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      INTEGER LEVEL,FIRST,LAST,OVERLP
      IF (N.LE.1) RETURN
      IF(INLEVL .LE. 0 .OR. INLEVL .GT. 5)GOTO 12851
      LEVEL = INLEVL
      GOTO 12861
12851 CONTINUE
      I = 25
      LEVEL = 1
12871 IF(N.LT.I .OR. LEVEL.EQ.5) GOTO 12872
      LEVEL=LEVEL+1
      I=I+I
      GOTO 12871
12872 CONTINUE
12861 CONTINUE
12841 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12891
      WRITE(DBGFIL,12900)N,LEVEL
12900 FORMAT(58X,I5,   14H POINTS, LEVEL,I2)
12891 CONTINUE
      OVERLP = 4*LEVEL
      NB = (BUFSIZ-OVERLP)/4
      IE = NB+1
      IF = IE+NB
      IG = IF+NB
      IF(N .GT. NB)GOTO 12921
      J=IG
      DO 12931 I=1,N
      BUFFER(J)=ARRAY(1,I)
      J=J+1
12931 CONTINUE
12932 CONTINUE
      CALL SMCTRL(BUFFER(1),BUFFER(IG),N,LEVEL,BUFFER(IE),BUFFER(IF))
      DO 12941 I=1,N
      ARRAY(1,I)=BUFFER(I)
12941 CONTINUE
12942 CONTINUE
      GOTO 12951
12921 CONTINUE
      MINH1 = IG + NB
      MAXH1 = MINH1 + OVERLP - 1
      FIRST = 1
      IPNT = 1
12961 CONTINUE
      LAST = MIN0(FIRST+NB-1,N)
      NP = LAST-FIRST+1
      NPM2OV = NP - 2*OVERLP
      J=IG
      DO 12971 I=FIRST,LAST
      BUFFER(J)=ARRAY(1,I)
      J=J+1
12971 CONTINUE
12972 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12991
      WRITE(DBGFIL,13000)FIRST,LAST
13000 FORMAT(60X,    6HSMOOTH,2I5)
12991 CONTINUE
      CALL SMCTRL(BUFFER(1),BUFFER(IG),  NP,LEVEL,BUFFER(IE),BUFFER(IF))
      IF(FIRST .EQ. 1)GOTO 13021
      I=0
      DO 13031 J=MINH1,MAXH1
      I=I+1
      BUFFER(I) = BUFFER(J)
13031 CONTINUE
13032 CONTINUE
13021 CONTINUE
      DO 13041 I=1,NPM2OV
      ARRAY(1,IPNT) = BUFFER(I)
      IPNT=IPNT+1
13041 CONTINUE
13042 CONTINUE
      I=NPM2OV
      DO 13051 J=MINH1,MAXH1
      I=I+1
      BUFFER(J) = BUFFER(I)
13051 CONTINUE
13052 CONTINUE
      FIRST = LAST - 2*OVERLP + 1
      IF(LAST .NE. N)GOTO 12961
12962 CONTINUE
      IPP =  NP - OVERLP + 1
      DO 13061 I=MINH1,MAXH1
      ARRAY(1,IPNT) = BUFFER(I)
      J = IPNT+OVERLP
      ARRAY(1,J) = BUFFER(IPP)
      IPNT=IPNT+1
      IPP=IPP+1
13061 CONTINUE
13062 CONTINUE
12951 CONTINUE
12911 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2TTXT
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL TCON(5),TANGDF(6),XY(2),XYZ(3)
cmt4       BYTE BLANK,ESCAPE,PLUS,MINUS,AMPRSN,ZERO
cmt4       BYTE CASE(160),TITLE(160)
      character*1  BLANK,ESCAPE,PLUS,MINUS,AMPRSN,ZERO
      character*1  CASE(160),TITLE(160)
      EQUIVALENCE (BUFFER(1),TITLE(1)),(BUFFER(200),CASE(1))
      DATA TANGDF /0.,90.,0.,-90.,0.,0./
      DATA BLANK /    1H /,ESCAPE/    1H@/,PLUS/    1H+/,MINUS/    1H-/,
     *  AMPRSN/    1H&/,ZERO/    1H0/
      ITMODE=INTEG
      ICHAR = 6
      NCASE=0
      IDATA=0
      IPSET=0
      ICNTR=0
      NCHARS = -1
      TITCON(5)=1.
      TITCON(3) = 0.
      DO 13071 I=1,5
      TCON(I) = 7777.
13071 CONTINUE
13072 CONTINUE
      IF (.NOT.FLAGS(73)) CALL TXDEF1
      GO TO 13081
13090 CONTINUE
      SAVE = FLOTNG
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 13111
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 13121
13111 CONTINUE
      TITX = SAVE
      TITY = FLOTNG
      IPSET=1
      ICHAR=5
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 13141
      INFO(3)=INFO(4)
      GOTO 13151
13141 CONTINUE
      TITZ=FLOTNG
13151 CONTINUE
13131 CONTINUE
13121 CONTINUE
13101 CONTINUE
      GO TO 13081
13160 CONTINUE
      NCHARS=NSTRNG
      DO 13171 I=1,NCHARS
      TITLE(I)=STRING(I)
13171 CONTINUE
13172 CONTINUE
      GO TO 13081
13200 CONTINUE
      ICHAR=INTEG
      IPSET=2
      IPSET = 2
      ICNTR = 1
      IF (.NOT.FLAGS(74)) CALL TXDEFW
      IDATA = 0
      GO TO 13081
13210 CONTINUE
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 5)GOTO 13231
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 13241
13231 CONTINUE
      NCASE=NSTRNG
      DO 13251 I=1,NCASE
      CASE(I)=STRING(I)
13251 CONTINUE
13252 CONTINUE
13241 CONTINUE
13221 CONTINUE
      GO TO 13081
13260 CONTINUE
      IDATA=IOR(IDATA,INTEG)
      GO TO 13081
13270 CONTINUE
      ICNTR=1
      GO TO 13081
13280 CONTINUE
      I=INTEG
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 13301
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 13311
13301 CONTINUE
      TCON(I)=FLOTNG
13311 CONTINUE
13291 CONTINUE
      GO TO 13081
13190 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
13180 GO TO ( 13200,13200,13200,13200,13200,13200,13200,13200,13210,1326
     *0,13260,13260,13260,13270,13280,13280,13280,13280,13280,13085),KEY
     *ORD
13085 CONTINUE
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      IF(.TRUE.) GOTO 13320
      GO TO 13081
13081 CALL TOKEN(INFO,CARD,  133HX:1,Y:2,LEFT:2,BOTTOM:1,G:3,TOP:3,RIGHT
     *:4,RITE:4,CASE,DATA:7,XDATA:1,YDATA:2,ZDATA:4,CENTER,SIZE:1,ANGLE:
     *2,SPACES:3,INDEX:4,LINES:5,;)
      GOTO(13082,13180,13090,13090,13160,13085,13085,13190,13085),INTERP
13082 CONTINUE
      IF(ICHAR .EQ. 6)GOTO 13341
      TITCON(2) = TANGDF(ICHAR)
      TITCON(1) = TSIZDF
      IF (ICHAR.EQ.3) TITCON(1) = 1.5*TSIZDF
      TITCON(3) = 0.
      TITCON(4) = 2.
13341 CONTINUE
      DO 13351 I=1,5
      IF (TCON(I).NE.7777.) TITCON(I)=TCON(I)
13351 CONTINUE
13352 CONTINUE
      HSPACE = 0.1*ABS(TITCON(1))
      VSPACE = TITCON(4)*HSPACE
      IXXXXX=ICHAR
13360 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 6))GOTO 13441
      WRITE(6,13450)IXXXXX
13450 FORMAT(    8H ICHAR (,I5,   31H) OUT OF RANGE FOR CASE (1 TO6))
      CALL ERRTRA
      STOP 7777
13441 CONTINUE
      GO TO (13370,13380,13390,13400,13410,13420),IXXXXX
13370 CONTINUE
      TITY = WINDOW(2) + (TITCON(5)-1.0)*VSPACE - HSPACE - 0.2*ABS(LBLSI
     *Z)
      TITX = 0.5*(WINDOW(1)+WINDOW(3))
      GOTO 13362
13380 CONTINUE
      IF((LBLCHR(2) .EQ. 0) .AND. (((FLAGS(36))) .AND. ((FLAGS(40)))))GO
     *TO 13471
      XLBL=0.1*FLOAT(LBLCHR(2))
      GOTO 13481
13471 CONTINUE
      XLBL=0.5
13481 CONTINUE
13461 CONTINUE
      TITX = WINDOW(1)-(0.75+TITCON(5))*VSPACE-ABS(LBLSIZ)*XLBL
      TITY = 0.5*(WINDOW(2)+WINDOW(4))
      GOTO 13362
13390 CONTINUE
      TITX=0.5*(WINDOW(3)+WINDOW(1))
      TITY = WINDOW(4) + TITCON(5)*VSPACE
      GOTO 13362
13400 CONTINUE
      TITX = WINDOW(3) + 1.5*HSPACE + (TITCON(5)-1.0)*VSPACE
      TITY = 0.5*(WINDOW(2)+WINDOW(4))
      GOTO 13362
13420 CONTINUE
      ANGLE=TITCON(2)*.017453
      TITX = TITX + VSPACE*SIN(ANGLE)
      TITY = TITY - VSPACE*COS(ANGLE)
      IDATA=ITITDT
      ICNTR=ITCNTR
      GOTO 13362
13410 CONTINUE
      GOTO 13362
13362 CONTINUE
      ITITDT=IDATA
      ITCNTR=ICNTR
      IF(ITITDT .EQ. 0)GOTO 13501
      IF((FLAGS(75)))GOTO 13521
      CALL T2STLM
      CALL TXDEF2
13521 CONTINUE
      IF(.NOT.(FLAGS(75)))GOTO 13541
      DO 13551 I=1,3
      XYZ(I)=XYZLIM(I,1)
13551 CONTINUE
13552 CONTINUE
      IF ((IAND(ITITDT,1).NE.0)) XYZ(1)=TITX
      IF ((IAND(ITITDT,2).NE.0)) XYZ(2)=TITY
      IF ((IAND(ITITDT,4).NE.0)) XYZ(3)=TITZ
      CALL T2XFRM(3,1,XYZ,NONLIN,XFRM13,2,XY)
      IF ((IAND(ITITDT,1).NE.0)) TITX=XY(1)
      IF ((IAND(ITITDT,2).NE.0)) TITY=XY(2)
      ITITDT=0
      GOTO 13561
13541 CONTINUE
      NCHARS = 0
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,3)
      WRITE(ERRFIL,13570)
13570 FORMAT (   32H DATA COORDINATE SYSTEM NOT SET.,     33H  PUT THIS
     *CARD AFTER DATA POINTS,     14H OR SET LIMITS)
13561 CONTINUE
13531 CONTINUE
13501 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 13591
      WRITE(DBGFIL,13600)TITX,TITY,TITCON,NCHARS,NCASE,(TITLE(I),I=1,28)
13600 FORMAT (60X,    5H(X,Y),2G10.2,    8H  TITCON,5F6.1/60X,28Z2)
13591 CONTINUE
      CALL TXSCIS(1)
      IF(ITMODE .NE. 1)GOTO 13621
13631 CONTINUE
      CALL TOKRED(INFO,CARD,LENCRD,INPFIL,INPFMT)
      IF((INFO(1).LE.0))GOTO 13632
      CALL T2ERR(INFO,INFO,CARD,OUTFIL,6)
      IF(.NOT.(FLAGS(29)))GOTO 13651
      WRITE(DBGFIL,13660)INFO
13660 FORMAT(60X,4HINFO,10I5)
13651 CONTINUE
      IUSED=0
      GO TO 13671
13700 CONTINUE
      IUSED=1
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 5)GOTO 13721
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 13731
13721 CONTINUE
      DO 13741 I=1,NSTRNG
      NCHARS=NCHARS+1
      TITLE(NCHARS)=STRING(I)
13741 CONTINUE
13742 CONTINUE
13731 CONTINUE
13711 CONTINUE
      GO TO 13671
13750 CONTINUE
      IUSED=1
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 5)GOTO 13771
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 13781
13771 CONTINUE
      DO 13791 I=1,NSTRNG
      NCASE=NCASE+1
      CASE(NCASE)=STRING(I)
13791 CONTINUE
13792 CONTINUE
13781 CONTINUE
13761 CONTINUE
      GO TO 13671
13690 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
13680 GO TO ( 13700,13750,13675),KEYORD
13675 CONTINUE
      IF(.TRUE.)GOTO 13672
      GO TO 13671
13671 CALL TOKEN(INFO,CARD,   11HMORE,CASE,;)
      GOTO(13672,13680,13675,13675,13675,13675,13675,13690,13675),INTERP
13672 CONTINUE
      IF(IUSED .NE. 0)GOTO 13631
13632 CONTINUE
      INFO(9)=1
      IF(NCHARS .LE. 0)GOTO 13811
      IF(ITCNTR .EQ. 0)GOTO 13831
      ANGLE=TITCON(2)*.017453
      IF (TITCON(3).EQ.0.) TITCON(3) = FLOAT(NCHARS)
      SPACE = 0.5*HSPACE*(TITCON(3)-1.)
      XYZ(1) = TITX - SPACE*COS(ANGLE)
      XYZ(2) = TITY - SPACE*SIN(ANGLE)
      GOTO 13841
13831 CONTINUE
      XYZ(1) = TITX
      XYZ(2) = TITY
13841 CONTINUE
13821 CONTINUE
      CALL T2XFRM(2,1,XYZ,LINEAR,XFRM34,2,XY)
      CALL TXTEXT(XY(1),XY(2),TITCON(1),  TITCON(2),TITLE,NCHARS,CASE,NC
     *ASE)
13811 CONTINUE
      GOTO 13851
13621 CONTINUE
      ANGLE = TITCON(2)*.017453
      XSPACE= VSPACE*SIN(ANGLE)
      YSPACE=-VSPACE*COS(ANGLE)
      IF(NCHARS .GE. 0)GOTO 13871
      TITX=TITX-XSPACE
      TITY=TITY-YSPACE
13871 CONTINUE
      IESCMD = 0
13881 CONTINUE
13891 CONTINUE
      IF(NCHARS.LE.0)GOTO 13892
      IF(TITLE(NCHARS).NE.BLANK)GOTO 13892
      NCHARS=NCHARS-1
      GOTO 13891
13892 CONTINUE
      INFO(3)=9999
      CALL TOKRED(INFO,CARD,LENCRD,INPFIL,INPFMT)
      IF(INFO(1).LE.0)GOTO 13882
      CALL T2ERR(INFO,INFO,CARD,OUTFIL,6)
      IF(.NOT.(FLAGS(29)))GOTO 13911
      WRITE(DBGFIL,13920)INFO
13920 FORMAT(60X,4HINFO,10I5)
13911 CONTINUE
      ICOL=1
      IF(CARD(1) .EQ. AMPRSN)GOTO 13941
      ICOL=0
      TITX=TITX+XSPACE
      TITY=TITY+YSPACE
      IF(NCHARS .LE. 0)GOTO 13961
      XYZ(1)=TITX
      XYZ(2)=TITY
      CALL T2XFRM(2,1,XYZ,LINEAR,XFRM34,2,XY)
      CALL TXTEXT(XY(1),XY(2),TITCON(1),  TITCON(2),TITLE,NCHARS,CASE,NC
     *HARS)
13961 CONTINUE
      NCHARS=0
      IF(CARD(1).EQ.ESCAPE)GOTO 13882
      IF(CARD(1) .NE. BLANK)GOTO 13981
      ICOL=1
13981 CONTINUE
      IF(CARD(1) .NE. PLUS)GOTO 14001
      ICOL=1
      TITX=TITX-XSPACE
      TITY=TITY-YSPACE
14001 CONTINUE
      IF(CARD(1) .NE. MINUS)GOTO 14021
      ICOL=1
      TITX=TITX+2.*XSPACE
      TITY=TITY+2.*YSPACE
14021 CONTINUE
      IF(CARD(1) .NE. ZERO)GOTO 14041
      ICOL=1
      TITX=TITX+XSPACE
      TITY=TITY+YSPACE
14041 CONTINUE
13941 CONTINUE
14051 CONTINUE
      IF(ICOL.GE.INFO(2))GOTO 14052
      IF(NCHARS.GE.160)GOTO 14052
      ICOL=ICOL+1
      NCHARS=NCHARS+1
      IF(CARD(ICOL) .NE. ESCAPE)GOTO 14071
      IF(CARD(ICOL+1) .EQ. ESCAPE)GOTO 14091
      IESCMD=1-IESCMD
      NCHARS=NCHARS-1
      GOTO 14051
14091 CONTINUE
      ICOL=ICOL+1
14071 CONTINUE
      IF(IESCMD .NE. 0)GOTO 14111
      TITLE(NCHARS) = CARD(ICOL)
      CASE(NCHARS)=BLANK
      GOTO 14121
14111 CONTINUE
      TITLE(NCHARS)=CARD(ICOL)
      ICOL=ICOL+1
      CASE(NCHARS)=CARD(ICOL)
14121 CONTINUE
14101 CONTINUE
      GOTO 14051
14052 CONTINUE
      GOTO 13881
13882 CONTINUE
13851 CONTINUE
13611 CONTINUE
13320 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2VALU
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1  STRING
      COMMON /T2NPMX/ NPTMAX
      COMMON /T2XPNT/ DATBUF(5125)
      INTEGER NPTMAX
      INTEGER ISET(4)
      REAL PARM(4),XPARM(4)
      ICH=INTEG
      IF (FLAGS(28)) NPOINT=0
      FLAGS(4)=.FALSE.
      FLAGS(28)=.FALSE.
      N1=1
      N2=NPOINT
      NMAX=(5125-NHEADR)/NINCR
      MODE = 2
      DO 14131 I=1,4
      ISET(I)=0
      PARM(I)=0.
      XPARM(I)=0.
14131 CONTINUE
14132 CONTINUE
      GO TO 14141
14170 CONTINUE
      MODE=INTEG
      ISET(4)=1
      GO TO 14181
14190 CONTINUE
      N2=INTEG
      GO TO 14182
14185 CONTINUE
      INFO(3)=INFO(4)
      GO TO 14182
14181 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(14185,14185,14190,14190,14185,14185,14185,14185,14185),INTERP
14182 CONTINUE
      CALL TOKEN(INFO,CARD,    3HTO;)
      IF(INTERP .EQ. 2)GOTO 14211
      INFO(3)=INFO(4)
14211 CONTINUE
      GO TO 14221
14230 CONTINUE
      N1=N2
      N2=INTEG
      GO TO 14222
14225 CONTINUE
      INFO(3)=INFO(4)
      GO TO 14222
14221 CALL TOKEN(INFO,CARD,    1H;)
      GOTO(14225,14225,14230,14230,14225,14225,14225,14225,14225),INTERP
14222 CONTINUE
      GO TO 14141
14240 CONTINUE
      PARM(3)=1.
      ISET(3)=1
      GO TO 14141
14250 CONTINUE
      I=INTEG
      CALL TOKEN(INFO,CARD,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 14271
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GOTO 14281
14271 CONTINUE
      PARM(I)=FLOTNG
      ISET(I)=1
14281 CONTINUE
14261 CONTINUE
      GO TO 14141
14160 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
14150 GO TO ( 14170,14170,14170,14240,14250,14250,14250,14250,14250,1425
     *0,14250,14250,14145),KEYORD
14290 CONTINUE
      PARM(1)=FLOTNG
      ISET(1)=1
      GO TO 14141
14141 CALL TOKEN(INFO,CARD,   89HNBINS:1,BINS:1,POINTS:2,INTEGERS,MIN:1,
     *FROM:1,MAX:2,TO:2,BINWIDTH:3,STEP:3,BY:3,WIDTH:3,;)
      GOTO(14142,14150,14290,14290,14145,14145,14145,14160,14145),INTERP
14145 CALL T2ERR(INFO,INFO,CARD,ERRFIL,5)
      GO TO 14141
14142 CONTINUE
      IF(N1 .LE. N2)GOTO 14311
      I=N1
      N1=N2
      N2=I
14311 CONTINUE
      IF (N1.LE.0 .OR. N1.GT.NMAX) N1=1
      IF (N2.LE.0 .OR. N2.GT.NMAX) N2=NMAX
      IF (PARM(1).EQ.PARM(2)) ISET(2)=0
      IF (PARM(3).EQ.0.) ISET(3)=0
      IF(.NOT.(FLAGS(29)))GOTO 14331
      WRITE(DBGFIL,14340)PARM,ISET
14340 FORMAT(60X,   11HPARM,ISET  ,4G10.3,4I2)
14331 CONTINUE
      DIFF = PARM(2)-PARM(1)
      IF (MODE.NE.1) DIFF=DIFF+1.0
      IF(ISET(1)+ISET(2)+ISET(3) .NE. 3)GOTO 14361
      N1=1
      N2=0.5+DIFF/PARM(3)
14361 CONTINUE
      XPARM(1)=PARM(2)
      XPARM(2)=PARM(1)
      XPARM(3) = DIFF/FLOAT(N2-N1+1)
      DO 14371 I=1,3
      IF (ISET(I).EQ.0) PARM(I)=XPARM(I)
14371 CONTINUE
14372 CONTINUE
      IF(NPOINT .GE. N2)GOTO 14391
      MIN=LISTPT+NHEADR+NPOINT*NINCR+1
      MAX=LISTPT+NHEADR+N2*NINCR
      DO 14401 I=MIN,MAX
      DATBUF(I)=0.
14401 CONTINUE
14402 CONTINUE
      MIN=IVRPTR(1)+NPOINT*NINCR
      DO 14411 I=MIN,MAX,NINCR
      DATBUF(I)=SYMBOL
14411 CONTINUE
14412 CONTINUE
      NPOINT=N2
      DATBUF(LISTPT+2)=NHEADR+NINCR*NPOINT
      DATBUF(LISTPT+4)=NPOINT
14391 CONTINUE
      XNSTEP = 0.
      IF (MODE.EQ.1 ) PARM(1)=PARM(1)+0.5*PARM(3)
      IF(.NOT.(FLAGS(29)))GOTO 14431
      WRITE(DBGFIL,14440)N1,N2,PARM(1),PARM(3)
14440 FORMAT (60X,2I5,2G10.3)
14431 CONTINUE
      IXXXXX=ICH
14450 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 4))GOTO 14511
      WRITE(6,14520)IXXXXX
14520 FORMAT(    6H ICH (,I5,   31H) OUT OF RANGE FOR CASE (1 TO4))
      CALL ERRTRA
      STOP 7777
14511 CONTINUE
      GO TO (14460,14470,14480,14490),IXXXXX
14460 CONTINUE
      MIN=IVRPTR(2)
      GOTO 14452
14480 CONTINUE
      MIN=IVRPTR(4)
      GOTO 14452
14470 CONTINUE
      MIN=IVRPTR(3)
      GOTO 14452
14490 CONTINUE
      MIN=IVRPTR(5)
      GOTO 14452
14452 CONTINUE
      MAX=MIN+(N2-1)*NINCR
      MIN=MIN+(N1-1)*NINCR
      DO 14531 I=MIN,MAX,NINCR
      DATBUF(I) = PARM(1)+XNSTEP*PARM(3)
      XNSTEP=XNSTEP+1.
14531 CONTINUE
14532 CONTINUE
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
      SUBROUTINE T2PLOT(XVALS,YVALS,ZVALS,XERRS,YERRS,ZERRS,SYMIN,NP,INC
     *RMT)
      INTEGER INCRMT,NP
      REAL XVALS(INCRMT,NP),YVALS(INCRMT,NP),ZVALS(INCRMT,NP),  XERRS(IN
     *CRMT,NP),YERRS(INCRMT,NP),ZERRS(INCRMT,NP),  SYMIN(INCRMT,NP)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL XYZV(3),XYZW(3),XYZE(3),XYZE1(3),XYZE2(3,2),XY(2,2)
      REAL SYMBL1,SYMBL,HNONE
cmt4       BYTE BLANKS(3)
      character*1 BLANKS(3)
      DATA HNONE/    4HNONE/
      DATA BLANKS/1H ,1H ,1H /
      IF(.NOT.(FLAGS(29)))GOTO 10021
      WRITE(DBGFIL,10030)NP
10030 FORMAT(60X,   18HT2PLOT CALLED WITH,I6,    7H POINTS)
10021 CONTINUE
      IF(NP .LE. 0)GOTO 10051
      BAR=BARSIZ(1)
      IF (NOSYMB.EQ.0.) NOSYMB=TXSYM(BLANKS)
      FLAGS(4) = .TRUE.
      FLAGS(28) = .TRUE.
      DO 10061 I=61,72
      FLAGS(I)=.FALSE.
10061 CONTINUE
10062 CONTINUE
      CALL T2AXES
      CALL TXSCIS(2)
      SIZNOW=0.
      SIZNEW=SYMSIZ
      SYMBL1=SYMIN(1,1)
      IF(SYMBL1 .NE. 0.)GOTO 10081
      SYMBL=SYMIN(1,2)
      IF (SYMBL.EQ.NOSYMB) SYMBL=SYMBOL
10081 CONTINUE
      IF(SYMBL1 .NE. HNONE)GOTO 10101
      SYMBL1=0.
      SYMBL=SYMBOL
10101 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10121
      WRITE(DBGFIL,10130)SYMBL1,SYMBL,SYMIN(1,1),SYMIN(1,2)
10130 FORMAT (60X,   10HPLOT SYMBL,4G10.3)
10121 CONTINUE
      DO 10141 IXYZ=1,3
      XYZV(IXYZ)=0.
      XYZE(IXYZ)=0.
10141 CONTINUE
10142 CONTINUE
      DO 10151 N=1,NP
      IF(SYMBL1 .EQ. 0.)GOTO 10171
      SYMBL = SYMIN(1,N)
      IF (SYMBL.EQ.NOSYMB) SYMBL=SYMBOL
10171 CONTINUE
      IF (XVALS(1,1).NE.HNONE) XYZV(1)=XVALS(1,N)
      IF (YVALS(1,1).NE.HNONE) XYZV(2)=YVALS(1,N)
      IF (ZVALS(1,1).NE.HNONE) XYZV(3)=ZVALS(1,N)
      IF (XERRS(1,1).NE.HNONE) XYZE(1)=XERRS(1,N)
      IF (YERRS(1,1).NE.HNONE) XYZE(2)=YERRS(1,N)
      IF (ZERRS(1,1).NE.HNONE) XYZE(3)=ZERRS(1,N)
      IWRT=0
      IF(.NOT.(FLAGS(29)))GOTO 10191
      IF(N .GT. 3)GOTO 10211
      WRITE(DBGFIL,10220)N,XYZV,XYZE,SYMBL
10220 FORMAT (50X,I1,    3H VL,3G10.3,    3H ER,3G10.3,1X,G10.3)
10211 CONTINUE
10191 CONTINUE
      DO 10231 IXYZ=1,3
      ERR=XYZE(IXYZ)
      IF(ERR .EQ. 0.)GOTO 10251
      IWRT=1
      DO 10261 J=1,3
      XYZE1(J)=XYZV(J)
10261 CONTINUE
10262 CONTINUE
      XYZE1(IXYZ)=XYZV(IXYZ)+ERR
      CALL T2XFRM(3,1,XYZE1,NONLIN,XFRM12,3,XYZE2(1,1))
      XYZE1(IXYZ)=XYZV(IXYZ)-ERR
      CALL T2XFRM(3,1,XYZE1,NONLIN,XFRM12,3,XYZE2(1,2))
      II=1
      IF (IXYZ.EQ.1) II=2
      IF(BAR .EQ. 0.)GOTO 10281
      SAVE=XYZE2(II,1)
      XYZE2(II,1)=SAVE+BAR
      CALL T2XFRM(3,1,XYZE2(1,1),LINEAR,XFRM24,  2,XY(1,1))
      XYZE2(II,1)=SAVE-BAR
      CALL T2XFRM(3,1,XYZE2(1,1),LINEAR,XFRM24,  2,XY(1,2))
      XYZE2(II,1)=SAVE
      CALL TXLINX(XY,2,LINTEX)
10281 CONTINUE
      CALL T2XFRM(3,2,XYZE2,LINEAR,XFRM24,2,XY)
      CALL TXLINX(XY,2,LINTEX)
      IF(BAR .EQ. 0.)GOTO 10301
      SAVE=XYZE2(II,2)
      XYZE2(II,2)=XYZE2(II,2)+BAR
      CALL T2XFRM(3,1,XYZE2(1,2),LINEAR,XFRM24,  2,XY(1,1))
      XYZE2(II,2)=SAVE-BAR
      CALL T2XFRM(3,1,XYZE2(1,2),LINEAR,XFRM24,  2,XY(1,2))
      XYZE2(II,2)=SAVE
      CALL TXLINX(XY,2,LINTEX)
10301 CONTINUE
10251 CONTINUE
10231 CONTINUE
10232 CONTINUE
      IF((.NOT.(.NOT.FLAGS(56))) .AND. (SYMBL .NE. NOSYMB))GOTO 10321
      CALL T2XFRM(3,1,XYZV,NONLIN,XFRM14,2,XY)
      GOTO 10331
10321 CONTINUE
      CALL T2XFRM(3,1,XYZV,NONLIN,XFRM12,3,XYZW)
      CALL T2XFRM(3,1,XYZW,LINEAR,XFRM24,2,XY)
10331 CONTINUE
10311 CONTINUE
      IF(SYMBL .EQ. NOSYMB)GOTO 10351
      IF(.NOT.(FLAGS(56)))GOTO 10371
      DIST=0.
      DO 10381 I=1,3
      DIST=DIST+VUEDIR(I)*(XYZW(I)-EYEPNT(I))
10381 CONTINUE
10382 CONTINUE
      SIZNEW=SYMSIZ*SCRD/AMAX1(0.0001,ABS(DIST))
10371 CONTINUE
      IF(ABS(SIZNEW-SIZNOW) .LE. ABS(0.005*SIZNOW))GOTO 10401
      CALL TXTEXT(XYZLIM(1,1),XYZLIM(2,1), SIZNEW,0.,DUMMY,0,DUMMY,0)
      SIZNOW=SIZNEW
10401 CONTINUE
      IWRT=1
      CALL TXTSYM(XY(1,1),XY(2,1),SYMBL)
10351 CONTINUE
      IF (IWRT.EQ.0) CALL TXDOT(XY(1,1),XY(2,1))
10151 CONTINUE
10152 CONTINUE
      CALL TXSCIS(1)
10051 CONTINUE
      CALL T2TCKS
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2ARRO(XYZARO,MODARO,DARO)
      REAL XYZARO(3,2),DARO(4)
      INTEGER MODARO(2)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL XYB(2,2),  XYH(2,4),  XY(2,4)
      IF(.NOT.(FLAGS(29)))GOTO 10421
      WRITE(DBGFIL,10430)ASIZE,AFLARE,XYZARO,DARO,MODARO
10430 FORMAT (60X,   18HT2ARRO. SIZE,FLARE,2F6.1/  62X,    5HXYZF ,3F10.
     *3/  62X,    5HXYZT ,3F10.3/  62X,    5HDF,DT,2F8.1,    8H  MODF,T,
     *2I3)
10421 CONTINUE
      IF (.NOT.FLAGS(73)) CALL TXDEF1
      IF(MODARO(1)+MODARO(2) .EQ. 0)GOTO 10451
      IF((FLAGS(75)))GOTO 10471
      CALL T2STLM
      CALL TXDEF2
10471 CONTINUE
      IF((FLAGS(75)))GOTO 10491
      WRITE(ERRFIL,10500)
10500 FORMAT(   40H *** WARNING *** DATA SYSTEM NOT DEFINED)
      RETURN
10491 CONTINUE
10451 CONTINUE
      DO 10511 I=1,2
      IF(MODARO(I) .EQ. 0)GOTO 10531
      CALL T2XFRM(3,1,XYZARO(1,I),NONLIN,XFRM13,2,XY(1,I))
      GOTO10521
10531 IF(.NOT.(FLAGS(56)))GOTO 10541
      CALL T2XFRM(3,1,XYZARO(1,I),LINEAR,XFRM23,2,XY(1,I))
      GOTO 10551
10541 CONTINUE
      XY(1,I)=XYZARO(1,I)
      XY(2,I)=XYZARO(2,I)
10551 CONTINUE
10521 CONTINUE
10511 CONTINUE
10512 CONTINUE
      IF((XY(1,2) .EQ. XY(1,1)) .AND. (XY(2,2) .EQ. XY(2,1)))GOTO 10571
      DX = XY(1,2)-XY(1,1)
      DY = XY(2,2)-XY(2,1)
      DSQ = DX**2+DY**2
      D = SQRT(DSQ)
      SINE = DY/D
      COSINE = DX/D
      XYH(1,3) = XY(1,2) - COSINE*DARO(2)
      XYH(2,3) = XY(2,2) - SINE*DARO(2)
      XYB(1,1) = XY(1,1) + COSINE*DARO(1)
      XYB(2,1) = XY(2,1) + SINE*DARO(1)
      DR = 0.1*REDUCE(2)*DARO(3)
      DX = COSINE*DR
      DY = SINE*DR
      XYB(1,2) = XYH(1,3) - DX
      XYB(2,2) = XYH(2,3) - DY
      DR = 0.5*DARO(4)
      SAVE = DX
      DX = DR*DY
      XYH(1,1) = XYB(1,2) + DX
      XYH(1,2) = XYB(1,2) - DX
      DY =-DR*SAVE
      XYH(2,1) = XYB(2,2) + DY
      XYH(2,2) = XYB(2,2) - DY
      XYH(1,4) = XYH(1,1)
      XYH(2,4) = XYH(2,1)
      CALL T2XFRM(2,2,XYB,LINEAR,XFRM34,2,XY)
      CALL TXLINX(XY,2,LINTEX)
      CALL T2XFRM(2,4,XYH,LINEAR,XFRM34,2,XY)
      CALL TXLINX(XY,4,4)
10571 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2CRCL(XYZCRC,MODCRC,SIZCRC)
      REAL XYZCRC(3),SIZCRC(2)
      INTEGER MODCRC(2)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL CRCFCX(27),CRCFCY(27)
      INTEGER CRCPNT(4), CRCLVL(3)
      REAL XYCEN(2),CRCX(20),CRCY(20)
      DATA CRCPNT /0,5,10,27/
      DATA CRCLVL /1,1,5/
      DATA CRCFCX / 0.5,-0.5,-0.5, 0.5, 0.5,  0.5, 0.,-0.5, 0., 0.5,  0.
     *,.1964,.3535,.4620,  0.5,.4620,.3535,.1964,  0.,-.1964,-.3535,-.46
     *20,  -0.5,-.4620,-.3535,-.1964,0./
      DATA CRCFCY / 0.5, 0.5,-0.5,-0.5, 0.5,  0., 0.5, 0.,-0.5, 0.,  0.5
     *,.4620,.3535,.1964,  0.,-.1964,-.3535,-.4620,  -0.5,-.4620,-.3535,
     *-.1964,  0.,.1964,.3535,.4620,0.5/
      IF(.NOT.(FLAGS(29)))GOTO 10591
      WRITE(DBGFIL,10600)XYZCRC,SIZCRC,MODCRC
10600 FORMAT(60X,    6HCIRCLE,3G10.3/60X,2G10.3,2I10)
10591 CONTINUE
      IF(MODCRC(1) .NE. 0)GOTO 10621
      IF (.NOT.FLAGS(73)) CALL TXDEF1
      XYCEN(1) = XYZCRC(1)
      XYCEN(2) = XYZCRC(2)
      GOTO 10631
10621 CONTINUE
      IF (.NOT.FLAGS(75)) CALL TXDEF2
      CALL T2XFRM(3,1,XYZCRC,NONLIN,XFRM13,2,XYCEN)
10631 CONTINUE
10611 CONTINUE
      MODE = MODCRC(2)
      J = CRCPNT(MODE)
      NP = CRCPNT(MODE+1) - J
      DO 10641 I=1,NP
      J = J+1
      CRCX(I) = XYCEN(1)+SIZCRC(1)*CRCFCX(J)
      CRCY(I) = XYCEN(2)+SIZCRC(2)*CRCFCY(J)
10641 CONTINUE
10642 CONTINUE
      FLAGS(31) = .TRUE.
      CALL T2JOIN(CRCX,CRCY,    4HNONE,2,CRCLVL(MODE),NP,1)
      FLAGS(31) = .FALSE.
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2HIST (XVALS,YVALS,ZVALS,XERRS,YERRS,ZERRS,JTYPE,INLEV
     *L,NP,INCRMT)
      INTEGER JTYPE,INLEVL,NP
      REAL XVALS(INCRMT,NP),YVALS(INCRMT,NP),ZVALS(INCRMT,NP),  XERRS(IN
     *CRMT,NP),YERRS(INCRMT,NP),ZERRS(INCRMT,NP)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL WID(3),XYZ(3)
      INTEGER IE(3),IV(3)
      LOGICAL LIMSET(3)
      DATA HNONE/    4HNONE/
      IF (NP.LE.1) R E T U R N
      IF(.NOT.(FLAGS(29)))GOTO 10661
      WRITE(DBGFIL,10670)NP
10670 FORMAT(60X,   18HT2HIST CALLED WITH,I6,    7H POINTS)
      N=MIN0(NP,6)
      WRITE (DBGFIL,10680) XVALS(1,1),(XVALS(1,I),I=1,N)
      WRITE (DBGFIL,10680) YVALS(1,1),(YVALS(1,I),I=1,N)
      WRITE (DBGFIL,10680) ZVALS(1,1),(ZVALS(1,I),I=1,N)
      WRITE (DBGFIL,10680) XERRS(1,1),(XERRS(1,I),I=1,N)
      WRITE (DBGFIL,10680) YERRS(1,1),(YERRS(1,I),I=1,N)
      WRITE (DBGFIL,10680) ZERRS(1,1),(ZERRS(1,I),I=1,N)
10680 FORMAT(60X,A4,6G10.3)
10661 CONTINUE
      FLAGS(4) = .TRUE.
      FLAGS(28) = .TRUE.
      DO 10691 I=61,72
      FLAGS(I)=.FALSE.
10691 CONTINUE
10692 CONTINUE
      DO 10701 I=1,3
      LIMSET(I)=FLAGS(2*I+3)
10701 CONTINUE
10702 CONTINUE
      CALL T2AXES
      CALL TXSCIS(2)
      IDEP=2
      IF (YVALS(1,1).EQ.HNONE .OR. FLAGS(56)) IDEP=3
      IF((LIMSET(IDEP)) .OR. NONLIN(IDEP) .NE. -1 .OR. XYZLIM(IDEP,1) .L
     *E. 0.)GOTO 10721
      XYZLIM(IDEP,1)=0.
      CALL TXDEF2
10721 CONTINUE
      ISAVE = LINTEX
      ITXTUR = JTYPE/64
      IF (ITXTUR.NE.0) LINTEX = ITXTUR
      DO 10731 I=1,3
      XYZ(I)=XYZLIM(I,1)
      IV(I)=0
      IE(I)=0
10731 CONTINUE
10732 CONTINUE
      IF (XVALS(1,1).NE.HNONE) IV(1)=1
      IF (XERRS(1,1).NE.HNONE) IE(1)=1
      IF (YVALS(1,1).NE.HNONE) IV(2)=1
      IF (YERRS(1,1).NE.HNONE) IE(2)=1
      IF (ZVALS(1,1).NE.HNONE) IV(3)=1
      IF (ZERRS(1,1).NE.HNONE) IE(3)=1
      IV(IDEP)=0
      IF(INLEVL .NE. 0)GOTO 10751
      IF(NONLIN(IDEP) .NE. (-1))GOTO 10771
      XYZ(IDEP)=0.
      GOTO 10781
10771 CONTINUE
      XYZ(IDEP)=XYZLIM(IDEP,1)
10781 CONTINUE
10761 CONTINUE
      YSTART=XYZ(IDEP)
      MAXJ = BUFSIZ - 15
      JXB=3
      IF(.NOT.(.NOT.FLAGS(56)))GOTO 10801
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
10801 CONTINUE
      IF(IV(1) .EQ. 0)GOTO 10821
      XYZ(1) = 3.*XVALS(1,1) - XVALS(1,2)
      IF (IE(1).NE.0) XYZ(1) = XYZ(1)+XERRS(1,2)-XERRS(1,1)
      XYZ(1)=0.5*XYZ(1)
10821 CONTINUE
      IF(IV(2) .EQ. 0)GOTO 10841
      XYZ(2) = 3.*YVALS(1,1) - YVALS(1,2)
      IF (IE(2).NE.0) XYZ(2) = XYZ(2)+YERRS(1,2)-YERRS(1,1)
      XYZ(2)=0.5*XYZ(2)
10841 CONTINUE
      IF(IV(3) .EQ. 0)GOTO 10861
      XYZ(3) = 3.*ZVALS(1,1) - ZVALS(1,2)
      IF (IE(3).NE.0) XYZ(3) = XYZ(3)+ZERRS(1,2)-ZERRS(1,1)
      XYZ(3)=0.5*XYZ(3)
10861 CONTINUE
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
      DO 10871 I=2,NP
      IF (IDEP-2) 10880,10890,10900
10880 CONTINUE
      XYZ(1)=XVALS(1,I-1)
      GO TO 10910
10890 CONTINUE
      XYZ(2)=YVALS(1,I-1)
      GO TO 10910
10900 CONTINUE
      XYZ(3)=ZVALS(1,I-1)
10910 CONTINUE
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
      IF(IV(1) .EQ. 0)GOTO 10931
      XYZ(1) = XVALS(1,I-1) + XVALS(1,I)
      IF (IE(1).NE.0) XYZ(1)=XYZ(1)-XERRS(1,I)+XERRS(1,I-1)
      XYZ(1)=XYZ(1)*0.5
10931 CONTINUE
      IF(IV(2) .EQ. 0)GOTO 10951
      XYZ(2) = YVALS(1,I-1) + YVALS(1,I)
      IF (IE(2).NE.0) XYZ(2)=XYZ(2)-YERRS(1,I)+YERRS(1,I-1)
      XYZ(2)=XYZ(2)*0.5
10951 CONTINUE
      IF(IV(3) .EQ. 0)GOTO 10971
      XYZ(3) = ZVALS(1,I-1) + ZVALS(1,I)
      IF (IE(3).NE.0) XYZ(3)=XYZ(3)-ZERRS(1,I)+ZERRS(1,I-1)
      XYZ(3)=XYZ(3)*0.5
10971 CONTINUE
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
      IF(JXB .LT. MAXJ)GOTO 10991
      JXB=(JXB-3)/3
      CALL T2XFRM(3,JXB,BUFFER(4),  NONLIN,XFRM14,2,BUFFER)
      CALL TXLINX(BUFFER,JXB,LINTEX)
      JXB=3
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
10991 CONTINUE
10871 CONTINUE
10872 CONTINUE
      IF (IDEP-2) 11000,11010,11020
11000 CONTINUE
      XYZ(1)=XVALS(1,I-1)
      GO TO 11030
11010 CONTINUE
      XYZ(2)=YVALS(1,I-1)
      GO TO 11030
11020 CONTINUE
      XYZ(3)=ZVALS(1,I-1)
11030 CONTINUE
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
      IF(IV(1) .EQ. 0)GOTO 11051
      XYZ(1) = 3.*XVALS(1,NP) - XVALS(1,NP-1)
      IF (IE(1).NE.0) XYZ(1)=XYZ(1)+XERRS(1,NP)-XERRS(1,NP-1)
      XYZ(1)=XYZ(1)*0.5
11051 CONTINUE
      IF(IV(2) .EQ. 0)GOTO 11071
      XYZ(2) = 3.*YVALS(1,NP) - YVALS(1,NP-1)
      IF (IE(2).NE.0) XYZ(2)=XYZ(2)+YERRS(1,NP)-YERRS(1,NP-1)
      XYZ(2)=XYZ(2)*0.5
11071 CONTINUE
      IF(IV(3) .EQ. 0)GOTO 11091
      XYZ(3) = 3.*ZVALS(1,NP) - ZVALS(1,NP-1)
      IF (IE(3).NE.0) XYZ(3)=XYZ(3)+ZERRS(1,NP)-ZERRS(1,NP-1)
      XYZ(3)=XYZ(3)*0.5
11091 CONTINUE
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
      XYZ(IDEP) = YSTART
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
      IF(.NOT.(.NOT.FLAGS(56)))GOTO 11111
      XYZ(1) = XYZLIM(1,2)
      BUFFER(JXB+1)=XYZ(1)
      BUFFER(JXB+2)=XYZ(2)
      BUFFER(JXB+3)=XYZ(3)
      JXB=JXB+3
11111 CONTINUE
      JXB=(JXB-3)/3
      CALL T2XFRM(3,JXB,BUFFER(4),NONLIN,XFRM14,2,BUFFER)
      CALL TXLINX(BUFFER,JXB,LINTEX)
      GOTO 11121
10751 CONTINUE
      J=0
      MAJOR=0
      DO 11131 I=1,3
      IF(IV(I) .EQ. 0)GOTO 11151
      J=I
      IF (IE(I).NE.0) MAJOR=I
11151 CONTINUE
11131 CONTINUE
11132 CONTINUE
      IF (MAJOR.EQ.0) MAJOR=J
      IF (MAJOR.EQ.0) RETURN
      MINOR=6-IDEP-MAJOR
      YSTART = AMAX1(0.,XYZLIM(IDEP,1))
      IF((FLAGS(56)))GOTO 11171
      IF(.NOT.((SIGN(1.,XYZLIM(IDEP,1)-YSTART).EQ.SIGN(1.,YSTART-XYZLIM(
     *IDEP,2)))))GOTO 11191
      SAV1 = XYZLIM(IDEP,1)
      SAV2 = XYZLIM(IDEP,2)
      XYZLIM(IDEP,1) = YSTART
      XYZLIM(IDEP,2) = YSTART
      CALL T2XFRM(3,2,XYZLIM,NONLIN,XFRM14,2,BUFFER)
      XYZLIM(IDEP,1) = SAV1
      XYZLIM(IDEP,2) = SAV2
      CALL TXLINX(BUFFER,2,LINTEX)
11191 CONTINUE
11171 CONTINUE
      WIDDF=1.E30
      IF(MAJOR .NE. 1)GOTO 11211
      DO 11221 I=2,NP
      W=XVALS(1,I)-XVALS(1,I-1)
      IF (W.NE.0.) WIDDF=AMIN1(WIDDF,ABS(W))
11221 CONTINUE
11222 CONTINUE
11211 CONTINUE
      IF(MAJOR .NE. 2)GOTO 11241
      DO 11251 I=2,NP
      W = YVALS(1,I)-YVALS(1,I-1)
      IF (W.NE.0.) WIDDF=AMIN1(WIDDF,ABS(W))
11251 CONTINUE
11252 CONTINUE
11241 CONTINUE
      IF(MAJOR .NE. 3)GOTO 11271
      DO 11281 I=2,NP
      W = ZVALS(1,I)-ZVALS(1,I-1)
      IF (W.NE.0.) WIDDF=AMIN1(WIDDF,ABS(W))
11281 CONTINUE
11282 CONTINUE
11271 CONTINUE
      IF (NONLIN(MAJOR).EQ.-4 .OR. NONLIN(MAJOR).EQ.-5) WIDDF = AMIN1(WI
     *DDF,0.05)
      WIDDF=0.33*WIDDF
      WIDTH=WIDDF
      DO 11291 I=1,3
      BUFFER(I+3)=XYZLIM(I,1)
      IF (SIGN(1.,XYZLIM(I,1)) .NE. SIGN(1.,XYZLIM(I,2)) ) BUFFER(I+3)=0
     *.
11291 CONTINUE
11292 CONTINUE
      BUFFER(IDEP+9)=BUFFER(IDEP+3)
      BUFFER(IDEP)=BUFFER(IDEP+3)
      IF(.NOT.(FLAGS(29)))GOTO 11311
      WRITE(DBGFIL,11320)IDEP,MAJOR,MINOR
11320 FORMAT(60X,   16HIDEP,MAJOR,MINOR,3I3)
11311 CONTINUE
      DO 11331 I=1,NP
      IF(IE(MAJOR) .EQ. 0)GOTO 11351
      WIDTH=0.
      IF (MAJOR-2) 11360,11370,11380
11360 CONTINUE
      WIDTH=XERRS(1,I)
      GO TO 11390
11370 CONTINUE
      WIDTH=YERRS(1,I)
      GO TO 11390
11380 CONTINUE
      WIDTH=ZERRS(1,I)
11390 CONTINUE
      IF (WIDTH.EQ.0.) WIDTH=WIDDF
11351 CONTINUE
      IF (XVALS(1,1).NE.HNONE) BUFFER(4)=XVALS(1,I)
      IF (YVALS(1,1).NE.HNONE) BUFFER(5)=YVALS(1,I)
      IF (ZVALS(1,1).NE.HNONE) BUFFER(6)=ZVALS(1,I)
      DO 11401 J=1,3
      IF(NONLIN(J).NE.-4 .AND. NONLIN(J).NE.-5)GOTO 11401
      IF (ABS( BUFFER(J+3)-AINT(BUFFER(J+3)) ).LT.0.002) BUFFER(J+3)=BUF
     *FER(J+3)+0.16
11401 CONTINUE
11402 CONTINUE
      BUFFER(MAJOR+6)=BUFFER(MAJOR+3)+WIDTH
      BUFFER(MAJOR+9)=BUFFER(MAJOR+6)
      BUFFER(MAJOR+3)=BUFFER(MAJOR+3)-WIDTH
      BUFFER(MAJOR)=BUFFER(MAJOR+3)
      BUFFER(IDEP+6)=BUFFER(IDEP+3)
      BUFFER(MINOR)=BUFFER(MINOR+3)
      BUFFER(MINOR+6)=BUFFER(MINOR+3)
      BUFFER(MINOR+9)=BUFFER(MINOR+3)
      CALL T2XFRM(3,4,BUFFER,NONLIN,XFRM14,2,BUFFER(13))
      CALL TXLINX(BUFFER(13),4,LINTEX)
11331 CONTINUE
11332 CONTINUE
11121 CONTINUE
10741 CONTINUE
      LINTEX = ISAVE
      CALL TXSCIS(1)
      CALL T2TCKS
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2JOIN  (XVALS,YVALS,ZVALS,JTYPE,INLEVL,NP,INCRMT)
      INTEGER JTYPE,INLEVL,NP,INCRMT
      DIMENSION XVALS(INCRMT,NP),YVALS(INCRMT,NP),ZVALS(INCRMT,NP)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1  STRING
      INTEGER FIRST,FIRSTJ,OVERLP,LAST,NDIM,ISC(4)
      REAL HNONE,XY(2),XYZ(3),XFRM(12)
      LOGICAL CLOSED
      LOGICAL XEXIST,YEXIST,ZEXIST
      DATA HNONE/    4HNONE/
      IF(.NOT.(FLAGS(29)))GOTO 11421
      WRITE(DBGFIL,11430)NP,INLEVL,JTYPE
11430 FORMAT (60X,   13HT2JOIN CALLED,I6,   14H POINTS  LEVEL,I2,    6H
     * TYPE,I2)
11421 CONTINUE
      IF (NP.LE.1) R E T U R N
      IF(.NOT.(FLAGS(29)))GOTO 11451
      LAST=MIN0(NP,25)
      WRITE(DBGFIL,11460)(XVALS(1,I),YVALS(1,I),ZVALS(1,I),I=1,LAST)
11460 FORMAT (64X,    1HX,9X,    1HY,9X,    1HZ,25(/60X,3G10.3))
11451 CONTINUE
      FLAGS(4) = .TRUE.
      FLAGS(28) = .TRUE.
      DO 11471 I=61,72
      FLAGS(I)=.FALSE.
11471 CONTINUE
11472 CONTINUE
      DO 11481 I=1,3
      XYZ(I)=XYZLIM(I,1)
11481 CONTINUE
11482 CONTINUE
      XEXIST=XVALS(1,1).NE.HNONE
      YEXIST=YVALS(1,1).NE.HNONE
      ZEXIST=ZVALS(1,1).NE.HNONE
      IF(.NOT.(FLAGS(31)))GOTO 11501
      IF (.NOT.FLAGS(73)) CALL TXDEF1
      NDIM=2
      DO 11511 I=1,3
      ISC(I)=LINEAR(I)
      XFRM(I)=XFRM34(I)
      XFRM(I+3)=XFRM34(I+3)
11511 CONTINUE
11512 CONTINUE
      ISC(4)=LINEAR(4)
      GOTO 11521
11501 CONTINUE
      CALL T2AXES
      CALL TXSCIS(2)
      NDIM=3
      DO 11531 I=1,4
      ISC(I)=NONLIN(I)
11531 CONTINUE
11532 CONTINUE
      DO 11541 I=1,12
      XFRM(I)=XFRM14(I)
11541 CONTINUE
11542 CONTINUE
11521 CONTINUE
11491 CONTINUE
      LEVEL=INLEVL
      IF (NP.EQ.2 .AND. LEVEL.EQ.0) LEVEL=1
      IF (FLAGS(56) .AND. LEVEL.EQ.0) LEVEL=1
      IF((LEVEL .GT. 0) .AND. (LEVEL .LE. 20))GOTO 11561
      IF(.NOT.(FLAGS(31)))GOTO 11581
      LEVEL=1
      GOTO 11591
11581 CONTINUE
      MAX=1
      LEVEL=10
11601 IF(MAX.GE.NP) GOTO 11602
      MAX=MAX+MAX
      LEVEL=LEVEL-1
      GOTO 11601
11602 CONTINUE
      IF (LEVEL.LE.0) LEVEL = 1
11591 CONTINUE
11571 CONTINUE
11561 CONTINUE
      IXB = 1
      ITXTUR = JTYPE/64
      IF (ITXTUR.EQ.0) ITXTUR = LINTEX
      LM1 = LEVEL - 1
      LP1 = LEVEL + 1
      NPM1 = NP - 1
      LAST = 0
      IF (LEVEL.EQ.1) GO TO 11610
      JJTYPE = MOD(JTYPE,64)
      IF (JJTYPE.EQ.0) JJTYPE=2
      IF(JJTYPE .NE. 1)GOTO 11631
      IF(NP .GE. 4)GOTO 11651
      IF(MOD(JTYPE,64) .NE. 1)GOTO 11671
      WRITE(ERRFIL,11680)
11680 FORMAT(' *** WARNING *** SPLINE FIT NEEDS 4 POINTS.',
     *' GENERAL FIT USED.')
11671 CONTINUE
      GO TO 11610
11651 CONTINUE
      NBUFF = (BUFSIZ - 2*LP1)/5
      IF(NBUFF .GE. NP)GOTO 11701
      IF(MOD(JTYPE,64) .NE. 1)GOTO 11721
      WRITE(ERRFIL,11730)NP,NBUFF
11730 FORMAT (   16H *** WARNING ***,I6,   25H POINTS EXCEEDS MAXIMUM (,
     *I3,     35H) FOR SPLINE FIT. GENERAL FIT USED.)
11721 CONTINUE
      GO TO 11610
11701 CONTINUE
      IYB=IXB+NBUFF
      IXP=IYB+NBUFF
      IXPP=IXP+NBUFF
      IXPPP=IXPP+NBUFF
      ILX=IXPPP+NBUFF
      ILY=ILX+LP1
      MINX = IXB - 1
      MINY = IYB - 1
      DO 11741 I=1,NP
      MINX=MINX+1
      MINY=MINY+1
      IF (XEXIST) XYZ(1)=XVALS(1,I)
      IF (YEXIST) XYZ(2)=YVALS(1,I)
      IF (ZEXIST) XYZ(3)=ZVALS(1,I)
      CALL T2XFRM(NDIM,1,XYZ,ISC,XFRM,2,XY)
      BUFFER(MINX) = XY(1)
      BUFFER(MINY) = XY(2)
11741 CONTINUE
11742 CONTINUE
      MINX = IXB
      MAXX = IXB+NP-2
      MINY = IYB
      MAXY = IYB+NP-2
11750 CONTINUE
11751 CONTINUE
      DO 11761 I=MINX,MAXX
      IF(BUFFER(I+1) .GT. BUFFER(I))GOTO 11781
      DO 11791 J=MINX,MAXX
      IF(BUFFER(J+1) .LT. BUFFER(J))GOTO 11811
      DO 11821 K=MINY,MAXY
      IF(BUFFER(K+1) .GT. BUFFER(K))GOTO 11841
      DO 11851 L=MINY,MAXY
      IF(BUFFER(L+1) .LT. BUFFER(L))GOTO 11871
      IF(MOD(JTYPE,64) .NE. 1)GOTO 11891
      WRITE(ERRFIL,11900)
11900 FORMAT (   34H *** WARNING *** SPLINE FIT NEEDS ,     19HMONOTONIC
     * POINTS.    ,   17HGENERAL FIT USED.)
11891 CONTINUE
      GO TO 11610
11871 CONTINUE
11851 CONTINUE
11852 CONTINUE
      IXMONO = 0
      GOTO 11752
11841 CONTINUE
11821 CONTINUE
11822 CONTINUE
      IXMONO = 0
      GOTO 11752
11811 CONTINUE
11791 CONTINUE
11792 CONTINUE
      IXMONO = 1
      GOTO 11752
11781 CONTINUE
11761 CONTINUE
11762 CONTINUE
      IXMONO = 1
11752 CONTINUE
      IF(IXMONO .NE. 1)GOTO 11921
      IXB1=IXB
      IYB1=IYB
      ILX1=ILX
      ILY1=ILY
      GOTO 11931
11921 CONTINUE
      IXB1=IYB
      IYB1=IXB
      ILX1=ILY
      ILY1=ILX
11931 CONTINUE
11911 CONTINUE
      CALL T2SPLN(NP,BUFFER(IXB1),BUFFER(IYB1),BUFFER(IXP),  BUFFER(IXPP
     *),BUFFER(IXPPP))
      DO 11941 I=1,NPM1
      X=BUFFER(IXB1)
      IXB1=IXB1+1
      DXX=(BUFFER(IXB1)-X)/FLOAT(LEVEL)
      Y=BUFFER(IYB1)
      IYB1=IYB1+1
      XP = BUFFER(IXP)
      IXP=IXP+1
      XPP = BUFFER(IXPP)
      IXPP=IXPP+1
      XPPP = BUFFER (IXPPP)
      IXPPP=IXPPP+1
      JJX = ILX1
      JJY = ILY1
      DO 11951 J=1,LEVEL
      DX = FLOAT(J-1)*DXX
      BUFFER(JJY) = Y + DX*(XP+DX*(XPP+DX*XPPP))
      BUFFER(JJX) = X + DX
      JJX = JJX+1
      JJY = JJY+1
11951 CONTINUE
11952 CONTINUE
      BUFFER(JJX) = BUFFER(IXB1)
      BUFFER(JJY) = BUFFER(IYB1)
      CALL TXLINE(BUFFER(ILX),BUFFER(ILY),LP1,ITXTUR)
11941 CONTINUE
11942 CONTINUE
      GOTO 11961
11631 CONTINUE
11610 CONTINUE
      JJTYPE = 2
      CLOSED=NP.GT.2
      IF(.NOT.(XEXIST))GOTO 11981
      IF (XVALS(1,1).NE.XVALS(1,NP)) CLOSED=.FALSE.
11981 CONTINUE
      IF(.NOT.(YEXIST))GOTO 12001
      IF (YVALS(1,1).NE.YVALS(1,NP)) CLOSED=.FALSE.
12001 CONTINUE
      IF(.NOT.(ZEXIST))GOTO 12021
      IF (ZVALS(1,1).NE.ZVALS(1,NP)) CLOSED=.FALSE.
12021 CONTINUE
      OVERLP = 2
      NBUFF = BUFSIZ/2
      MAXM1=(NBUFF-1)/LEVEL
      IYB=IXB+NBUFF
      FIRST = 1
      IF(.NOT.(CLOSED))GOTO 12041
      FIRSTJ=OVERLP*LEVEL
      MAXPNT=NP+2*OVERLP
      GOTO 12051
12041 CONTINUE
      FIRSTJ=0
      MAXPNT=NP
12051 CONTINUE
12031 CONTINUE
12061 IF(LAST .EQ. MAXPNT) GOTO 12062
      LAST = MIN0(MAXPNT,FIRST+MAXM1)
      LASTNP=MIN0(LAST,NP)
      N = LAST-FIRST+1
      NLOAD = LASTNP-FIRST+1
      IXBPTR=IXB-1
      IYBPTR=IYB-1
      DO 12071 I=FIRST,LASTNP
      IXBPTR=IXBPTR+1
      IYBPTR=IYBPTR+1
      IF (XEXIST) XYZ(1)=XVALS(1,I)
      IF (YEXIST) XYZ(2)=YVALS(1,I)
      IF (ZEXIST) XYZ(3)=ZVALS(1,I)
      CALL T2XFRM(NDIM,1,XYZ,ISC,XFRM,2,XY)
      BUFFER(IXBPTR)=XY(1)
      BUFFER(IYBPTR)=XY(2)
12071 CONTINUE
12072 CONTINUE
      I=1
12081 IF(LASTNP.GE.LAST) GOTO 12082
      I=I+1
      LASTNP=LASTNP+1
      IXBPTR=IXBPTR+1
      IYBPTR=IYBPTR+1
      IF (XEXIST) XYZ(1)=XVALS(1,I)
      IF (YEXIST) XYZ(2)=YVALS(1,I)
      IF (ZEXIST) XYZ(3)=ZVALS(1,I)
      CALL T2XFRM(NDIM,1,XYZ,ISC,XFRM,2,XY)
      BUFFER(IXBPTR)=XY(1)
      BUFFER(IYBPTR)=XY(2)
      GOTO 12081
12082 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12101
      WRITE(DBGFIL,12110)FIRST,LAST,LASTNP
12110 FORMAT (60X,   17HFIRST,LAST,LASTNP,3I5)
12101 CONTINUE
      CALL T2CFIT(N,BUFFER(IXB),  BUFFER(IYB),LEVEL,NJ,BUFFER(IXB),BUFFE
     *R(IYB))
      IF ((LAST.NE.MAXPNT) .OR. CLOSED) NJ=NJ-OVERLP*LEVEL
      IXBPTR=IXB+FIRSTJ
      IYBPTR=IYB+FIRSTJ
      CALL TXLINE(BUFFER(IXBPTR),BUFFER(IYBPTR),  NJ-FIRSTJ,ITXTUR)
      FIRSTJ=OVERLP*LEVEL
      FIRST = LAST - 2*OVERLP
      GOTO 12061
12062 CONTINUE
11961 CONTINUE
11621 CONTINUE
      IF((FLAGS(31)))GOTO 12131
      CALL TXSCIS(1)
      CALL T2TCKS
12131 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2CFIT(L,X,Y,M,N,U,V)
      INTEGER L,M,N
      REAL X(L),Y(L)
      REAL U(*),V(*)
      M0=MAX0(M,1)
      L0=MAX0(L,1)
      N0=(L0-1)*M0+1
      MM1=M0-1
      N=N0
      RM=1./FLOAT(M0)
      K2=N0+M0
      I=L0+1
      DO 12141 J=1,L0
      K2=K2-M0
      I=I-1
      U(K2)=X(I)
      V(K2)=Y(I)
12141 CONTINUE
12142 CONTINUE
      IF (N0.EQ.1 .OR. MM1.EQ.0) RETURN
      K2=1
      K5=1
      X3=U(1)
      Y3=V(1)
12151 CONTINUE
      K5=K5+M0
      X4=U(K5)
      Y4=V(K5)
      A3=X4-X3
      B3=Y4-Y3
      IF((A3 .EQ. 0.0) .AND. ((B3 .EQ. 0.0) .AND. (K5 .NE. N0)))GOTO 121
     *51
12152 CONTINUE
      IF (K5.NE.N0) K5=K5+M0
12161 CONTINUE
      X5=U(K5)
      Y5=V(K5)
      A4=X5-X4
      B4=Y5-Y4
      IF(A4.NE.0.0 .OR. B4.NE.0.0. OR. K5.EQ.N0)GOTO 12162
      K5=K5+M0
      GOTO 12161
12162 CONTINUE
      A2=A3+A3-A4
      B2=B3+B3-B4
      A1=A2+A2-A3
      B1=B2+B2-B3
      R23=SQRT(A2**2+B2**2)
      R34=SQRT(A3**2+B3**2)
      IRTRN=0
      GO TO 12170
12180 CONTINUE
      IRTRN=1
12191 CONTINUE
      X2=X3
      X3=X4
      X4=X5
      Y2=Y3
      Y3=Y4
      Y4=Y5
      A1=A2
      A2=A3
      A3=A4
      B1=B2
      B2=B3
      B3=B4
      COS2=COS3
      SIN2=SIN3
      R12=R23
      R23=R34
12201 CONTINUE
      IF(K5 .NE. N0)GOTO 12221
      B4=B3+B3-B2
      GOTO 12202
12221 CONTINUE
      K5=K5+M0
      X5=U(K5)
      Y5=V(K5)
      A4=X5-X4
      B4=Y5-Y4
      IF(A4.EQ.0.0 .AND. B4.EQ.0.0)GOTO 12201
12202 CONTINUE
12231 CONTINUE
      J=K2+M0
      IF(X2.NE.U(J).OR.Y2.NE.V(J))GOTO 12232
      DO 12241 J=1,MM1
      K2=K2+1
      U(K2)=X2
      V(K2)=Y2
12241 CONTINUE
12242 CONTINUE
      K2=K2+1
      IF(K2.GE.N0) RETURN
      GOTO 12231
12232 CONTINUE
12170 CONTINUE
      W2=ABS(A3*B4-A4*B3)
      W3=ABS(A1*B2-B1*A2)
      IF((W3+W2) .NE. 0.0)GOTO 12261
      W2=SQRT(A3**2+B3**2)
      W3=SQRT(A2**2+B2**2)
12261 CONTINUE
      COS3=W2*A2+W3*A3
      SIN3=W2*B2+W3*B3
      R3=COS3**2+SIN3**2
      IF(R3 .EQ. 0.0)GOTO 12281
      R3=SQRT(R3)
      COS3=COS3/R3
      SIN3=SIN3/R3
12281 CONTINUE
      IF (IRTRN.EQ.0) GO TO 12180
      R34=SQRT(A3**2+B3**2)
      IF((R12 .NE. 0.0) .AND. ((R23 .NE. 0.0) .AND. (R34 .NE. 0.0)))GOTO
     * 12301
      T2=R23
      T3=R23
      GOTO 12311
12301 CONTINUE
      CT2 = (A2*COS2+B2*SIN2)/R23
      CT3 = (A2*COS3+B2*SIN3)/R23
      DOT2 = A1*A2+B1*B2
      DOT3 = A2*A3+B2*B3
      R3 = 2.0*R23
      IF (DOT2.LT.0.0) R3=R3*(1.0+DOT2/(R12*R23))
      IF (DOT3.LT.0.0) R3=R3*(1.0+DOT3/(R23*R34))
      T2=R3/(1.0+0.667*CT3+0.333*CT2)
      T3=R3/(1.0+0.667*CT2+0.333*CT3)
12311 CONTINUE
12291 CONTINUE
      P1=T2*COS2
      P2=3.0*A2-2.0*P1-T3*COS3
      P3=A2-P1-P2
      Q1=T2*SIN2
      Q2=3.0*B2-2.0*Q1-T3*SIN3
      Q3=B2-Q1-Q2
      R=0.0
      DO 12321 J=1,MM1
      K2=K2+1
      R=R+RM
      U(K2)=X2+R*(P1+R*(P2+R*P3))
      V(K2)=Y2+R*(Q1+R*(Q2+R*Q3))
12321 CONTINUE
12322 CONTINUE
      K2=K2+1
      IF(K2 .LT. N0)GOTO 12191
12192 CONTINUE
      RETURN
      END
C SPLINE FITTER FROM FXB$CG.  I THINK THIS IS FROM THE STANFORD
C SPSS PACKAGE.
cnext ****************************************
       SUBROUTINE  T2SPLN (N, X, Y, B, C, D)
      INTEGER  N
      REAL  X(N), Y(N), B(N), C(N), D(N)
C
C  THE COEFFICIENTS B(I), C(I), AND D(I), I=1,2,...,N-1 ARE COMPUTED
C  FOR THE NATURAL CUBIC INTERPOLATING SPLINE
C
C     Y(X) = Y(I) + B(I)*(X-X(I)) + C(I)*(X-X(I))**2 + D(I)*(X-X(I))**3
C
C  FOR X(I) .LE. X .LE. X(I+1) .
C
C  INPUT..
C
C     N  THE NUMBER OF DATA POINTS OR KNOTS  (N.GE.4)
C     X  THE ABSCISSAS OF THE KNOTS IN STRICTLY INCREASING ORDER
C     Y  THE ORDINATES OF THE KNOTS
C
C  OUTPUT..
C
C     B, C, D  ARRAYS OF SPLINE COEFFICIENTS AS DEFINED ABOVE.
C     B(N)  HAS THE APPROPRIATE VALUE TO DEFINE
C     THE SPLINE FOR  X .GT. X(N) .
C
C  USING  P  TO DENOTE DIFFERENTIATION,
C
C     Y(I) = S(X(I))
C     B(I) = SP(X(I))
C     C(I) = SPP(X(I))/2
C     D(I) = SPPP(X(I))/6
C
      INTEGER  NM1, NM2, IB, I
      REAL T
      NM1 = N-1
      NM2 = N-2
C
C  SET UP TRIDIAGONAL SYSTEM
C
C  B = DIAGONAL, D = OFFDIAGONAL, C = RIGHT HAND SIDE.
C
      D(1) = X(2) - X(1)
      C(2) = (Y(2) - Y(1))/D(1)
      DO 10 I = 2,NM1
        D(I) = X(I+1) - X(I)
        B(I) = 2.*(D(I-1) + D(I))
        C(I+1) = (Y(I+1) - Y(I))/D(I)
        C(I) = C(I+1) - C(I)
10    CONTINUE
C
C  NATURAL SPLINE END CONDITIONS
C
      C(1) = 0.
      C(N) = 0.
C
C  FORWARD ELIMINATION
C
      DO 20 I = 3,NM1
        T = D(I-1)/B(I-1)
        B(I) = B(I) - T*D(I-1)
        C(I) = C(I) - T*C(I-1)
20    CONTINUE
C
C  BACK SUBSTITUTION
C
      C(NM1) = C(NM1)/B(NM1)
      DO 30 IB = 2,NM2
        I = N-IB
        C(I) = (C(I) - D(I)*C(I+1))/B(I)
30    CONTINUE
C
C  C(I) IS NOW THE SIGMA(I) OF THE TEXT
C
C  COMPUTE POLYNOMIAL COEFFICIENTS
C
      B(N) = (Y(N) - Y(NM1))/D(NM1) + D(NM1)*C(NM1)
      DO 40 I = 1,NM1
        B(I) = (Y(I+1) - Y(I))/D(I) - D(I)*(C(I+1) + 2.*C(I))
        D(I) = (C(I+1) - C(I))/D(I)
        C(I) = 3.*C(I)
40    CONTINUE
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
      SUBROUTINE T2SETC(INFOIN,CARDIN)
      INTEGER INFOIN(10)
cmt4       BYTE CARDIN(160)
      character*1 CARDIN(160)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4      BYTE STRING
      character*1  STRING
      CALL TOKEN(INFOIN,CARDIN,    4HSET;)
      IF(INTERP .EQ. 2)GOTO 10021
      INFOIN(3)=INFOIN(4)
10021 CONTINUE
      ISAVE = INFOIN(3)
      CALL T2SET1(INFOIN,CARDIN)
      IF (ISAVE.EQ.INFOIN(3)) CALL T2SET2(INFOIN,CARDIN)
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2CRMF(XVALUE,W,D,TEXT,IFIRST,ILAST)
      INTEGER W,D
      REAL XVALUE
ccccccccccccccccc 21-Aug-91 Modified by Y.Teramoto
cccc      BYTE TEXT(100)
cmt4       BYTE TEXT(*)
      character*1 TEXT(*)
ccccccccccccccccc end of modification
cmt4       BYTE BLANK,PERIOD,E,MINUS,STAR,DIGIT(10)
      character*1 BLANK,PERIOD,E,MINUS,STAR,DIGIT(10)
      DOUBLE PRECISION DFIVE,DTENTH,DTENM5,DPOWER,DPART,TENPWR(6)
      DATA DFIVE/5.0D0/,DTENTH/0.1D0/,DTENM5/1.D-5/
      DATA TENPWR(1)/1.D0/,TENPWR(2)/1.D1/,TENPWR(3)/1.D2/  ,TENPWR(4)/1
     *.D3/,TENPWR(5)/1.D4/,TENPWR(6)/1.D5/
      DATA BLANK /1H /,PERIOD/1H./,E/1HE/,MINUS/1H-/,STAR/1H*/
      DATA DIGIT/1H0,1H1,1H2,1H3,1H4,1H5,1H6,1H7,1H8,1H9/
      ICOL=0
      PART=ABS(XVALUE)
      DPART=PART
      IABSD = IABS(D)
      IABSW = IABS(W)
      LOG=0
      IF(PART .EQ. 0.)GOTO 10041
      IF(D .LT. 0 .OR. PART .GE. 10.)GOTO 10061
      GOTO10051
10061 IF(D .GE. 0 .OR. PART .GE. 10. .OR. PART .LT. 1.)GOTO 10071
      GOTO 10081
10071 CONTINUE
      AL1=ALOG10(PART)
      LOG=AL1
      AL2=LOG
      IF (AL2.GT.AL1) LOG=LOG-1
      IOVER5=LOG/5
      I=5*IOVER5-LOG
      IF(I .GE. 0)GOTO 10101
      I=I+5
      IOVER5=IOVER5+1
10101 CONTINUE
      DPART=DPART*TENPWR(I+1)
      IF(IOVER5 .EQ. 0)GOTO 10121
      IF(IOVER5 .GE. 0)GOTO 10141
      IOVER5=-IOVER5
      DO 10151 I=1,IOVER5
      DPART=DPART*TENPWR(6)
10151 CONTINUE
10152 CONTINUE
      GOTO 10161
10141 CONTINUE
      DO 10171 I=1,IOVER5
      DPART=DPART*DTENM5
10171 CONTINUE
10172 CONTINUE
10161 CONTINUE
10131 CONTINUE
10121 CONTINUE
10081 CONTINUE
10051 CONTINUE
      I=IABSD+1
      IF (D.GE.0) I=I+LOG
      IOVER5=I/5
      I=5*IOVER5-I
      IF(I .GE. 0)GOTO 10191
      I=I+5
      IOVER5=IOVER5+1
10191 CONTINUE
      DPOWER=DFIVE*TENPWR(I+1)
      DO 10201 I=1,IOVER5
      DPOWER=DPOWER*DTENM5
10201 CONTINUE
10202 CONTINUE
      DPART=DPART+DPOWER
      IF(DPART .LT. TENPWR(2))GOTO 10221
      LOG=LOG+1
      DPART=DPART*DTENTH
10221 CONTINUE
10041 CONTINUE
      IWHOLE=2
      IF (D.GE.0 .AND. LOG.GT.0) IWHOLE=LOG+2
      LEAD=IABSW-IWHOLE-IABSD
      IF (D.LT.0) LEAD=LEAD-4
      IF (XVALUE.LT.0.) LEAD=LEAD-1
      IF (LEAD.LT.0) GO TO 10230
      IFIRST=LEAD+1
      IF(LEAD .LE. 0)GOTO 10251
      DO 10261 I=1,LEAD
      ICOL=ICOL+1
      TEXT(ICOL)=BLANK
10261 CONTINUE
10262 CONTINUE
10251 CONTINUE
      IF(XVALUE .GE. 0.)GOTO 10281
      ICOL=ICOL+1
      TEXT(ICOL)=MINUS
10281 CONTINUE
      N=IWHOLE+IABSD
      DO 10291 I=1,N
      IF(I .NE. IWHOLE)GOTO 10311
      ILAST=ICOL
      ICOL=ICOL+1
      TEXT(ICOL)=PERIOD
      GOTO 10321
10311 CONTINUE
      J=DPART
      ICOL=ICOL+1
      TEXT(ICOL)=DIGIT(J+1)
      DPOWER=J
      DPART=(DPART-DPOWER)*TENPWR(2)
      IF (J.NE.0) ILAST=ICOL
10321 CONTINUE
10301 CONTINUE
10291 CONTINUE
10292 CONTINUE
      IF(D .GE. 0 .OR. XVALUE .EQ. 0.)GOTO 10341
      ICOL=ICOL+1
      TEXT(ICOL)=E
      IF(LOG .LT. 0)GOTO 10361
      ICOL=ICOL+1
      TEXT(ICOL)=BLANK
      GOTO 10371
10361 CONTINUE
      ICOL=ICOL+1
      TEXT(ICOL)=MINUS
10371 CONTINUE
10351 CONTINUE
      LOG = IABS(LOG)
      J=LOG/10
      ICOL=ICOL+1
      TEXT(ICOL)=DIGIT(J+1)
      J=MOD(LOG,10)
      ICOL=ICOL+1
      TEXT(ICOL)=DIGIT(J+1)
      ILAST=ICOL
10341 CONTINUE
      LEFT=IABSW-LEAD-IWHOLE-IABSD
      IF(LEFT .LE. 0)GOTO 10391
      DO 10401 I=1,LEFT
      ICOL=ICOL+1
      TEXT(ICOL)=BLANK
10401 CONTINUE
10402 CONTINUE
10391 CONTINUE
      RETURN
10230 CONTINUE
      DO 10231 I=1,IABSW
      ICOL=ICOL+1
      TEXT(ICOL)=STAR
10231 CONTINUE
10232 CONTINUE
      IFIRST=1
      ILAST=W
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2ERR(LENGTH,INFOIN,CARDIN,FILE,IFLAG)
      INTEGER LENGTH,INFOIN(10),FILE,IFLAG
cmt4       BYTE CARDIN(LENGTH)
      character*1 CARDIN(LENGTH)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
cmt4       BYTE POINTR,BLANK,FIRSTU(12),MATCHS(9)
      character*1 STRING
      character*1 POINTR,BLANK,FIRSTU(12),MATCHS(9)
      DATA FIRSTU/1H ,1HF,1HI,1HR,1HS,1HT,1H ,1HU,1HS,1HE,1HD,1H./
      DATA MATCHS/1H ,1HM,1HA,1HT,1HC,1HH,1HE,1HS,1H /
      DATA POINTR/    1H!/,BLANK/    1H /
      IF(LENGTH .LE. 0)GOTO 10421
      GO TO (10430,10440,10450,10450,10450,10460),IFLAG
10470 FORMAT(1X,I12,2X,90A1/30X,75A1)
10460 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10491
      WRITE (DBGFIL,10470) INFOIN(7),CARDIN
      IF(FILE .NE. DBGFIL)GOTO 10511
      INFOIN(10)=1
10511 CONTINUE
10491 CONTINUE
      IF (INFOIN(10).NE.0) GO TO 10430
      IF (FLAGS(57)) GO TO 10430
10440 CONTINUE
      INFOIN(10)=1
      IF((FILE .NE. OUTFIL) .AND. ((FLAGS(57))))GOTO 10531
10540 CONTINUE
10541 CONTINUE
      IF(INFOIN(2) .GE. INFOIN(1))GOTO 10561
      L1 = INFOIN(2)+1
      DO 10571 L=L1,LENGTH
      IF(CARDIN(L) .EQ. BLANK)GOTO 10591
      L2=INFOIN(2)
      WRITE (OUTFIL,10470) INFOIN(7),  (CARDIN(LL),LL=1,L2),POINTR,  (CA
     *RDIN(LL),LL=L1,LENGTH)
      GOTO 10542
10591 CONTINUE
10571 CONTINUE
10572 CONTINUE
10561 CONTINUE
      WRITE (OUTFIL,10470) INFOIN(7),CARDIN
10542 CONTINUE
10531 CONTINUE
      IF (FILE.NE.OUTFIL) WRITE (FILE,10470) INFOIN(7),CARDIN
      GO TO 10600
10450 CONTINUE
      WRITE (FILE,10610) CARDIN
      IF((FLAGS(57)))GOTO 10631
      IF (FILE.NE.OUTFIL .AND. .NOT.FLAGS(23)) WRITE (OUTFIL,10610) CARD
     *IN
10631 CONTINUE
10610 FORMAT(   15H *** ERROR *** ,90A1/30X,75A1)
10600 CONTINUE
      IF (FILE.EQ.ERRFIL) FLAGS(23)=.TRUE.
10430 CONTINUE
10421 CONTINUE
      GO TO (10640,10640,10640,10650,10640,10640),IFLAG
10650 CONTINUE
      IF(INFOIN(5) .LE. 0 .OR. INFOIN(5) .GT. LENGTH)GOTO 10671
      I=INFOIN(5)
      WRITE(FILE,10680)(BLANK,J=1,I),POINTR
10680 FORMAT(14X,91A1,30X,75A1)
10671 CONTINUE
10640 CONTINUE
      GO TO (10690,10690,10690, 10700,10710,10690),IFLAG
10710 CONTINUE
      GO TO (10720,10730,10740,10740,10750,10760, 10770,10780,10770),INT
     *ERP
10780 CONTINUE
      MIN=INFOIN(3)-1
      MAX=INFOIN(4)+1
      WRITE(FILE,10790)(CARDIN(I),I=MIN,MAX),MATCHS, (STRING(I),I=1,NSTR
     *NG),FIRSTU
10790 FORMAT (   17H *** WARNING *** ,115A1)
      GO TO 10690
10720 CONTINUE
      WRITE(FILE,10800)
10800 FORMAT(   40H MORE INFORMATION EXPECTED ON THIS CARD.)
      GO TO 10690
10760 CONTINUE
10750 CONTINUE
10740 CONTINUE
10770 CONTINUE
      WRITE(FILE,10810)
10810 FORMAT (   33H *** ERROR *** UNRECOGNIZED INPUT)
10730 CONTINUE
10700 CONTINUE
      IF(NSTRNG .LE. 0)GOTO 10831
      WRITE(FILE,10840)(STRING(J),J=1,NSTRNG)
10840 FORMAT(   12H   FIELD IS ,90A1)
10831 CONTINUE
10690 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2RDNM(INFOIN,CARDIN,NMIN,NMAX,ARRAY,NOUT)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      INTEGER NMIN,NMAX,NOUT,INFOIN(9)
cmt4       BYTE CARDIN(160)
      character*1 CARDIN(160)
      REAL ARRAY(NMAX)
      DO 10851 I=1,NMAX
      GO TO 10861
10870 CONTINUE
      ARRAY(I)=FLOTNG
      GO TO 10862
10865 CONTINUE
      INFOIN(3)=INFOIN(4)
      IF(.TRUE.)GOTO 10862
      GO TO 10862
10861 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(10865,10865,10870,10870,10865,10865,10865,10865,10865),INTERP
10862 CONTINUE
10851 CONTINUE
      I=NMAX+1
10852 CONTINUE
      IF(I .GT. NMIN)GOTO 10891
      WRITE(ERRFIL,10900)
10900 FORMAT(   39H *** ERROR *** EXPECTING ANOTHER NUMBER)
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10891 CONTINUE
      NOUT=I-1
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2REST
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
cccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      INTEGER IFLDDF(15)
      DATA IFLDDF/5,1,3,2,4,0,0,0,0,0,0,0,0,0,0/
      DO 10911 I=2,13
      FLAGS(I)=.FALSE.
10911 CONTINUE
10912 CONTINUE
      DO 10921 I=19,22
      FLAGS(I)=.TRUE.
10921 CONTINUE
10922 CONTINUE
      FLAGS(27)=.FALSE.
      DO 10931 I=32,52
      FLAGS(I)=.TRUE.
10931 CONTINUE
10932 CONTINUE
      FLAGS(38)=.FALSE.
      FLAGS(39)=.FALSE.
      DO 10941 I=73,78
      FLAGS(I)=.FALSE.
10941 CONTINUE
10942 CONTINUE
      FLAGS(82)=.FALSE.
      DO 10951 I=84,90
      FLAGS(I)=.FALSE.
10951 CONTINUE
10952 CONTINUE
      DO 10961 I=1,3
      NDIMNS(I)=0
      TIKSIZ(I)=0.1
      BARSIZ(I)=0.1
      XYZBAS(I)=10.
      SCLPRM(1,I)=10.
      LBLCHR(I)=0
      NONLIN(I)=-1
      NXYZ1(I)=0
      NXYZ2(I)=0
10961 CONTINUE
10962 CONTINUE
      NFIELD=5
      SYMBOL=NOSYMB
      DO 10971 I=1,15
      IFIELD(I)=IFLDDF(I)
      FACTXY(I)=1.
10971 CONTINUE
10972 CONTINUE
      GRDTYP=1
      TITX = SCREEN(1)*0.1
      TITY = SCREEN(2) * 0.9
      DO 10981 I=1,4
cccccccccccccccccc 26-dec-1995 Modified by Y.Teramoto
	if( I .le. 2 ) then
	  XYPART(I,1)=1.
	  XYPART(I,2)=1.
	endif
cccc      XYPART(I,1)=1.
cccccccccccccccccc end of 26-dec-1995 modification
      FRELBL(I)=1.
10981 CONTINUE
10982 CONTINUE
      TSIZDF=-2.
      LBLSIZ=-2.
      LINWID = LINWDF
      DOTWID=MAX0(LINWID,DOTWDF)
      LINTEX = 4
      NPATRN=1
      PATRN(1)=1.
cccccccccccccccccccc 26-dec-1995 Modified by Y.Teramoto
      DO 10991 I=1,3
        CIRSIZ(1,I)=1.
        CIRSIZ(2,I)=1.
ccc      DO 10991 I=1,6
ccc      CIRSIZ(I,1)=1.
cccccccccccccccccccc end of 26-dec-1995 modification
10991 CONTINUE
10992 CONTINUE
      EYESEP=0.
      EYEDIS=0.
      SCRD=0.
      DO 11001 I=1,3
      VERTCL(I)=0.
      EYEDIR(I)=0.
      VUECEN(I)=-77777.
      ORAXES(I)=0.
11001 CONTINUE
11002 CONTINUE
      FLAGS(55)=.FALSE.
      TTHETA=60.
      TPHI=60.
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2SETS
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1  STRING
      COMMON /T2NPMX/ NPTMAX
      COMMON /T2XPNT/ DATBUF(5125)
      INTEGER NPTMAX
      NPOINT=0
      IF (IBLKTP.EQ.2) NINCR=5
      NHEADR=9+NNAME+NINCR
      DATBUF(LISTPT+2)=NHEADR
      DATBUF(LISTPT+3)=DATBUF(LISTPT+2)
      DATBUF(LISTPT+4)=IDIMNS
      DATBUF(LISTPT+5)=IBLKTP
      DATBUF(LISTPT+6)=NPOINT
      DATBUF(LISTPT+7)=NINCR
      DATBUF(LISTPT+8)=NNAME
      I=LISTPT+NHEADR-NINCR
      DO 11011 J=1,15
      IVRPTR(J)=1
11011 CONTINUE
11012 CONTINUE
      IF(IBLKTP .NE. 1)GOTO 11031
      DO 11041 J=1,NINCR
      K=IVARBL(J)
      DATBUF(I)=K
      IVRPTR(K)=I+NINCR
      I=I+1
11041 CONTINUE
11042 CONTINUE
      GOTO11021
11031 IF(IBLKTP .NE. 2)GOTO 11051
      DATBUF(I+1)=NMESH1
      DATBUF(I+2)=NMESH2
      DATBUF(I+3)=MESH1
      DATBUF(I+4)=MESH2
      DATBUF(I+5)=MESH3
      IVRPTR(8) = LISTPT+NHEADR
      GOTO 11061
11051 CONTINUE
      STOP 653
11061 CONTINUE
11021 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11081
      MIN=LISTPT
      MAX=LISTPT+NNAME+8
      WRITE(DBGFIL,11090)(DATBUF(I),I=MIN,MAX)
11090 FORMAT (60X,   51HT2SETS NEXT LAST NBLK NHED DIMN TYPE NPNT INCR N
     *NAM/  65X,9F5.0,2X,20A1)
      MIN=MAX+1
      MAX=MAX+NINCR
      WRITE(DBGFIL,11100)(DATBUF(I),I=MIN,MAX)
11100 FORMAT (63X,    4HVARS,15F5.0)
11081 CONTINUE
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
      SUBROUTINE T2SET1(INFOIN,CARDIN)
      INTEGER INFOIN(10)
cmt4       BYTE CARDIN(160)
      character*1 CARDIN(160)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL TXSYM
      EXTERNAL TXSYM
cmt4       BYTE GRIDDF(3)
      character*1 GRIDDF(3)
      INTEGER ICNVRT(6)
      LOGICAL LLLL
      DATA GRIDDF/1H0,1HP,1H /
      DATA ICNVRT/1,3,5,2,4,6/
      GO TO 10011
10040 CONTINUE
      SIZ=0.
      GO TO 10051
10080 CONTINUE
      IXY=INTEG
      IF (IXY.EQ.1) SIZ=-2.
      CALL TOKEN(INFOIN,CARDIN,    5HFROM;)
      IF(INTERP .EQ. 2)GOTO 10101
      INFOIN(3)=INFOIN(4)
10101 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10121
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10131
10121 CONTINUE
      SAVE1=FLOTNG
10131 CONTINUE
10111 CONTINUE
      GO TO 10141
10170 CONTINUE
      I2OF=0
      GO TO 10142
10180 CONTINUE
      I2OF = 1
      GO TO 10142
10160 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10150 GO TO ( 10170,10180,10145),KEYORD
10145 CONTINUE
      I2OF=0
      INFOIN(3)=INFOIN(4)
      GO TO 10142
10141 CALL TOKEN(INFOIN,CARDIN,    7HTO,OF,;)
      GOTO(10145,10150,10145,10145,10145,10145,10145,10160,10145),INTERP
10142 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10201
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10211
10201 CONTINUE
      SAVE2=FLOTNG
10211 CONTINUE
10191 CONTINUE
      IF(I2OF .EQ. 0)GOTO 10231
      IF(SAVE2 .LE. 0.)GOTO 10251
      XYPART(IXY+1,1)=SAVE1
      XYPART(IXY+1,2)=SAVE2
10251 CONTINUE
      GOTO 10261
10231 CONTINUE
      IF(SAVE1 .LE. SAVE2)GOTO 10281
      T=SAVE1
      SAVE1=SAVE2
      SAVE2=T
10281 CONTINUE
      IF(SAVE1 .LT. 0.0 .OR. SAVE2 .GT. SCREEN(IXY+1) .OR. SAVE1 .EQ. SA
     *VE2)GOTO 10301
      WINDOW(IXY+1)=SAVE1
      WINDOW(IXY+3)=SAVE2
      XYPART(IXY+1,2)=1.
      FLAGS(IXY+11)=.TRUE.
      FLAGS(IXY+71)=.TRUE.
      GOTO 10311
10301 CONTINUE
      SAVE2=-1.
10311 CONTINUE
10291 CONTINUE
10261 CONTINUE
10221 CONTINUE
      IF(SAVE2 .GT. 0.)GOTO 10331
      WRITE(ERRFIL,10340)SCREEN
10340 FORMAT (   38H *** ERROR *** WINDOW MUST LIE WITHIN ,     30HLIMIT
     *S OF PAPER/SCREEN (X 0 TO,F6.2,      8H, Y 0 TO,F6.2,    1H))
      FLAGS(23)=.TRUE.
      GOTO 10351
10331 CONTINUE
      FLAGS(2*IXY+5)=.FALSE.
      FLAGS(2*IXY+6)=.FALSE.
10351 CONTINUE
10321 CONTINUE
      GO TO 10051
10051 CALL TOKEN(INFOIN,CARDIN,    7HX,Y:1,;)
      GO TO (10052,10057,10055,10055,10055,10055,10055,10056, 10055),INT
     *ERP
10056 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10057 GO TO ( 10080,10080,10055),KEYORD
10055 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10051
10052 CONTINUE
      FLAGS(2)=.FALSE.
      FLAGS(3)=.FALSE.
      FLAGS(74)=.FALSE.
      FLAGS(75)=.FALSE.
      LBLCHR(1)=0
      LBLCHR(2)=0
      LBLCHR(3)=0
      IF(.NOT.(FLAGS(29)))GOTO 10371
      WRITE(DBGFIL,10380)WINDOW
10380 FORMAT (60X,    6HWINDOW,4G10.3)
10371 CONTINUE
      GO TO 10012
10390 CONTINUE
      GO TO 10401
10430 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10451
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10461
10451 CONTINUE
      ASIZE = FLOTNG
10461 CONTINUE
10441 CONTINUE
      GO TO 10401
10470 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10491
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10501
10491 CONTINUE
      AFLARE = FLOTNG
10501 CONTINUE
10481 CONTINUE
      GO TO 10401
10401 CALL TOKEN(INFOIN,CARDIN,   18HSIZE,FLARE,FLAIR,;)
      GO TO (10402,10407,10405,10405,10405,10405,10405,10406, 10405),INT
     *ERP
10406 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10407 GO TO ( 10430,10470,10470,10405),KEYORD
10405 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10401
10402 CONTINUE
      GO TO 10012
10510 CONTINUE
      IFLAG=1
      FLAGS(2) = .FALSE.
      FLAGS(3) = .FALSE.
      GO TO 10521
10550 CONTINUE
      LLLL=INTEG.EQ.0
      IXXXXX=IFLAG
10560 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 8))GOTO 10661
      WRITE(6,10670)IXXXXX
10670 FORMAT(    8H IFLAG (,I5,   31H) OUT OF RANGE FOR CASE (1 TO8))
      CALL ERRTRA
      STOP 7777
10661 CONTINUE
      GO TO (10570,10580,10590,10600,10610,10620,10630,10640),IXXXXX
10570 CONTINUE
      DO 10681 I=19,22
      FLAGS(I)=LLLL
10681 CONTINUE
10682 CONTINUE
      DO 10691 I=50,52
      FLAGS(I)=LLLL
10691 CONTINUE
10692 CONTINUE
      GOTO 10562
10610 CONTINUE
      FLAGS(19)=LLLL
      FLAGS(51)=.TRUE.
      GOTO 10562
10620 CONTINUE
      FLAGS(20)=LLLL
      FLAGS(50)=.TRUE.
      GOTO 10562
10630 CONTINUE
      FLAGS(21)=LLLL
      FLAGS(51)=.TRUE.
      GOTO 10562
10640 CONTINUE
      FLAGS(22)=LLLL
      FLAGS(50)=.TRUE.
      GOTO 10562
10580 CONTINUE
      FLAGS(20)=LLLL
      FLAGS(22)=LLLL
      FLAGS(50)=LLLL
      GOTO 10562
10590 CONTINUE
      FLAGS(19)=LLLL
      FLAGS(21)=LLLL
      FLAGS(51)=LLLL
      GOTO 10562
10600 CONTINUE
      FLAGS(52)=LLLL
      GOTO 10562
10562 CONTINUE
      GO TO 10521
10700 CONTINUE
      IFLAG=INTEG
      GO TO 10521
10521 CALL TOKEN(INFOIN,CARDIN,   65HOFF:1,ON,ALL:1,X:2,Y:3,Z:4,LEFT:5,B
     *OTTOM:6,RIGHT:7,RITE:7,TOP:8,;)
      GO TO (10522,10527,10525,10525,10525,10525,10525,10526, 10525),INT
     *ERP
10526 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10527 GO TO ( 10550,10550,10700,10700,10700,10700,10700,10700,10700,1070
     *0,10700,10525),KEYORD
10525 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10521
10522 CONTINUE
      GO TO 10012
10710 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    5HSIZE;)
      IF(INTERP .EQ. 2)GOTO 10731
      INFOIN(3)=INFOIN(4)
10731 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10751
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10761
10751 CONTINUE
      DO 10771 I=1,3
      BARSIZ(I)=FLOTNG
10771 CONTINUE
10772 CONTINUE
10761 CONTINUE
10741 CONTINUE
      GO TO 10012
10780 CONTINUE
      I = INTEG
      CALL TOKEN(INFOIN,CARDIN,    5HSIZE;)
      IF(INTERP .EQ. 2)GOTO 10801
      INFOIN(3)=INFOIN(4)
10801 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10821
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10831
10821 CONTINUE
      CIRSIZ(1,I) = FLOTNG
10831 CONTINUE
10811 CONTINUE
      GO TO 10841
10850 CONTINUE
      CIRSIZ(2,I) = FLOTNG
      GO TO 10842
10845 CONTINUE
      CIRSIZ(2,I) = CIRSIZ(1,I)
      INFOIN(3)=INFOIN(4)
      GO TO 10842
10841 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(10845,10845,10850,10850,10845,10845,10845,10845,10845),INTERP
10842 CONTINUE
      GO TO 10012
10860 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    7HLENGTH;)
      IF(INTERP .EQ. 2)GOTO 10881
      INFOIN(3)=INFOIN(4)
10881 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10901
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10911
10901 CONTINUE
      INFOIN(2)=INTEG
      LENCRD = INTEG
      IF (LENCRD.LT.80) LENCRD=80
      IF (LENCRD.GT.160) LENCRD=160
10911 CONTINUE
10891 CONTINUE
      GO TO 10012
10920 CONTINUE
      GO TO 10931
10960 CONTINUE
      LINCOL=INTEG
      GO TO 10932
ccccccccccccccccc 3-Apr-96 modified by Y.Teramoto
10931 CALL TOKEN(INFOIN,CARDIN,
	1 56HBLACK:1,RED:2,BLUE:3,GREEN:4,MAGENTA:5,YELLOW:6,CYAN:7,;)
ccc10931 CALL TOKEN(INFOIN,CARDIN,   22HBLACK:1,RED:2,BLUE:3,;)
ccccccccccccccccccccccccccccccccc
      GO TO (10935,10937,10935,10935,10935,10935,10935,10936, 10935),INT
     *ERP
10936 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
cccccccccccccccccccccc 3-Apr-96 modified by Y.Teramoto
10937 GO TO ( 10960,10960,10960,10960,10960,10960,10960,10935),KEYORD
cccc10937 GO TO ( 10960,10960,10960,10935),KEYORD
cccccccccccccccccccccccccc
10935 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10932 CONTINUE
      GO TO 10012
10970 CONTINUE
      IF((FLAGS(57)))GOTO 10991
      CALL TXDEVC(INFOIN,CARDIN)
      CALL T2REST
10991 CONTINUE
      GO TO 10012
11000 CONTINUE
      GO TO 11011
11040 CONTINUE
      IFILE=INTEG
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11061
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11071
11061 CONTINUE
      IXXXXX=IFILE
11080 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 4))GOTO 11141
      WRITE(6,11150)IXXXXX
11150 FORMAT(    8H IFILE (,I5,   31H) OUT OF RANGE FOR CASE (1 TO4))
      CALL ERRTRA
      STOP 7777
11141 CONTINUE
      GO TO (11090,11100,11110,11120),IXXXXX
11090 CONTINUE
      DBGFIL=INTEG
      GOTO 11082
11100 CONTINUE
      ERRFIL=INTEG
      GOTO 11082
11110 CONTINUE
      OUTFIL=INTEG
      ERRFIL=INTEG
      DBGFIL=INTEG
      GOTO 11082
11120 CONTINUE
      INPFIL=INTEG
      GOTO 11082
11082 CONTINUE
11071 CONTINUE
11051 CONTINUE
      GO TO 11011
11011 CALL TOKEN(INFOIN,CARDIN,   34HDEBUG:1,ERROR:2,OUTPUT:3,INPUT:4,;)
      GO TO (11012,11017,11015,11015,11015,11015,11015,11016, 11015),INT
     *ERP
11016 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11017 GO TO ( 11040,11040,11040,11040,11015),KEYORD
11015 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11011
11012 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11171
      WRITE(DBGFIL,11180)OUTFIL,DBGFIL,ERRFIL,INPFIL
11180 FORMAT (60X,    4HOUT=,I3,    5H DBG=,I3,    5H ERR=,I3,    5H INP
     *=,I3)
11171 CONTINUE
      GO TO 10012
11190 CONTINUE
      GO TO 11201
11230 CONTINUE
      LETSET=INTEG
      CALL TXCSET
      GO TO 11202
cccccccccccccccccccc 14-Jun-96 Modified by Y.Teramoto
11201 CALL TOKEN(INFOIN,CARDIN,   
	1	38HBASIC:1,EXTENDED:2,DUPLEX:3,DEVICE:4,;)
cccc11201 CALL TOKEN(INFOIN,CARDIN,   29HBASIC:1,EXTENDED:2,DUPLEX:3,;)
cccccccccccccccccccccccccccccccccccc
      GO TO (11205,11207,11205,11205,11205,11205,11205,11206, 11205),INT
     *ERP
11206 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
ccccccccccccccccccc 14-Jun-96 Modified by Y.Teramoto
11207 GO TO ( 11230,11230,11230,11230,11205),KEYORD
ccccc11207 GO TO ( 11230,11230,11230,11205),KEYORD
cccccccccccccccccccccccccccccccccccc
11205 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11202 CONTINUE
      GO TO 10012
11240 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 5)GOTO 11261
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11271
11261 CONTINUE
      IF(NSTRNG .GT. 16)GOTO 11291
      DO 11301 I=1,NSTRNG
      INPFMT(I)=STRING(I)
11301 CONTINUE
11302 CONTINUE
      GOTO 11311
11291 CONTINUE
      WRITE(ERRFIL,11320)
11320 FORMAT (   23H FORMAT STRING TOO LONG)
      CALL T2ERR(INFO,INFO,CARD,ERRFIL,4)
11311 CONTINUE
11281 CONTINUE
11271 CONTINUE
11251 CONTINUE
      GO TO 10012
11330 CONTINUE
      GRDTYP=4
      GRDSIZ=2.
      GRDSYM=TXSYM(GRIDDF)
      GO TO 11341
11370 CONTINUE
      IF(GRDTYP .NE. 3)GOTO 11391
      GRDTYP=4
      GOTO 11401
11391 CONTINUE
      GRDTYP=2
11401 CONTINUE
11381 CONTINUE
      GO TO 11341
11410 CONTINUE
      IF(GRDTYP .NE. 2)GOTO 11431
      GRDTYP=4
      GOTO 11441
11431 CONTINUE
      GRDTYP=3
11441 CONTINUE
11421 CONTINUE
      GO TO 11341
11450 CONTINUE
      GRDTYP=INTEG
      GO TO 11341
11460 CONTINUE
      GRDTYP=5
      GO TO 11471
11480 CONTINUE
      GO TO 11472
11475 CONTINUE
      GRDSYM=TXSYM(STRING)
      GO TO 11472
11471 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(11480,11475,11475,11475,11475,11475,11475,11475,11475),INTERP
11472 CONTINUE
      GO TO 11341
11490 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11511
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11521
11511 CONTINUE
      GRDSIZ=FLOTNG
11521 CONTINUE
11501 CONTINUE
      GO TO 11341
11341 CALL TOKEN(INFOIN,CARDIN,   48HH,HORIZONTAL,V,VERTICAL,OFF:1,ON:4,
     *SYMBOL,SIZE,;)
      GO TO (11342,11347,11345,11345,11345,11345,11345,11346, 11345),INT
     *ERP
11346 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11347 GO TO ( 11370,11370,11410,11410,11450,11450,11460,11490,11345),KEY
     *ORD
11345 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11341
11342 CONTINUE
      GO TO 10012
11530 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11551
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11561
11551 CONTINUE
      LINWID = INTEG
11561 CONTINUE
11541 CONTINUE
      GO TO 10012
11570 CONTINUE
      IFLAG=1
      FLAGS(3) = .FALSE.
      GO TO 11581
11610 CONTINUE
      I=INTEG
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11631
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11641
11631 CONTINUE
      FRELBL(I)=FLOTNG
11641 CONTINUE
11621 CONTINUE
      FRELBL(3)=FRELBL(1)
      GO TO 11581
11650 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11671
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11681
11671 CONTINUE
      LBLSIZ=FLOTNG
      FLAGS(77)=.TRUE.
11681 CONTINUE
11661 CONTINUE
      GO TO 11581
11690 CONTINUE
      LLLL=INTEG.EQ.0
      IXXXXX=IFLAG
11700 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 8))GOTO 11801
      WRITE(6,11810)IXXXXX
11810 FORMAT(    8H IFLAG (,I5,   31H) OUT OF RANGE FOR CASE (1 TO8))
      CALL ERRTRA
      STOP 7777
11801 CONTINUE
      GO TO (11710,11720,11730,11740,11750,11760,11770,11780),IXXXXX
11710 CONTINUE
      DO 11821 I=36,39
      FLAGS(I)=LLLL
11821 CONTINUE
11822 CONTINUE
      DO 11831 I=47,49
      FLAGS(I)=LLLL
11831 CONTINUE
11832 CONTINUE
      GOTO 11702
11750 CONTINUE
      FLAGS(36)=LLLL
      FLAGS(48)=.TRUE.
      GOTO 11702
11760 CONTINUE
      FLAGS(37)=LLLL
      FLAGS(47)=.TRUE.
      GOTO 11702
11770 CONTINUE
      FLAGS(38)=LLLL
      FLAGS(48)=.TRUE.
      GOTO 11702
11780 CONTINUE
      FLAGS(39)=LLLL
      FLAGS(47)=.TRUE.
      GOTO 11702
11720 CONTINUE
      FLAGS(37)=LLLL
      FLAGS(39)=LLLL
      FLAGS(47)=LLLL
      GOTO 11702
11730 CONTINUE
      FLAGS(36)=LLLL
      FLAGS(38)=LLLL
      FLAGS(48)=LLLL
      GOTO 11702
11740 CONTINUE
      FLAGS(49)=LLLL
      GOTO 11702
11702 CONTINUE
      GO TO 11581
11840 CONTINUE
      IFLAG=INTEG
      GO TO 11581
11581 CALL TOKEN(INFOIN,CARDIN,   93HDOWN:2,UP:4,SIDEWAYS:1,SIZE,OFF:1,O
     *N,ALL:1,X:2,Y:3,Z:4,LEFT:5,BOTTOM:6,RIGHT:7,RITE:7,TOP:8,;)
      GO TO (11582,11587,11585,11585,11585,11585,11585,11586, 11585),INT
     *ERP
11586 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11587 GO TO ( 11610,11610,11610,11650,11690,11690,11840,11840,11840,1184
     *0,11840,11840,11840,11840,11840,11585),KEYORD
11585 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11581
11582 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11861
      WRITE(DBGFIL,11870)(FLAGS(I),I=36,39),LBLSIZ
11870 FORMAT(60X,    7HLABELS ,4L1,G10.3)
11861 CONTINUE
      GO TO 10012
11880 CONTINUE
      ICH=0
      GO TO 11891
11920 CONTINUE
      ICH = INTEG
      CALL TOKEN(INFOIN,CARDIN,    5HFROM;)
      IF(INTERP .EQ. 2)GOTO 11941
      INFOIN(3)=INFOIN(4)
11941 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11961
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11971
11961 CONTINUE
      VALSAV=FLOTNG
11971 CONTINUE
11951 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    3HTO;)
      IF(INTERP .EQ. 2)GOTO 11991
      INFOIN(3)=INFOIN(4)
11991 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12011
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12021
12011 CONTINUE
      XYZLIM(ICH,1)=VALSAV
      XYZLIM(ICH,2)=FLOTNG
      XYZLIM(ICH,1)=VALSAV
      XYZLIM(ICH,2)=FLOTNG
      I=2*ICH+3
      FLAGS(I)=.TRUE.
      FLAGS(I+1)=.TRUE.
      FLAGS(I+60)=.TRUE.
      FLAGS(I+61)=.TRUE.
12021 CONTINUE
12001 CONTINUE
      GO TO 11891
12030 CONTINUE
      ICH=INTEG
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12051
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12061
12051 CONTINUE
      XYZLIM(ICH,1)=FLOTNG
      ICH=ICNVRT(ICH)
      FLAGS(ICH+4)=.TRUE.
      FLAGS(ICH+64)=.TRUE.
12061 CONTINUE
12041 CONTINUE
      GO TO 11891
12070 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12091
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12101
12091 CONTINUE
      RADMAX = FLOTNG
      FLAGS(85)=.TRUE.
      FLAGS(86)=.TRUE.
12101 CONTINUE
12081 CONTINUE
      GO TO 11891
11891 CALL TOKEN(INFOIN,CARDIN,   62HX:1,Y:2,Z:3,XMIN:1,XMAX:4,YMIN:2,YM
     *AX:5,ZMIN:3,ZMAX:6,RADIUS,;)
      GO TO (11892,11897,11895,11895,11895,11895,11895,11896, 11895),INT
     *ERP
11896 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11897 GO TO ( 11920,11920,11920,12030,12030,12030,12030,12030,12030,1207
     *0,11895),KEYORD
11895 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11891
11892 CONTINUE
      IF(ICH .NE. 0)GOTO 12121
      DO 12131 I=1,6
      FLAGS(I+4)=.FALSE.
      FLAGS(I+64)=.FALSE.
12131 CONTINUE
12132 CONTINUE
12121 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12151
      WRITE(DBGFIL,12160)(FLAGS(I),I=5,10),XYZLIM
12160 FORMAT (60X,6L1,    7HXYZLIM ,1P3E13.6/73X,1P4E13.6)
12151 CONTINUE
      FLAGS(75) = .FALSE.
      GO TO 10012
10030 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10020 GO TO ( 10040,10040,10390,10510,10510,10510,10710,10780,10780,1078
     *0,10780,10860,10920,10970,11000,11190,11240,11330,11530,11530,1157
     *0,11880,10015),KEYORD
10015 CONTINUE
      INFOIN(3)=INFOIN(4)
      GO TO 10012
10011 CALL TOKEN(INFOIN,CARDIN,  141HAREA,WINDOW,ARROW,AX,AXIS,AXES,BAR,
     *BOX:1,DIAMOND:2,CIRCLE:3,ELLIPSE:3,CARD,COLOR,DEVICE,FILE,FONT,FOR
     *MAT,GRID,INTENSITY,WIDTH,LABELS,LIMITS,;)
      GOTO(10015,10020,10015,10015,10015,10015,10015,10030,10015),INTERP
10012 CONTINUE
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
      SUBROUTINE T2SET2(INFOIN,CARDIN)
      INTEGER INFOIN(10)
cmt4       BYTE CARDIN(160)
      character*1 CARDIN(160)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL TXSYM
      EXTERNAL TXSYM
      DOUBLE PRECISION TWO PI,ANGFAC(2)
      LOGICAL LLLL
      DATA TWO PI /6.2831 85307 18 D0/
      DATA ANGFAC/1.0D0  ,0.0174532925 D0  /
      GO TO 10011
10040 CONTINUE
      GO TO 10051
10080 CONTINUE
      FLAGS(83)=INTEG.NE.0
      CALL TXQUIK
      GO TO 10051
10090 CONTINUE
      FLAGS(25)=.TRUE.
      GO TO 10051
10100 CONTINUE
      FLAGS(25)=.FALSE.
      GO TO 10051
10110 CONTINUE
      FLAGS(29)=.TRUE.
      GO TO 10051
10120 CONTINUE
      FLAGS(29)=.FALSE.
      GO TO 10051
10130 CONTINUE
      FLAGS(18)=.TRUE.
      GO TO 10051
10140 CONTINUE
      FLAGS(18)=.FALSE.
      GO TO 10051
10150 CONTINUE
      GO TO 10161
10170 CONTINUE
      MXECHO=INTEG
      GO TO 10162
10165 CONTINUE
      MXECHO=10000
      INFOIN(3)=INFOIN(4)
      GO TO 10162
10161 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(10165,10165,10170,10170,10165,10165,10165,10165,10165),INTERP
10162 CONTINUE
      GO TO 10051
10180 CONTINUE
      MXECHO=0
      GO TO 10051
10051 CALL TOKEN(INFOIN,CARDIN,   70HQUICK:1,SLOW,VECTOR,NOVECTOR,DEBUG,
     *NODEBUG,TRACE,NOTRACE,ECHO,NOECHO,;)
      GO TO (10052,10057,10055,10055,10055,10055,10055,10056, 10055),INT
     *ERP
10056 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10057 GO TO ( 10080,10080,10090,10100,10110,10120,10130,10140,10150,1018
     *0,10055),KEYORD
10055 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10051
10052 CONTINUE
      GO TO 10012
10190 CONTINUE
      DO 10201 I=2,15
      IFIELD(I)=0
10201 CONTINUE
10202 CONTINUE
      IFIELD(1)=-1
      NFIELD=0
      NDIMNS(3)=2
      DO 10211 I=79,81
      FLAGS(I)=.FALSE.
10211 CONTINUE
10212 CONTINUE
      IF (IBLKTP.NE.1) CALL T2PNTR
10220 CONTINUE
10221 CONTINUE
      NFIELD=NFIELD+1
      GO TO 10231
10240 CONTINUE
      NFIELD=NFIELD-1
      IF(.TRUE.)GOTO 10222
      GO TO 10232
10270 CONTINUE
      FLAGS(INTEG+79)=.TRUE.
      INTEG=2*INTEG+3
      GO TO 10232
10280 CONTINUE
      IF(NFIELD .NE. 1)GOTO 10301
      WRITE(ERRFIL,10310)
10310 FORMAT (   40H *** ERROR ***  SYMBOL MUST NOT BE FIRST)
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,3)
10301 CONTINUE
      GO TO 10232
10320 CONTINUE
      NDIMNS(3)=3
      GO TO 10232
10330 CONTINUE
      GO TO 10232
10260 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10250 GO TO ( 10270,10270,10270,10280,10320,10330,10330,10330,10330,1033
     *0,10330,10235),KEYORD
10231 CALL TOKEN(INFOIN,CARDIN,   56HRX,RY:1,RZ:2,SYMBOL:1,Z:6,X:2,Y:4,D
     *X:3,DY:5,DZ:7,DUMMY,;)
      GOTO(10240,10250,10235,10235,10235,10235,10235,10260,10235),INTERP
10235 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10232
10232 CONTINUE
      IF(INTEG .LE. 0)GOTO 10351
      I=INTEG
      IFIELD(I)=NFIELD
      GO TO 10361
10370 CONTINUE
      FACTXY(I)=FLOTNG
      GO TO 10362
10365 CONTINUE
      FACTXY(I)=1.
      INFOIN(3)=INFOIN(4)
      GO TO 10362
10361 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(10365,10365,10370,10370,10365,10365,10365,10365,10365),INTERP
10362 CONTINUE
      DO 10381 J=1,15
      IF(IVARBL(J).EQ.I)GOTO 10382
10381 CONTINUE
      IF (NDIMNS(3).EQ.3) IDIMNS=3
      NINCR=NINCR+1
      IVARBL(NINCR)=I
      CALL T2SETS
10382 CONTINUE
10351 CONTINUE
      GOTO 10221
10222 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10401
      WRITE(DBGFIL,10410)IFIELD,IVARBL,FACTXY
10410 FORMAT(60X,    6HIFIELD,15I3  /60X,    6HIVARBL,15I3  /60X,    6HF
     *ACTXY,5G10.3/60X,10F3.1)
10401 CONTINUE
      GO TO 10012
10420 CONTINUE
      IPTR=0
      FLAGS(2)=.FALSE.
      GO TO 10431
10460 CONTINUE
      IF(IPTR .NE. 0)GOTO 10481
      DO 10491 I=40,43
      FLAGS(I)=INTEG.EQ.0
10491 CONTINUE
10492 CONTINUE
      GOTO 10501
10481 CONTINUE
      FLAGS(IPTR)=INTEG.EQ.0
      IPTR=0
10501 CONTINUE
10471 CONTINUE
      GO TO 10431
10510 CONTINUE
      IPTR=INTEG
      GO TO 10431
10431 CALL TOKEN(INFOIN,CARDIN,   56HOFF:1,ON,ALL,LEFT:40,BOTTOM:41,RIGH
     *T:42,RITE:42,TOP:43,;)
      GO TO (10432,10437,10435,10435,10435,10435,10435,10436, 10435),INT
     *ERP
10436 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10437 GO TO ( 10460,10460,10510,10510,10510,10510,10510,10510,10435),KEY
     *ORD
10435 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10431
10432 CONTINUE
      IF (IPTR.NE.0) FLAGS(IPTR)=.TRUE.
      GO TO 10012
10520 CONTINUE
      NPATRN = 0
      FLAGS(98)=.TRUE.
      GO TO 10531
10540 CONTINUE
      IF(NPATRN.EQ.10)GOTO 10532
      NPATRN=NPATRN+1
      PATRN(NPATRN)=FLOTNG
      GO TO 10531
10535 CONTINUE
      IF(.TRUE.)GOTO 10532
      GO TO 10531
10531 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(10532,10535,10540,10540,10535,10535,10535,10535,10535),INTERP
10532 CONTINUE
      GO TO 10012
10550 CONTINUE
      IF(INTEG .NE. 0)GOTO 10571
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10591
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10601
10591 CONTINUE
10601 CONTINUE
10581 CONTINUE
10571 CONTINUE
      IF(INTEG .LE. 0 .OR. INTEG .GT. 3)GOTO 10621
      LINCOL=INTEG
      GOTO 10631
10621 CONTINUE
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10631 CONTINUE
10611 CONTINUE
      GO TO 10012
10640 CONTINUE
      FLAGS(27)=.TRUE.
      GO TO 10651
10680 CONTINUE
      FLAGS(27)=.FALSE.
      GO TO 10651
10690 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10711
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10721
10711 CONTINUE
      RADX0=FLOTNG
      FLAGS(89)=.TRUE.
10721 CONTINUE
10701 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10741
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10751
10741 CONTINUE
      RADY0=FLOTNG
      FLAGS(90)=.TRUE.
10751 CONTINUE
10731 CONTINUE
      GO TO 10651
10760 CONTINUE
      IF(INTEG .EQ. 0)GOTO 10781
      RADANG= SNGL( ANGFAC(INTEG) )
      GOTO 10791
10781 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10811
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10821
10811 CONTINUE
10821 CONTINUE
10801 CONTINUE
      IF (FLOTNG.LE.0.) FLOTNG=360.
      RADANG = SNGL(TWO PI)/FLOTNG
10791 CONTINUE
10771 CONTINUE
      FLAGS(88)=.TRUE.
      GO TO 10651
10830 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10851
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10861
10851 CONTINUE
10861 CONTINUE
10841 CONTINUE
      RADRMX=FLOTNG
      FLAGS(85)=.TRUE.
      GO TO 10651
10870 CONTINUE
      I=INTEG
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 10891
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 10901
10891 CONTINUE
      FLAGS(I)=.TRUE.
      IF(I .NE. 86)GOTO 10921
      RADRMN=FLOTNG
      GOTO 10931
10921 CONTINUE
      RADAMN=FLOTNG
10931 CONTINUE
10911 CONTINUE
10901 CONTINUE
10881 CONTINUE
      GO TO 10651
10651 CALL TOKEN(INFOIN,CARDIN,   90HOFF,CENTER,RADIANS:1,DEGREES:2,FULL
     *CIRCLE,RA:1,RAD:1,RADI:1,RADIUS:1,MINRAD:86,MINANG:87,;)
      GO TO (10652,10657,10655,10655,10655,10655,10655,10656, 10655),INT
     *ERP
10656 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10657 GO TO ( 10680,10690,10760,10760,10760,10830,10830,10830,10830,1087
     *0,10870,10655),KEYORD
10655 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10651
10652 CONTINUE
      GO TO 10012
10940 CONTINUE
      GO TO 10951
10980 CONTINUE
      IXY=INTEG
      BUFFER(1)=10.
      ISCAL=NONLIN(IXY)
      N1=0
      N2=0
      GO TO 10991
11020 CONTINUE
      INFOIN(3)=INFOIN(4)
      IF(.TRUE.)GOTO 10992
      GO TO 10991
11030 CONTINUE
      ISCAL=-INTEG
      IF(INTEG .NE. 6)GOTO 11051
      BUFFER(2)=0.
      BUFFER(3)=1.
      GO TO 11061
11090 CONTINUE
      I=INTEG
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11111
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11121
11111 CONTINUE
      BUFFER(I)=FLOTNG
11121 CONTINUE
11101 CONTINUE
      GO TO 11061
11080 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11070 GO TO ( 11090,11090,11065),KEYORD
11065 CONTINUE
      INFOIN(3)=INFOIN(4)
      IF(.TRUE.)GOTO 11062
      GO TO 11061
11061 CALL TOKEN(INFOIN,CARDIN,   20HMEAN:2,DEVIATION:3,;)
      GOTO(11062,11070,11065,11065,11065,11065,11065,11080,11065),INTERP
11062 CONTINUE
11051 CONTINUE
      GO TO 10991
11130 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11151
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11161
11151 CONTINUE
      IF(FLOTNG .LE. 0.)GOTO 11181
      BUFFER(1)=FLOTNG
      GOTO 11191
11181 CONTINUE
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11191 CONTINUE
11171 CONTINUE
11161 CONTINUE
11141 CONTINUE
      GO TO 10991
11200 CONTINUE
      GO TO 11211
11220 CONTINUE
      ISCAL=INTEG
      GO TO 11212
11215 CONTINUE
      ISCAL=0
      INFOIN(3)=INFOIN(4)
      GO TO 11212
11211 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(11215,11215,11220,11220,11215,11215,11215,11215,11215),INTERP
11212 CONTINUE
      GO TO 10991
11010 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11000 GO TO ( 11020,11020,11020,11030,11030,11030,11030,11030,11130,1120
     *0,10995),KEYORD
11230 CONTINUE
      N1=INTEG
      GO TO 11241
11250 CONTINUE
      N2=INTEG
      GO TO 11242
11245 CONTINUE
      N2=0
      INFOIN(3)=INFOIN(4)
      GO TO 11242
11241 CALL TOKEN(INFOIN,CARDIN,    1H;)
      GOTO(11245,11245,11250,11250,11245,11245,11245,11245,11245),INTERP
11242 CONTINUE
      GO TO 10991
10991 CALL TOKEN(INFOIN,CARDIN,   66HX,Y,Z,LINEAR:1,LOGARITHMIC:2,MONTHS
     *:4,YEARS:5,NORMAL:6,BASE,USER,;)
      GOTO(10992,11000,11230,11230,10995,10995,10995,11010,10995),INTERP
10995 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10991
10992 CONTINUE
      IF (ISCAL.NE.NONLIN(IXY)) FLAGS(75)=.FALSE.
      FLAGS(IXY+61)=.TRUE.
      NONLIN(IXY)=ISCAL
      NXYZ1(IXY)=N1
      NXYZ2(IXY)=N2
      XYZBAS(IXY)=BUFFER(1)
      DO 11261 I=1,10
      SCLPRM(I,IXY)=BUFFER(I)
11261 CONTINUE
11262 CONTINUE
      GO TO 10951
10951 CALL TOKEN(INFOIN,CARDIN,   13HX:1,Y:2,Z:3,;)
      GO TO (10952,10957,10955,10955,10955,10955,10955,10956, 10955),INT
     *ERP
10956 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10957 GO TO ( 10980,10980,10980,10955),KEYORD
10955 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 10951
10952 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11281
      WRITE(DBGFIL,11290)NONLIN,NXYZ1,NXYZ2
11290 FORMAT (60X,    6HNONLIN,3I5/  60X,    6HNXYZ1 ,3I5/  60X,    6HNX
     *YZ2 ,3I5)
11281 CONTINUE
      GO TO 10012
11300 CONTINUE
      IF((FLAGS(1)))GOTO 11321
      WRITE(ERRFIL,11330)
11330 FORMAT(   24H0*** ERROR *** SIZE MAY   ,   37HNOT BE CHANGED AFTER
     * PLOT IS STARTED.)
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,4)
      GOTO 11341
11321 CONTINUE
      GO TO 11351
11380 CONTINUE
      I = INTEG
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11401
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11411
11401 CONTINUE
      REDUCE(I)=FLOTNG
11411 CONTINUE
11391 CONTINUE
      GO TO 11351
11370 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11360 GO TO ( 11380,11380,11355),KEYORD
11420 CONTINUE
      VALSAV=FLOTNG
      CALL TOKEN(INFOIN,CARDIN,    3HBY;)
      IF(INTERP .EQ. 2)GOTO 11441
      INFOIN(3)=INFOIN(4)
11441 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11461
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11471
11461 CONTINUE
      SCREEN(1)=VALSAV
      SCREEN(2)=FLOTNG
11471 CONTINUE
11451 CONTINUE
      GO TO 11351
11351 CALL TOKEN(INFOIN,CARDIN,   18HUNITS:2,REDUCE:1,;)
      GOTO(11352,11360,11420,11420,11355,11355,11355,11370,11355),INTERP
11355 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11351
11352 CONTINUE
      FLAGS(73) = .FALSE.
      FLAGS(74) = .FALSE.
      FLAGS(75) = .FALSE.
      FLAGS(98) = .FALSE.
11341 CONTINUE
11311 CONTINUE
      GO TO 10012
11480 CONTINUE
      NINCR=0
      NDIMNS(2)=2
      GO TO 11491
11520 CONTINUE
      NINCR=NINCR+1
      IVARBL(NINCR)=INTEG
      IF (INTEG.EQ.6) NDIMNS(2)=3
      GO TO 11491
11491 CALL TOKEN(INFOIN,CARDIN,   37HSYMBOL:1,X:2,DX:3,Y:4,DY:5,Z:6,DZ:7
     *,;)
      GO TO (11492,11497,11495,11495,11495,11495,11495,11496, 11495),INT
     *ERP
11496 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11497 GO TO ( 11520,11520,11520,11520,11520,11520,11520,11495),KEYORD
11495 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11491
11492 CONTINUE
      IDIMNS=NDIMNS(3)
      CALL T2SETS
      IF(.NOT.(FLAGS(29)))GOTO 11541
      WRITE(DBGFIL,11550)IVARBL,IVRPTR
11550 FORMAT (60X,    6HIVARBL,15I3/60X,    6HIVRPTR,15I3)
11541 CONTINUE
      GO TO 10012
11560 CONTINUE
      N1=0
      N2=0
      SYMSIZ = SMSZDF
      GO TO 11571
11600 CONTINUE
      N2=1
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11621
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11631
11621 CONTINUE
      SYMSIZ=FLOTNG
11631 CONTINUE
11611 CONTINUE
      GO TO 11571
11590 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11580 GO TO ( 11600,11575),KEYORD
11575 CONTINUE
      SYMBOL = TXSYM(STRING)
      IF(N1 .EQ. 0)GOTO 11651
      WRITE(ERRFIL,11660)
11660 FORMAT (   34H *** WARNING *** SYMBOL SET TWICE.  ,   11H LAST USE
     *D.)
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL, 1)
11651 CONTINUE
      N1=1
      GO TO 11571
11571 CALL TOKEN(INFOIN,CARDIN,    6HSIZE,;)
      GOTO(11572,11580,11575,11575,11575,11575,11575,11590,11575),INTERP
11572 CONTINUE
      IF (SYMSIZ.EQ.0.) SYMSIZ = 2.
      IF(N1 .NE. 0)GOTO 11681
      IF(N2 .NE. 0)GOTO 11701
      SYMBOL = NOSYMB
      GOTO 11711
11701 CONTINUE
      SMSZDF = SYMSIZ
11711 CONTINUE
11691 CONTINUE
11681 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 11731
      WRITE(DBGFIL,11740)SYMBOL,SYMSIZ
11740 FORMAT(60X,    4HSYM ,G10.3,    5H SIZE,F8.2)
11731 CONTINUE
      GO TO 10012
11750 CONTINUE
      ITXTUR=0
      GO TO 11761
11790 CONTINUE
      ITXTUR=INTEG
      GO TO 11761
11800 CONTINUE
      ITXTUR=IOR(ITXTUR,INTEG)
      GO TO 11761
11761 CALL TOKEN(INFOIN,CARDIN,   79HSOLID:4,DOTDASH:3,DOT-DASH:3,DAASHE
     *S:7,PATTERNED:6,DASHES:2,DO:1,DOT:1,DOTS:1,;)
      GO TO (11762,11767,11765,11765,11765,11765,11765,11766, 11765),INT
     *ERP
11766 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11767 GO TO ( 11790,11790,11790,11790,11790,11800,11800,11800,11800,1176
     *5),KEYORD
11765 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11761
11762 CONTINUE
      IF (ITXTUR.NE.0) LINTEX = ITXTUR
      IF(.NOT.(FLAGS(29)))GOTO 11821
      WRITE(DBGFIL,11830)LINTEX
11830 FORMAT(60X,    7HTEXTURE,I2)
11821 CONTINUE
      GO TO 10012
11840 CONTINUE
      NDIMNS(1)=3
      FLAGS(75)=.FALSE.
      GO TO 11851
11880 CONTINUE
      DO 11891 I=1,3
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11911
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11921
11911 CONTINUE
      EYEDIR(I)=FLOTNG
11921 CONTINUE
11901 CONTINUE
11891 CONTINUE
11892 CONTINUE
      FLAGS(55)=.TRUE.
      GO TO 11851
11930 CONTINUE
      DO 11941 I=1,3
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 11961
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 11971
11961 CONTINUE
      ORAXES(I)=FLOTNG
11971 CONTINUE
11951 CONTINUE
11941 CONTINUE
11942 CONTINUE
      GO TO 11851
11980 CONTINUE
      DO 11991 I=1,3
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12011
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12021
12011 CONTINUE
      VUECEN(I)=FLOTNG
12021 CONTINUE
12001 CONTINUE
11991 CONTINUE
11992 CONTINUE
      GO TO 11851
12030 CONTINUE
      DO 12041 I=1,3
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12061
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12071
12061 CONTINUE
      VERTCL(I)=FLOTNG
12071 CONTINUE
12051 CONTINUE
12041 CONTINUE
12042 CONTINUE
      GO TO 11851
12080 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12101
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12111
12101 CONTINUE
      TTHETA=FLOTNG
      IF (FLAGS(55))TPHI=60.
      FLAGS(55)=.FALSE.
12111 CONTINUE
12091 CONTINUE
      GO TO 11851
12120 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12141
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12151
12141 CONTINUE
      TPHI=FLOTNG
      IF (FLAGS(55))TTHETA=60.
      FLAGS(55)=.FALSE.
12151 CONTINUE
12131 CONTINUE
      GO TO 11851
12160 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12181
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12191
12181 CONTINUE
      EYESEP=FLOTNG
12191 CONTINUE
12171 CONTINUE
      GO TO 11851
12200 CONTINUE
      EYESEP=1.5*FLOAT(INTEG-1)
      GO TO 11851
12210 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12231
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12241
12231 CONTINUE
      SCRD=FLOTNG
12241 CONTINUE
12221 CONTINUE
      GO TO 11851
12250 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12271
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12281
12271 CONTINUE
      EYEDIS=FLOTNG
12281 CONTINUE
12261 CONTINUE
      GO TO 11851
12290 CONTINUE
      I=0
      DO 12301 J=1,3
      CALL TOKEN(INFOIN,CARDIN,    3HBY;)
      IF(INTERP .EQ. 2)GOTO 12321
      INFOIN(3)=INFOIN(4)
12321 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12341
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12351
12341 CONTINUE
      I=I+1
      BUFFER(I)=FLOTNG
12351 CONTINUE
12331 CONTINUE
12301 CONTINUE
12302 CONTINUE
      IF(I .NE. 3)GOTO 12371
      DO 12381 J=1,3
      WORLD(J)=BUFFER(J)
12381 CONTINUE
12382 CONTINUE
12371 CONTINUE
      GO TO 11851
12390 CONTINUE
      NDIMNS(1)=INTEG
      GO TO 11851
11851 CALL TOKEN(INFOIN,CARDIN,  108HDIRECTION,ORIGIN,CENTER,VERTICAL,TH
     *ETA,PHI,SEPARATION,LEFT,MIDDLE:1,RIGHT:2,SCRD,DISTANCE,WORLD,OFF:2
     *,ON:3,;)
      GO TO (11852,11857,11855,11855,11855,11855,11855,11856, 11855),INT
     *ERP
11856 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
11857 GO TO ( 11880,11930,11980,12030,12080,12120,12160,12200,12200,1220
     *0,12210,12250,12290,12390,12390,11855),KEYORD
11855 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 11851
11852 CONTINUE
      GO TO 10012
12400 CONTINUE
      IFLAG=1
      FLAGS(3) = .FALSE.
      GO TO 12411
12440 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12461
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12471
12461 CONTINUE
      DO 12481 I=1,3
      TIKSIZ(I)=FLOTNG
12481 CONTINUE
12482 CONTINUE
      FLAGS(78)=.TRUE.
12471 CONTINUE
12451 CONTINUE
      GO TO 12411
12490 CONTINUE
      LLLL=INTEG.EQ.0
      IXXXXX=IFLAG
12500 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 8))GOTO 12601
      WRITE(6,12610)IXXXXX
12610 FORMAT(    8H IFLAG (,I5,   31H) OUT OF RANGE FOR CASE (1 TO8))
      CALL ERRTRA
      STOP 7777
12601 CONTINUE
      GO TO (12510,12520,12530,12540,12550,12560,12570,12580),IXXXXX
12510 CONTINUE
      DO 12621 I=32,35
      FLAGS(I)=LLLL
12621 CONTINUE
12622 CONTINUE
      DO 12631 I=44,46
      FLAGS(I)=LLLL
12631 CONTINUE
12632 CONTINUE
      GOTO 12502
12550 CONTINUE
      FLAGS(32)=LLLL
      FLAGS(45)=.TRUE.
      GOTO 12502
12560 CONTINUE
      FLAGS(33)=LLLL
      FLAGS(44)=.TRUE.
      GOTO 12502
12570 CONTINUE
      FLAGS(34)=LLLL
      FLAGS(45)=.TRUE.
      GOTO 12502
12580 CONTINUE
      FLAGS(35)=LLLL
      FLAGS(44)=.TRUE.
      GOTO 12502
12520 CONTINUE
      FLAGS(33)=LLLL
      FLAGS(35)=LLLL
      FLAGS(44)=LLLL
      GOTO 12502
12530 CONTINUE
      FLAGS(32)=LLLL
      FLAGS(34)=LLLL
      FLAGS(45)=LLLL
      GOTO 12502
12540 CONTINUE
      FLAGS(46)=LLLL
      GOTO 12502
12502 CONTINUE
      GO TO 12411
12640 CONTINUE
      IFLAG=INTEG
      GO TO 12411
12411 CALL TOKEN(INFOIN,CARDIN,   70HSIZE,OFF:1,ON,ALL:1,X:2,Y:3,Z:4,LEF
     *T:5,BOTTOM:6,RIGHT:7,RITE:7,TOP:8,;)
      GO TO (12412,12417,12415,12415,12415,12415,12415,12416, 12415),INT
     *ERP
12416 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
12417 GO TO ( 12440,12490,12490,12640,12640,12640,12640,12640,12640,1264
     *0,12640,12640,12415),KEYORD
12415 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GO TO 12411
12412 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 12661
      WRITE(DBGFIL,12670)(FLAGS(I),I=32,35),TIKSIZ
12670 FORMAT (60X,    6HTICKS ,4L1,3G10.3)
12661 CONTINUE
      GO TO 10012
12680 CONTINUE
      CALL TXTIME
      GO TO 10012
12690 CONTINUE
      GO TO 12701
12730 CONTINUE
      CALL TOKEN(INFOIN,CARDIN,    1H;)
      IF(INTERP .EQ. 3 .OR. INTERP .EQ. 4)GOTO 12751
      CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
      GOTO 12761
12751 CONTINUE
      TSIZDF=FLOTNG
      FLAGS(76)=.TRUE.
12761 CONTINUE
12741 CONTINUE
      GO TO 12702
12701 CALL TOKEN(INFOIN,CARDIN,    6HSIZE,;)
      GO TO (12705,12707,12705,12705,12705,12705,12705,12706, 12705),INT
     *ERP
12706 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
12707 GO TO ( 12730,12705),KEYORD
12705 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
12702 CONTINUE
      GO TO 10012
10011 CALL TOKEN(INFOIN,CARDIN,  119HMODE,ORDER,OUTLINE,PATTERN,PEN,PE,P
     *EN1:1,PEN2:2,PEN3:3,POLAR,SCALE,SIZE,STORAGE,SYMBOL,TEXTURE,THREE,
     *TICKS,TIME,TITLE,;)
      GO TO (10015,10017,10015,10015,10015,10015,10015,10016, 10015),INT
     *ERP
10016 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10017 GO TO ( 10040,10190,10420,10520,10550,10550,10550,10550,10550,1064
     *0,10940,11300,11480,11560,11750,11840,12400,12680,12690,10015),KEY
     *ORD
10015 CALL T2ERR(INFOIN,INFOIN,CARDIN,ERRFIL,5)
10012 CONTINUE
      RETURN
      END
C        MORTRAN 2.79 (BRACKETED KEYWORD MACROS OF 02/11/81)
cnext ****************************************
      SUBROUTINE T2XFRM(NIN,NUM,VECIN,ISCAL,XFRM,NOUT,VECOUT)
      INTEGER NIN,NOUT,ISCAL(4)
      REAL VECIN(NIN,NUM),VECOUT(NOUT,NUM),XFRM(12)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL VEC(3),VEC2(3),LEAPYR,REGYR,DAYS(12)
      DATA DAYS/0.31,0.31,0.28,0.31,0.30,0.31,0.30,0.31,0.31  ,0.30,0.31
     *,0.30/
      DATA LEAPYR/2.73224/,REGYR/2.73973/
      DO 10011 NPT=1,NUM
cccccccccccccc 21-Aug-91 Modified by Y.Teramoto
cccc      DO 10021 I=1,3
      DO 10021 I=1, NIN
ccccccccccccccccccccc end of modification
      VEC(I)=VECIN(I,NPT)
10021 CONTINUE
10022 CONTINUE
      DO 10031 I=1,NIN
      IF(ISCAL(I) .GE. 0)GOTO 10051
      IXXXXX=-ISCAL(I)
10060 IF((IXXXXX .GT. 0) .AND. (IXXXXX .LE. 6))GOTO 10141
      WRITE(6,10150)IXXXXX
10150 FORMAT(   12H -ISCAL(I) (,I5,   31H) OUT OF RANGE FOR CASE (1 TO6)
     *)
      CALL ERRTRA
      STOP 7777
10141 CONTINUE
      GO TO (10070,10080,10090,10100,10110,10120),IXXXXX
10070 CONTINUE
      GOTO 10062
10080 CONTINUE
      IF(VEC(I) .LE. 0.)GOTO 10171
      VEC(I)=ALOG10(VEC(I))
      GOTO 10181
10171 CONTINUE
      VEC(I) = -10000.
10181 CONTINUE
10161 CONTINUE
      GOTO 10062
10090 CONTINUE
      IF(VEC(I) .GE. 0.)GOTO 10201
      VEC(I)=ALOG10(-VEC(I))
      GOTO 10211
10201 CONTINUE
      VEC(I) = 10000.
10211 CONTINUE
10191 CONTINUE
      GOTO 10062
10110 CONTINUE
      YEAR = AINT(VEC(I))
      PART = VEC(I)-YEAR
      IF(AMOD(VEC(I)+.001,4.) .GE. 1.)GOTO 10231
      PART = PART*LEAPYR
      GOTO 10241
10231 CONTINUE
      PART = PART*REGYR
10241 CONTINUE
10221 CONTINUE
      VEC(I) = YEAR + AMIN1(PART,1.)
      GOTO 10062
10100 CONTINUE
      MONTH = INT(VEC(I)+0.35)
      XMONTH = FLOAT(MONTH)
      LEAP = MOD(MONTH,48)
      MONTH = MOD(MONTH,12)
      IF(LEAP .NE. 38)GOTO 10261
      D = 0.29
      GOTO 10271
10261 CONTINUE
      D =DAYS(MONTH+1)
10271 CONTINUE
10251 CONTINUE
      VEC(I) = XMONTH + (VEC(I)-0.005-XMONTH)/D
      GOTO 10062
10120 CONTINUE
      VEC(I)=T2FRQF((VEC(I)-SCLPRM(2,I))/SCLPRM(3,I))
      GOTO 10062
10062 CONTINUE
      GOTO 10281
10051 CONTINUE
      VEC(I)=TDFNCT(VECIN(I,NPT),ISCAL(I))
10281 CONTINUE
10041 CONTINUE
10031 CONTINUE
10032 CONTINUE
      IF(ISCAL(4) .EQ. 0 .OR. .NOT.(FLAGS(27)))GOTO 10301
      ANGLE=VEC(2)*RADANG-RADAMN
      RAD = VEC(1)-RADRMN
      VEC(2)=RAD*SIN(ANGLE)+RADY0
      VEC(1)=RAD*COS(ANGLE)+RADX0
10301 CONTINUE
      M=0
      DO 10311 I=1,NIN
      M=M+1
      SUM=XFRM(M)
      DO 10321 J=1,NIN
      M=M+1
      SUM=SUM+XFRM(M)*VEC(J)
10321 CONTINUE
10322 CONTINUE
      VEC2(I)=SUM
10311 CONTINUE
10312 CONTINUE
      IF(NIN .EQ. NOUT)GOTO 10341
      VECOUT(1,NPT)=VEC2(1)/VEC2(3)
      VECOUT(2,NPT)=VEC2(2)/VEC2(3)
      GOTO 10351
10341 CONTINUE
      DO 10361 I=1,NOUT
      VECOUT(I,NPT)=VEC2(I)
10361 CONTINUE
10362 CONTINUE
10351 CONTINUE
10331 CONTINUE
10011 CONTINUE
10012 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2CROS(A,B,C)
      REAL A(3),B(3),C(3)
      C(1)=A(2)*B(3)-A(3)*B(2)
      C(2)=A(3)*B(1)-A(1)*B(3)
      C(3)=A(1)*B(2)-A(2)*B(1)
      RETURN
      END
cnext ****************************************
      REAL FUNCTION T2FRQF(T)
      REAL A(6),SQT2
      DATA A/0.070523078,.042282012,.009270527 ,.000152014 ,  .000276567
     * ,.000043064 /
      DATA SQT2/ 1.414214/
      TT=ABS(T)/SQT2
      SUM=TT*(A(1)+TT*(A(2)+TT*(A(3)+TT*(A(4)+TT*(A(5)+TT*A(6))))))+1.0
      IF(SUM .LE. 10.0)GOTO 10381
      T2FRQF=0.0
      GOTO 10391
10381 CONTINUE
      T2FRQF = 0.5/SUM**16
10391 CONTINUE
10371 CONTINUE
      IF (T.GT.0.) T2FRQF = 1.0-T2FRQF
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2MPRJ(XYLIM,XFRM)
      REAL XYLIM(2,2),XFRM(12)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL VECT(3),UDIR(3),VDIR(3),HDIR(3),XHAT(3)
      DATA XHAT/1.,0.,0./
      IF(.NOT.(FLAGS(56)))GOTO 10411
      IF (EYEDIS.LE.0.) EYEDIS=35.*REDUCE(2)
      IF (SCRD.LE.0.) SCRD=18.*REDUCE(2)
      IF((ABS(VERTCL(1))+ABS(VERTCL(2))+ABS(VERTCL(3))) .NE. 0.0)GOTO 10
     *431
      UDIR(1)=0.
      UDIR(2)=0.
      UDIR(3)=1.
      GOTO 10441
10431 CONTINUE
      DO 10451 I=1,3
      UDIR(I)=VERTCL(I)
10451 CONTINUE
10452 CONTINUE
10441 CONTINUE
10421 CONTINUE
      IF(((FLAGS(55))) .AND. ((ABS(EYEDIR(1))+ABS(EYEDIR(2))+ABS(EYEDIR(
     *3))) .NE. 0.))GOTO 10471
      CALL T2CROS(UDIR,XHAT,VECT)
      IF (ABS(VECT(1))+ABS(VECT(2))+ABS(VECT(3)) .EQ.0.0) VECT(3)=1.
      CALL T2CROS(VECT,UDIR,HDIR)
      TVR1=0.0174532925*TTHETA
      TVR2=0.0174532925*TPHI
      SINTH=SIN(TVR1)
      COSTH=COS(TVR1)
      SINPH=-SIN(TVR2)
      COSPH=-COS(TVR2)
      DO 10481 I=1,3
      VDIR(I) = COSPH*UDIR(I) +  SINPH*(SINTH*HDIR(I) + COSTH*VECT(I))
10481 CONTINUE
10482 CONTINUE
      GOTO 10491
10471 CONTINUE
      DO 10501 I=1,3
      VDIR(I)=-EYEDIR(I)
10501 CONTINUE
10502 CONTINUE
10491 CONTINUE
10461 CONTINUE
      DO 10511 I=1,3
      IF (ABS(VUECEN(I)).GT.1000.) VUECEN(I)=0.5*WORLD(I)
10511 CONTINUE
10512 CONTINUE
      IF(.NOT.(FLAGS(29)))GOTO 10531
      WRITE(DBGFIL,10540)UDIR,VDIR,VUECEN,SCRD
10540 FORMAT (60X,    7HVERTCL ,3G10.3,/60X,    7HVUEDIR ,3G10.3,  /60X,
     *    7HVUECEN ,3G10.3,/60X,    7HSCRD   , G10.3)
10531 CONTINUE
      CALL T2CROS(UDIR,VDIR,VECT)
      IF(ABS(VECT(1))+ABS(VECT(2))+ ABS(VECT(3)) .NE. 0.)GOTO 10561
      IF(ABS(VDIR(1)) + ABS(VDIR(2)) .EQ. 0.0)GOTO 10581
      VECT(3)=1.0
      GOTO 10591
10581 CONTINUE
      VECT(2)=1.0
10591 CONTINUE
10571 CONTINUE
10561 CONTINUE
      CALL T2CROS(VDIR,VECT,UDIR)
      TVR1=SQRT(VDIR(1)**2+VDIR(2)**2+VDIR(3)**2)
      TVR2=SQRT(UDIR(1)**2+UDIR(2)**2+UDIR(3)**2)
      DO 10601 I=1,3
      VDIR(I)=VDIR(I)/TVR1
      UDIR(I)=UDIR(I)/TVR2
      VUEDIR(I)=VDIR(I)
10601 CONTINUE
10602 CONTINUE
      CALL T2CROS(VDIR,UDIR,HDIR)
      IF(.NOT.(FLAGS(29)))GOTO 10621
      WRITE(DBGFIL,10630)UDIR,VDIR,HDIR
10630 FORMAT (60X,    5HUDIR ,3G10.3/  60X,    5HVDIR ,3G10.3/  60X,
     *5HHDIR ,3G10.3)
10621 CONTINUE
      TVR1=(WINDOW(3)-WINDOW(1))/(XYLIM(1,2)-XYLIM(1,1))
      TVR2=(WINDOW(4)-WINDOW(2))/(XYLIM(2,2)-XYLIM(2,1))
      TVR3=(TVR1*0.5*(XYLIM(1,2)+XYLIM(1,1))+EYESEP)/  (TVR1*SCRD)
      TVR4=(0.5*(XYLIM(2,2)+XYLIM(2,1)))/SCRD
      DO 10641 I=1,3
      XFRM(I+1)=HDIR(I)/TVR1+VDIR(I)*TVR3
      XFRM(I+5)=UDIR(I)/TVR2+VDIR(I)*TVR4
      XFRM(I+9)=VDIR(I)/SCRD
      EYEPNT(I)=VUECEN(I)-VDIR(I)*EYEDIS+EYESEP*HDIR(I)
10641 CONTINUE
10642 CONTINUE
      DO 10651 I=1,9,4
      XFRM(I)=-XFRM(I+1)*EYEPNT(1)-XFRM(I+2)*EYEPNT(2)  -XFRM(I+3)*EYEPN
     *T(3)
10651 CONTINUE
10652 CONTINUE
      GOTO 10661
10411 CONTINUE
      DO 10671 I=1,12
      XFRM(I)=0.
10671 CONTINUE
10672 CONTINUE
      XFRM(2)=(XYLIM(1,2)-XYLIM(1,1))/WORLD(1)
      XFRM(1)=XYLIM(1,1)
      XFRM(7)=(XYLIM(2,2)-XYLIM(2,1))/WORLD(2)
      XFRM(5)=XYLIM(2,1)
      XFRM(9)=1.
      VUEDIR(1)=0.
      VUEDIR(2)=0.
      VUEDIR(3)=-1.
      EYEPNT(1)=0.
      EYEPNT(2)=0.
      EYEPNT(3)=18.
10661 CONTINUE
10401 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2AXST(XYZWRL,IXYZ,IAX2,IAX3)
      INTEGER IXYZ,IAX2,IAX3
      REAL XYZWRL(3)
      COMMON /T2FLGC/ FLAGS(100)
      LOGICAL FLAGS
ccccccccccccccccccccccccc 25-Jan-96 Modified by Y.Teramoto
	include	't2com.inc'
ccccccccccccccccccccccccccccc
      COMMON /TOKENC/ INTERP,INTEG,FLOTNG,KEYORD,NSTRNG,MAXSTR, STRING(1
     *60)
      REAL FLOTNG
      INTEGER MAXSTR,INTERP,KEYORD,NSTRNG
      INTEGER*4 INTEG
cmt4       BYTE STRING
      character*1 STRING
      REAL XYZ(3,4),XY(2,4)
      IAX2=IXYZ-1
      IF (IAX2.EQ.0) IAX2=3
      IAX3=6-IXYZ-IAX2
      DO 10681 I=1,3
      DO 10691 J=1,4
      XYZ(I,J)=XYZWRL(I)
10691 CONTINUE
10692 CONTINUE
10681 CONTINUE
10682 CONTINUE
      XYZ(IXYZ,1)=XYZ(IXYZ,1)+0.1
      XYZ(IAX2,2)=XYZ(IAX2,2)+0.1
      XYZ(IAX3,3)=XYZ(IAX3,3)+0.1
      CALL T2XFRM(3,4,XYZ,LINEAR,XFRM23,2,XY)
      IF(ABS(XY(1,2)-XY(1,4))+ABS(XY(2,2)-XY(2,4)) .GE. ABS(XY(1,3)-XY(1
     *,4))+ABS(XY(2,3)-XY(2,4)))GOTO 10711
      I=IAX2
      IAX2=IAX3
      IAX3=I
10711 CONTINUE
      IF(XY(1,1) .NE. XY(1,4) .OR. XY(2,1) .NE. XY(2,4))GOTO 10731
      IAX2=-IAX2
10731 CONTINUE
      RETURN
      END
cnext ****************************************
      SUBROUTINE T2XMUL(X1,X2,N,X3)
      INTEGER N
      REAL X1(12),X2(12),X3(12)
      J3=0
      NP1=N+1
      MAX=N*NP1
      DO 10741 I=1,MAX,NP1
      DO 10751 K1=1,4
      SUM=0.
      J1=I
      J2=K1
      DO 10761 J=1,3
      J1=J1+1
      SUM=SUM+X1(J1)*X2(J2)
      J2=J2+NP1
10761 CONTINUE
10762 CONTINUE
      J3=J3+1
      X3(J3)=SUM
10751 CONTINUE
10752 CONTINUE
10741 CONTINUE
10742 CONTINUE
      DO 10771 K=1,MAX,NP1
      X3(K)=X3(K)+X1(K)
10771 CONTINUE
10772 CONTINUE
      RETURN
      END
cnext ****************************************
	SUBROUTINE	TXPACK(IN,OUT)
cmt4 	BYTE	IN(8)
	character*1 IN(8)
	Character*8 OUT
	DO 10 I=1,8
cmt4 10	OUT(I:I) = CHAR(IN(I))
10	OUT(I:I) = IN(I)
	RETURN
	END
